@@ -12,6 +12,7 @@
 #include "mozilla/layers/ZoomConstraints.h"
 #include "mozilla/Preferences.h"
 #include "mozilla/PresShell.h"
+#include "mozilla/ScrollContainerFrame.h"
 #include "mozilla/StaticPrefs_apz.h"
 #include "mozilla/dom/Document.h"
 #include "mozilla/dom/Event.h"
@@ -32,6 +33,7 @@ NS_IMPL_ISUPPORTS(ZoomConstraintsClient, nsIDOMEventListener, nsIObserver)
 #define DOM_META_CHANGED u"DOMMetaChanged"_ns
 #define FULLSCREEN_CHANGED u"fullscreenchange"_ns
 #define BEFORE_FIRST_PAINT "before-first-paint"_ns
+#define COMPOSITOR_REINITIALIZED "compositor-reinitialized"_ns
 #define NS_PREF_CHANGED "nsPref:changed"_ns
 
 using namespace mozilla;
@@ -51,7 +53,14 @@ static nsIWidget* GetWidget(PresShell* aPresShell) {
     return nullptr;
   }
   if (nsIFrame* rootFrame = aPresShell->GetRootFrame()) {
-#if defined(MOZ_WIDGET_ANDROID) || defined(MOZ_WIDGET_UIKIT)
+#if defined(MOZ_WIDGET_ANDROID)
+    // On Android in cases of about:XX pages loaded in the browser parent
+    // process we need to return the nearest widget since it's the widget owning
+    // an IAPZCTreeManager to communicate with the APZCTreeManager for the
+    // browser.
+    // In bug 1648427 we will apply this code to desktops as well to make
+    // about pages zoomable on desktops, but it will be involving more works,
+    // see https://bugzilla.mozilla.org/show_bug.cgi?id=1648427#c7 .
     return rootFrame->GetNearestWidget();
 #else
     if (nsView* view = rootFrame->GetView()) {
@@ -80,6 +89,7 @@ void ZoomConstraintsClient::Destroy() {
       mozilla::services::GetObserverService();
   if (observerService) {
     observerService->RemoveObserver(this, BEFORE_FIRST_PAINT.Data());
+    observerService->RemoveObserver(this, COMPOSITOR_REINITIALIZED.Data());
   }
 
   Preferences::RemoveObserver(this, "browser.ui.zoom.force-user-scalable");
@@ -119,6 +129,7 @@ void ZoomConstraintsClient::Init(PresShell* aPresShell, Document* aDocument) {
       mozilla::services::GetObserverService();
   if (observerService) {
     observerService->AddObserver(this, BEFORE_FIRST_PAINT.Data(), false);
+    observerService->AddObserver(this, COMPOSITOR_REINITIALIZED.Data(), false);
   }
 
   Preferences::AddStrongObserver(this, "browser.ui.zoom.force-user-scalable");
@@ -150,6 +161,9 @@ ZoomConstraintsClient::Observe(nsISupports* aSubject, const char* aTopic,
       BEFORE_FIRST_PAINT.EqualsASCII(aTopic)) {
     ZCC_LOG("Got a before-first-paint event in %p\n", this);
     RefreshZoomConstraints();
+  } else if (COMPOSITOR_REINITIALIZED.EqualsASCII(aTopic)) {
+    ZCC_LOG("Got a compositor-reinitialized notification in %p\n", this);
+    RefreshZoomConstraints();
   } else if (NS_PREF_CHANGED.EqualsASCII(aTopic)) {
     ZCC_LOG("Got a pref-change event in %p\n", this);
     // We need to run this later because all the pref change listeners need
@@ -160,7 +174,7 @@ ZoomConstraintsClient::Observe(nsISupports* aSubject, const char* aTopic,
     RefPtr<nsRunnableMethod<ZoomConstraintsClient>> event =
         NewRunnableMethod("ZoomConstraintsClient::RefreshZoomConstraints", this,
                           &ZoomConstraintsClient::RefreshZoomConstraints);
-    mDocument->Dispatch(TaskCategory::Other, event.forget());
+    mDocument->Dispatch(event.forget());
   }
   return NS_OK;
 }
@@ -196,6 +210,11 @@ void ZoomConstraintsClient::RefreshZoomConstraints() {
     return;
   }
 
+  // Ignore documents which has been removed from the doc shell.
+  if (!mDocument->IsActive()) {
+    return;
+  }
+
   uint32_t presShellId = 0;
   ScrollableLayerGuid::ViewID viewId = ScrollableLayerGuid::NULL_SCROLL_ID;
   bool scrollIdentifiersValid =
@@ -206,8 +225,8 @@ void ZoomConstraintsClient::RefreshZoomConstraints() {
   }
 
   LayoutDeviceIntSize screenSize;
-  if (!nsLayoutUtils::GetContentViewerSize(mPresShell->GetPresContext(),
-                                           screenSize)) {
+  if (!nsLayoutUtils::GetDocumentViewerSize(mPresShell->GetPresContext(),
+                                            screenSize)) {
     return;
   }
 
@@ -256,8 +275,8 @@ void ZoomConstraintsClient::RefreshZoomConstraints() {
   // We only ever create a ZoomConstraintsClient for an RCD, so the RSF of
   // the presShell must be the RCD-RSF (if it exists).
   MOZ_ASSERT(mPresShell->GetPresContext()->IsRootContentDocumentCrossProcess());
-  if (nsIScrollableFrame* rcdrsf =
-          mPresShell->GetRootScrollFrameAsScrollable()) {
+  if (ScrollContainerFrame* rcdrsf =
+          mPresShell->GetRootScrollContainerFrame()) {
     ZCC_LOG("Notifying RCD-RSF that it is zoomable: %d\n",
             mZoomConstraints.mAllowZoom);
     rcdrsf->SetZoomableByAPZ(mZoomConstraints.mAllowZoom);
