@@ -14,12 +14,9 @@
 #include "nsLayoutUtils.h"
 #include "nsPresContext.h"
 #include "nsGkAtoms.h"
-#include "nsNameSpaceManager.h"
 #include "nsNodeInfoManager.h"
 #include "nsContentCreatorFunctions.h"
 #include "nsFontMetrics.h"
-#include "nsCSSPseudoElements.h"
-#include "nsStyleConsts.h"
 #include <algorithm>
 
 using namespace mozilla;
@@ -52,7 +49,12 @@ nsresult nsProgressFrame::CreateAnonymousContent(
   mBarDiv = doc->CreateHTMLElement(nsGkAtoms::div);
 
   // Associate ::-moz-progress-bar pseudo-element to the anonymous child.
-  mBarDiv->SetPseudoElementType(PseudoStyleType::mozProgressBar);
+  if (StaticPrefs::layout_css_modern_range_pseudos_enabled()) {
+    // TODO(emilio): Create also a slider-track pseudo-element.
+    mBarDiv->SetPseudoElementType(PseudoStyleType::sliderFill);
+  } else {
+    mBarDiv->SetPseudoElementType(PseudoStyleType::mozProgressBar);
+  }
 
   // XXX(Bug 1631371) Check if this should use a fallible operation as it
   // pretended earlier, or change the return type to void.
@@ -84,7 +86,6 @@ void nsProgressFrame::Reflow(nsPresContext* aPresContext,
                              nsReflowStatus& aStatus) {
   MarkInReflow();
   DO_GLOBAL_REFLOW_COUNT("nsProgressFrame");
-  DISPLAY_REFLOW(aPresContext, this, aReflowInput, aDesiredSize, aStatus);
   MOZ_ASSERT(aStatus.IsEmpty(), "Caller should pass a fresh reflow status!");
 
   NS_ASSERTION(mBarDiv, "Progress bar div must exist!");
@@ -97,11 +98,18 @@ void nsProgressFrame::Reflow(nsPresContext* aPresContext,
                "need to call RegUnregAccessKey only for the first.");
 
   const auto wm = aReflowInput.GetWritingMode();
-  aDesiredSize.SetSize(wm, aReflowInput.ComputedSizeWithBorderPadding(wm));
+  const auto contentBoxSize = aReflowInput.ComputedSizeWithBSizeFallback([&] {
+    nscoord em = OneEmInAppUnits();
+    return ResolvedOrientationIsVertical() == wm.IsVertical() ? em : 10 * em;
+  });
+  aDesiredSize.SetSize(
+      wm,
+      contentBoxSize + aReflowInput.ComputedLogicalBorderPadding(wm).Size(wm));
   aDesiredSize.SetOverflowAreasToDesiredBounds();
 
-  for (auto childFrame : PrincipalChildList()) {
-    ReflowChildFrame(childFrame, aPresContext, aReflowInput, aStatus);
+  for (nsIFrame* childFrame : PrincipalChildList()) {
+    ReflowChildFrame(childFrame, aPresContext, aReflowInput, contentBoxSize,
+                     aStatus);
     ConsiderChildOverflow(aDesiredSize.mOverflowAreas, childFrame);
   }
 
@@ -113,14 +121,19 @@ void nsProgressFrame::Reflow(nsPresContext* aPresContext,
 void nsProgressFrame::ReflowChildFrame(nsIFrame* aChild,
                                        nsPresContext* aPresContext,
                                        const ReflowInput& aReflowInput,
+                                       const LogicalSize& aParentContentBoxSize,
                                        nsReflowStatus& aStatus) {
   bool vertical = ResolvedOrientationIsVertical();
-  WritingMode wm = aChild->GetWritingMode();
-  LogicalSize availSize = aReflowInput.ComputedSize(wm);
+  const WritingMode wm = aChild->GetWritingMode();
+  const LogicalSize parentSizeInChildWM =
+      aParentContentBoxSize.ConvertTo(wm, aReflowInput.GetWritingMode());
+  const nsSize parentPhysicalSize = parentSizeInChildWM.GetPhysicalSize(wm);
+  LogicalSize availSize = parentSizeInChildWM;
   availSize.BSize(wm) = NS_UNCONSTRAINEDSIZE;
-  ReflowInput reflowInput(aPresContext, aReflowInput, aChild, availSize);
+  ReflowInput reflowInput(aPresContext, aReflowInput, aChild, availSize,
+                          Some(parentSizeInChildWM));
   nscoord size =
-      vertical ? aReflowInput.ComputedHeight() : aReflowInput.ComputedWidth();
+      vertical ? parentPhysicalSize.Height() : parentPhysicalSize.Width();
   nscoord xoffset = aReflowInput.ComputedPhysicalBorderPadding().left;
   nscoord yoffset = aReflowInput.ComputedPhysicalBorderPadding().top;
 
@@ -133,7 +146,7 @@ void nsProgressFrame::ReflowChildFrame(nsIFrame* aChild,
   }
 
   if (!vertical && wm.IsPhysicalRTL()) {
-    xoffset += aReflowInput.ComputedWidth() - size;
+    xoffset += parentPhysicalSize.Width() - size;
   }
 
   // The bar size is fixed in these cases:
@@ -146,8 +159,7 @@ void nsProgressFrame::ReflowChildFrame(nsIFrame* aChild,
   if (position != -1 || ShouldUseNativeStyle()) {
     if (vertical) {
       // We want the bar to begin at the bottom.
-      yoffset += aReflowInput.ComputedHeight() - size;
-
+      yoffset += parentPhysicalSize.Height() - size;
       size -= reflowInput.ComputedPhysicalMargin().TopBottom() +
               reflowInput.ComputedPhysicalBorderPadding().TopBottom();
       size = std::max(size, 0);
@@ -159,10 +171,12 @@ void nsProgressFrame::ReflowChildFrame(nsIFrame* aChild,
       reflowInput.SetComputedWidth(size);
     }
   } else if (vertical) {
-    // For vertical progress bars, we need to position the bar specificly when
+    // For vertical progress bars, we need to position the bar specifically when
     // the width isn't constrained (position == -1 and !ShouldUseNativeStyle())
-    // because aReflowInput.ComputedHeight() - size == 0.
-    yoffset += aReflowInput.ComputedHeight() - reflowInput.ComputedHeight();
+    // because parentPhysiscalSize.Height() - size == 0.
+    // FIXME(emilio): This assumes that the bar's height is constrained, which
+    // seems like a wrong assumption?
+    yoffset += parentPhysicalSize.Height() - reflowInput.ComputedHeight();
   }
 
   xoffset += reflowInput.ComputedPhysicalMargin().left;
@@ -193,43 +207,13 @@ nsresult nsProgressFrame::AttributeChanged(int32_t aNameSpaceID,
   return nsContainerFrame::AttributeChanged(aNameSpaceID, aAttribute, aModType);
 }
 
-LogicalSize nsProgressFrame::ComputeAutoSize(
-    gfxContext* aRenderingContext, WritingMode aWM, const LogicalSize& aCBSize,
-    nscoord aAvailableISize, const LogicalSize& aMargin,
-    const LogicalSize& aBorderPadding, const StyleSizeOverrides& aSizeOverrides,
-    ComputeSizeFlags aFlags) {
-  const WritingMode wm = GetWritingMode();
-  LogicalSize autoSize(wm);
-  autoSize.BSize(wm) = autoSize.ISize(wm) =
-      StyleFont()
-          ->mFont.size.ScaledBy(nsLayoutUtils::FontSizeInflationFor(this))
-          .ToAppUnits();  // 1em
-
-  if (ResolvedOrientationIsVertical() == wm.IsVertical()) {
-    autoSize.ISize(wm) *= 10;  // 10em
-  } else {
-    autoSize.BSize(wm) *= 10;  // 10em
-  }
-
-  return autoSize.ConvertTo(aWM, wm);
-}
-
-nscoord nsProgressFrame::GetMinISize(gfxContext* aRenderingContext) {
-  RefPtr<nsFontMetrics> fontMet =
-      nsLayoutUtils::GetFontMetricsForFrame(this, 1.0f);
-
-  nscoord minISize = fontMet->Font().size.ToAppUnits();  // 1em
-
+nscoord nsProgressFrame::IntrinsicISize(const IntrinsicSizeInput& aInput,
+                                        IntrinsicISizeType aType) {
+  nscoord iSize = OneEmInAppUnits();
   if (ResolvedOrientationIsVertical() == GetWritingMode().IsVertical()) {
-    // The orientation is inline
-    minISize *= 10;  // 10em
+    iSize *= 10;
   }
-
-  return minISize;
-}
-
-nscoord nsProgressFrame::GetPrefISize(gfxContext* aRenderingContext) {
-  return GetMinISize(aRenderingContext);
+  return iSize;
 }
 
 bool nsProgressFrame::ShouldUseNativeStyle() const {
