@@ -222,8 +222,7 @@ void nsAbsoluteContainingBlock::Reflow(nsContainerFrame* aDelegatingFrame,
                  "ShouldAvoidBreakInside should prevent this from happening");
       nsIFrame* nextFrame = kidFrame->GetNextInFlow();
       if (!kidStatus.IsFullyComplete() &&
-          aDelegatingFrame->IsFrameOfType(
-              nsIFrame::eCanContainOverflowContainers)) {
+          aDelegatingFrame->CanContainOverflowContainers()) {
         // Need a continuation
         if (!nextFrame) {
           nextFrame = aPresContext->PresShell()
@@ -283,11 +282,13 @@ void nsAbsoluteContainingBlock::Reflow(nsContainerFrame* aDelegatingFrame,
 static inline bool IsFixedPaddingSize(const LengthPercentage& aCoord) {
   return aCoord.ConvertsToLength();
 }
-static inline bool IsFixedMarginSize(const LengthPercentageOrAuto& aCoord) {
-  return aCoord.ConvertsToLength();
+static inline bool IsFixedMarginSize(const AnchorResolvedMargin& aCoord) {
+  return aCoord->ConvertsToLength();
 }
-static inline bool IsFixedOffset(const LengthPercentageOrAuto& aCoord) {
-  return aCoord.ConvertsToLength();
+static inline bool IsFixedOffset(const AnchorResolvedInset& aInset) {
+  // For anchor positioning functions, even if the computed value may be a
+  // fixed length, it depends on the absolute containing block's size.
+  return aInset->ConvertsToLength();
 }
 
 bool nsAbsoluteContainingBlock::FrameDependsOnContainer(nsIFrame* f,
@@ -306,6 +307,7 @@ bool nsAbsoluteContainingBlock::FrameDependsOnContainer(nsIFrame* f,
   const nsStylePadding* padding = f->StylePadding();
   const nsStyleMargin* margin = f->StyleMargin();
   WritingMode wm = f->GetWritingMode();
+  const auto positionProperty = f->StyleDisplay()->mPosition;
   if (wm.IsVertical() ? aCBHeightChanged : aCBWidthChanged) {
     // See if f's inline-size might have changed.
     // If margin-inline-start/end, padding-inline-start/end,
@@ -313,9 +315,12 @@ bool nsAbsoluteContainingBlock::FrameDependsOnContainer(nsIFrame* f,
     // then our frame isize does not depend on the parent isize.
     // Note that borders never depend on the parent isize.
     // XXX All of the enumerated values except -moz-available are ok too.
-    if (pos->ISizeDependsOnContainer(wm) ||
-        pos->MinISizeDependsOnContainer(wm) ||
-        pos->MaxISizeDependsOnContainer(wm) ||
+    if (nsStylePosition::ISizeDependsOnContainer(
+            pos->ISize(wm, positionProperty)) ||
+        nsStylePosition::MinISizeDependsOnContainer(
+            pos->MinISize(wm, positionProperty)) ||
+        nsStylePosition::MaxISizeDependsOnContainer(
+            pos->MaxISize(wm, positionProperty)) ||
         !IsFixedPaddingSize(padding->mPadding.GetIStart(wm)) ||
         !IsFixedPaddingSize(padding->mPadding.GetIEnd(wm))) {
       return true;
@@ -324,8 +329,10 @@ bool nsAbsoluteContainingBlock::FrameDependsOnContainer(nsIFrame* f,
     // See if f's position might have changed. If we're RTL then the
     // rules are slightly different. We'll assume percentage or auto
     // margins will always induce a dependency on the size
-    if (!IsFixedMarginSize(margin->mMargin.GetIStart(wm)) ||
-        !IsFixedMarginSize(margin->mMargin.GetIEnd(wm))) {
+    if (!IsFixedMarginSize(
+            margin->GetMargin(LogicalSide::IStart, wm, positionProperty)) ||
+        !IsFixedMarginSize(
+            margin->GetMargin(LogicalSide::IEnd, wm, positionProperty))) {
       return true;
     }
   }
@@ -339,19 +346,28 @@ bool nsAbsoluteContainingBlock::FrameDependsOnContainer(nsIFrame* f,
     //
     // FIXME(emilio): Should the BSize(wm).IsAuto() check also for the extremum
     // lengths?
-    if ((pos->BSizeDependsOnContainer(wm) &&
-         !(pos->BSize(wm).IsAuto() && pos->mOffset.GetBEnd(wm).IsAuto() &&
-           !pos->mOffset.GetBStart(wm).IsAuto())) ||
-        pos->MinBSizeDependsOnContainer(wm) ||
-        pos->MaxBSizeDependsOnContainer(wm) ||
+    const auto bSize = pos->BSize(wm, positionProperty);
+    if ((nsStylePosition::BSizeDependsOnContainer(bSize) &&
+         !(bSize->IsAuto() &&
+           pos->GetAnchorResolvedInset(LogicalSide::BEnd, wm, positionProperty)
+               ->IsAuto() &&
+           !pos->GetAnchorResolvedInset(LogicalSide::BStart, wm,
+                                        positionProperty)
+                ->IsAuto())) ||
+        nsStylePosition::MinBSizeDependsOnContainer(
+            pos->MinBSize(wm, positionProperty)) ||
+        nsStylePosition::MaxBSizeDependsOnContainer(
+            pos->MaxBSize(wm, positionProperty)) ||
         !IsFixedPaddingSize(padding->mPadding.GetBStart(wm)) ||
         !IsFixedPaddingSize(padding->mPadding.GetBEnd(wm))) {
       return true;
     }
 
     // See if f's position might have changed.
-    if (!IsFixedMarginSize(margin->mMargin.GetBStart(wm)) ||
-        !IsFixedMarginSize(margin->mMargin.GetBEnd(wm))) {
+    if (!IsFixedMarginSize(
+            margin->GetMargin(LogicalSide::BStart, wm, positionProperty)) ||
+        !IsFixedMarginSize(
+            margin->GetMargin(LogicalSide::BEnd, wm, positionProperty))) {
       return true;
     }
   }
@@ -363,7 +379,8 @@ bool nsAbsoluteContainingBlock::FrameDependsOnContainer(nsIFrame* f,
   // sides (left and top) that we use to store coordinates, these tests
   // are easier to do using physical coordinates rather than logical.
   if (aCBWidthChanged) {
-    if (!IsFixedOffset(pos->mOffset.Get(eSideLeft))) {
+    if (!IsFixedOffset(
+            pos->GetAnchorResolvedInset(eSideLeft, positionProperty))) {
       return true;
     }
     // Note that even if 'left' is a length, our position can still
@@ -373,19 +390,20 @@ bool nsAbsoluteContainingBlock::FrameDependsOnContainer(nsIFrame* f,
     // and be positioned relative to the containing block right edge.
     // 'left' length and 'right' auto is the only combination we can be
     // sure of.
-    if ((wm.GetInlineDir() == WritingMode::eInlineRTL ||
-         wm.GetBlockDir() == WritingMode::eBlockRL) &&
-        !pos->mOffset.Get(eSideRight).IsAuto()) {
+    if ((wm.GetInlineDir() == WritingMode::InlineDir::RTL ||
+         wm.GetBlockDir() == WritingMode::BlockDir::RL) &&
+        !pos->GetAnchorResolvedInset(eSideRight, positionProperty)->IsAuto()) {
       return true;
     }
   }
   if (aCBHeightChanged) {
-    if (!IsFixedOffset(pos->mOffset.Get(eSideTop))) {
+    if (!IsFixedOffset(
+            pos->GetAnchorResolvedInset(eSideTop, positionProperty))) {
       return true;
     }
     // See comment above for width changes.
-    if (wm.GetInlineDir() == WritingMode::eInlineBTT &&
-        !pos->mOffset.Get(eSideBottom).IsAuto()) {
+    if (wm.GetInlineDir() == WritingMode::InlineDir::BTT &&
+        !pos->GetAnchorResolvedInset(eSideBottom, positionProperty)->IsAuto()) {
       return true;
     }
   }
@@ -423,12 +441,31 @@ static nsContainerFrame* GetPlaceholderContainer(nsIFrame* aPositionedFrame) {
   return placeholder ? placeholder->GetParent() : nullptr;
 }
 
+struct NonAutoAlignParams {
+  nscoord mCurrentStartInset;
+  nscoord mCurrentEndInset;
+
+  NonAutoAlignParams(nscoord aStartInset, nscoord aEndInset)
+      : mCurrentStartInset(aStartInset), mCurrentEndInset(aEndInset) {}
+};
+
 /**
  * This function returns the offset of an abs/fixed-pos child's static
  * position, with respect to the "start" corner of its alignment container,
  * according to CSS Box Alignment.  This function only operates in a single
  * axis at a time -- callers can choose which axis via the |aAbsPosCBAxis|
- * parameter.
+ * parameter. This is called under two scenarios:
+ * 1. We're statically positioning this absolutely positioned box, meaning
+ *    that the offsets are auto and will change depending on the alignment
+ *    of the box.
+ * 2. The offsets are non-auto, but the element may not fill the inset-reduced
+ *    containing block, so its margin box needs to be aligned in that axis.
+ *    This is the step 4 of [1]. Should also be noted that, unlike static
+ *    positioning, where we may confine the alignment area for flex/grid
+ *    parent containers, we explicitly align to the inset-reduced absolute
+ *    container size.
+ *
+ * [1]: https://drafts.csswg.org/css-position-3/#abspos-layout
  *
  * @param aKidReflowInput The ReflowInput for the to-be-aligned abspos child.
  * @param aKidSizeInAbsPosCBWM The child frame's size (after it's been given
@@ -442,13 +479,14 @@ static nsContainerFrame* GetPlaceholderContainer(nsIFrame* aPositionedFrame) {
  * @param aAbsPosCBWM The child frame's containing block's WritingMode.
  * @param aAbsPosCBAxis The axis (of the containing block) that we should
  *                      be doing this computation for.
+ * @param aNonAutoAlignParams Parameters, if specified, indicating that we're
+ *                            handling scenario 2.
  */
-static nscoord OffsetToAlignedStaticPos(const ReflowInput& aKidReflowInput,
-                                        const LogicalSize& aKidSizeInAbsPosCBWM,
-                                        const LogicalSize& aAbsPosCBSize,
-                                        nsContainerFrame* aPlaceholderContainer,
-                                        WritingMode aAbsPosCBWM,
-                                        LogicalAxis aAbsPosCBAxis) {
+static nscoord OffsetToAlignedStaticPos(
+    const ReflowInput& aKidReflowInput, const LogicalSize& aKidSizeInAbsPosCBWM,
+    const LogicalSize& aAbsPosCBSize,
+    const nsContainerFrame* aPlaceholderContainer, WritingMode aAbsPosCBWM,
+    LogicalAxis aAbsPosCBAxis, Maybe<NonAutoAlignParams> aNonAutoAlignParams) {
   if (!aPlaceholderContainer) {
     // (The placeholder container should be the thing that kicks this whole
     // process off, by setting PLACEHOLDER_STATICPOS_NEEDS_CSSALIGN.  So it
@@ -475,70 +513,91 @@ static nscoord OffsetToAlignedStaticPos(const ReflowInput& aKidReflowInput,
       (pcWM.IsOrthogonalTo(aAbsPosCBWM) ? GetOrthogonalAxis(aAbsPosCBAxis)
                                         : aAbsPosCBAxis);
 
-  const bool placeholderContainerIsContainingBlock =
-      aPlaceholderContainer == aKidReflowInput.mCBReflowInput->mFrame;
+  const LogicalSize alignAreaSize = [&]() {
+    if (!aNonAutoAlignParams) {
+      const bool placeholderContainerIsContainingBlock =
+          aPlaceholderContainer == aKidReflowInput.mCBReflowInput->mFrame;
 
-  LayoutFrameType parentType = aPlaceholderContainer->Type();
-  LogicalSize alignAreaSize(pcWM);
-  if (parentType == LayoutFrameType::FlexContainer) {
-    // We store the frame rect in FinishAndStoreOverflow, which runs _after_
-    // reflowing the absolute frames, so handle the special case of the frame
-    // being the actual containing block here, by getting the size from
-    // aAbsPosCBSize.
-    //
-    // The alignment container is the flex container's content box.
-    if (placeholderContainerIsContainingBlock) {
-      alignAreaSize = aAbsPosCBSize.ConvertTo(pcWM, aAbsPosCBWM);
-      // aAbsPosCBSize is the padding-box, so substract the padding to get the
-      // content box.
-      alignAreaSize -=
-          aPlaceholderContainer->GetLogicalUsedPadding(pcWM).Size(pcWM);
-    } else {
-      alignAreaSize = aPlaceholderContainer->GetLogicalSize(pcWM);
-      LogicalMargin pcBorderPadding =
-          aPlaceholderContainer->GetLogicalUsedBorderAndPadding(pcWM);
-      alignAreaSize -= pcBorderPadding.Size(pcWM);
+      LayoutFrameType parentType = aPlaceholderContainer->Type();
+      LogicalSize alignAreaSize(pcWM);
+      if (parentType == LayoutFrameType::FlexContainer) {
+        // We store the frame rect in FinishAndStoreOverflow, which runs _after_
+        // reflowing the absolute frames, so handle the special case of the
+        // frame being the actual containing block here, by getting the size
+        // from aAbsPosCBSize.
+        //
+        // The alignment container is the flex container's content box.
+        if (placeholderContainerIsContainingBlock) {
+          alignAreaSize = aAbsPosCBSize.ConvertTo(pcWM, aAbsPosCBWM);
+          // aAbsPosCBSize is the padding-box, so substract the padding to get
+          // the content box.
+          alignAreaSize -=
+              aPlaceholderContainer->GetLogicalUsedPadding(pcWM).Size(pcWM);
+        } else {
+          alignAreaSize = aPlaceholderContainer->GetLogicalSize(pcWM);
+          LogicalMargin pcBorderPadding =
+              aPlaceholderContainer->GetLogicalUsedBorderAndPadding(pcWM);
+          alignAreaSize -= pcBorderPadding.Size(pcWM);
+        }
+        return alignAreaSize;
+      }
+      if (parentType == LayoutFrameType::GridContainer) {
+        // This abspos elem's parent is a grid container. Per CSS Grid 10.1
+        // & 10.2:
+        //  - If the grid container *also* generates the abspos containing block
+        //  (a
+        // grid area) for this abspos child, we use that abspos containing block
+        // as the alignment container, too. (And its size is aAbsPosCBSize.)
+        //  - Otherwise, we use the grid's padding box as the alignment
+        //  container.
+        // https://drafts.csswg.org/css-grid/#static-position
+        if (placeholderContainerIsContainingBlock) {
+          // The alignment container is the grid area that we're using as the
+          // absolute containing block.
+          alignAreaSize = aAbsPosCBSize.ConvertTo(pcWM, aAbsPosCBWM);
+        } else {
+          // The alignment container is a the grid container's content box
+          // (which we can get by subtracting away its border & padding from
+          // frame's size):
+          alignAreaSize = aPlaceholderContainer->GetLogicalSize(pcWM);
+          LogicalMargin pcBorderPadding =
+              aPlaceholderContainer->GetLogicalUsedBorderAndPadding(pcWM);
+          alignAreaSize -= pcBorderPadding.Size(pcWM);
+        }
+        return alignAreaSize;
+      }
     }
-  } else if (parentType == LayoutFrameType::GridContainer) {
-    // This abspos elem's parent is a grid container. Per CSS Grid 10.1 & 10.2:
-    //  - If the grid container *also* generates the abspos containing block (a
-    // grid area) for this abspos child, we use that abspos containing block as
-    // the alignment container, too. (And its size is aAbsPosCBSize.)
-    //  - Otherwise, we use the grid's padding box as the alignment container.
-    // https://drafts.csswg.org/css-grid/#static-position
-    if (placeholderContainerIsContainingBlock) {
-      // The alignment container is the grid area that we're using as the
-      // absolute containing block.
-      alignAreaSize = aAbsPosCBSize.ConvertTo(pcWM, aAbsPosCBWM);
-    } else {
-      // The alignment container is a the grid container's content box (which
-      // we can get by subtracting away its border & padding from frame's size):
-      alignAreaSize = aPlaceholderContainer->GetLogicalSize(pcWM);
-      LogicalMargin pcBorderPadding =
-          aPlaceholderContainer->GetLogicalUsedBorderAndPadding(pcWM);
-      alignAreaSize -= pcBorderPadding.Size(pcWM);
-    }
-  } else {
-    NS_ERROR("Unsupported container for abpsos CSS Box Alignment");
-    return 0;  // (leave the child at the start of its alignment container)
-  }
+    // Either we're in scenario 1 but within a non-flex/grid parent, or in
+    // scenario 2.
+    return aAbsPosCBSize.ConvertTo(pcWM, aAbsPosCBWM);
+  }();
 
-  nscoord alignAreaSizeInAxis = (pcAxis == eLogicalAxisInline)
-                                    ? alignAreaSize.ISize(pcWM)
-                                    : alignAreaSize.BSize(pcWM);
+  const nscoord existingOffset = aNonAutoAlignParams
+                                     ? aNonAutoAlignParams->mCurrentStartInset +
+                                           aNonAutoAlignParams->mCurrentEndInset
+                                     : 0;
+  const nscoord alignAreaSizeInAxis =
+      ((pcAxis == LogicalAxis::Inline) ? alignAreaSize.ISize(pcWM)
+                                       : alignAreaSize.BSize(pcWM)) -
+      existingOffset;
 
   AlignJustifyFlags flags = AlignJustifyFlags::IgnoreAutoMargins;
+  // Given that scenario 2 ignores the parent container type, special handling
+  // of absolutely-positioned child is also ignored.
   StyleAlignFlags alignConst =
-      aPlaceholderContainer->CSSAlignmentForAbsPosChild(aKidReflowInput,
-                                                        pcAxis);
+      aNonAutoAlignParams
+          ? aPlaceholderContainer
+                ->CSSAlignmentForAbsPosChildWithinContainingBlock(
+                    aKidReflowInput, pcAxis)
+          : aPlaceholderContainer->CSSAlignmentForAbsPosChild(aKidReflowInput,
+                                                              pcAxis);
   // If the safe bit in alignConst is set, set the safe flag in |flags|.
-  // Note: If no <overflow-position> is specified, we behave as 'unsafe'.
-  // This doesn't quite match the css-align spec, which has an [at-risk]
-  // "smart default" behavior with some extra nuance about scroll containers.
-  if (alignConst & StyleAlignFlags::SAFE) {
+  const auto safetyBits =
+      alignConst & (StyleAlignFlags::SAFE | StyleAlignFlags::UNSAFE);
+  alignConst &= ~StyleAlignFlags::FLAG_BITS;
+  if (safetyBits & StyleAlignFlags::SAFE) {
     flags |= AlignJustifyFlags::OverflowSafe;
   }
-  alignConst &= ~StyleAlignFlags::FLAG_BITS;
 
   // Find out if placeholder-container & the OOF child have the same start-sides
   // in the placeholder-container's pcAxis.
@@ -547,6 +606,10 @@ static nscoord OffsetToAlignedStaticPos(const ReflowInput& aKidReflowInput,
     flags |= AlignJustifyFlags::SameSide;
   }
 
+  if (aNonAutoAlignParams) {
+    flags |= AlignJustifyFlags::AligningMarginBox;
+  }
+
   // (baselineAdjust is unused. CSSAlignmentForAbsPosChild() should've
   // converted 'baseline'/'last baseline' enums to their fallback values.)
   const nscoord baselineAdjust = nscoord(0);
@@ -563,6 +626,40 @@ static nscoord OffsetToAlignedStaticPos(const ReflowInput& aKidReflowInput,
       alignConst, kidAxis, flags, baselineAdjust, alignAreaSizeInAxis,
       aKidReflowInput, kidSizeInOwnWM);
 
+  const auto rawAlignConst =
+      (pcAxis == LogicalAxis::Inline)
+          ? aKidReflowInput.mStylePosition->mJustifySelf._0
+          : aKidReflowInput.mStylePosition->mAlignSelf._0;
+  if (aNonAutoAlignParams && !safetyBits &&
+      rawAlignConst != StyleAlignFlags::AUTO) {
+    // No `safe` or `unsafe` specified - "in-between" behaviour for relevant
+    // alignment values: https://drafts.csswg.org/css-position-3/#abspos-layout
+    // Skip if the raw self alignment for this element is `auto` to preserve
+    // legacy behaviour.
+    // We've already aligned as if unsafe. Now get the union of inset-reduced
+    // containing block and the containing block.
+    const auto unionedStartOffset =
+        std::min(0, aNonAutoAlignParams->mCurrentStartInset);
+    const auto cbSize = aAbsPosCBSize.Size(aAbsPosCBAxis, aAbsPosCBWM);
+    const auto unionedEndOffset =
+        std::max(cbSize, cbSize - aNonAutoAlignParams->mCurrentEndInset);
+    const auto kidSizeInAxis =
+        aKidSizeInAbsPosCBWM.Size(aAbsPosCBAxis, aAbsPosCBWM);
+    if (unionedEndOffset - unionedStartOffset < kidSizeInAxis) {
+      // Kid is bigger than the union - start align it.
+      offset = -aNonAutoAlignParams->mCurrentStartInset + unionedStartOffset;
+    } else {
+      const auto start = aNonAutoAlignParams->mCurrentStartInset;
+      const auto end = start + kidSizeInAxis;
+      // Nudge into the union
+      if (start < unionedStartOffset) {
+        offset = unionedStartOffset - start;
+      } else if (end > unionedEndOffset) {
+        offset = unionedEndOffset - end;
+      }
+    }
+  }
+
   // "offset" is in terms of the CSS Box Alignment container (i.e. it's in
   // terms of pcWM). But our return value needs to in terms of the containing
   // block's writing mode, which might have the opposite directionality in the
@@ -620,7 +717,7 @@ void nsAbsoluteContainingBlock::ResolveSizeDependentOffsets(
       placeholderContainer = GetPlaceholderContainer(aKidReflowInput.mFrame);
       nscoord offset = OffsetToAlignedStaticPos(
           aKidReflowInput, aKidSize, logicalCBSizeOuterWM, placeholderContainer,
-          outerWM, eLogicalAxisInline);
+          outerWM, LogicalAxis::Inline, Nothing{});
       // Shift IStart from its current position (at start corner of the
       // alignment container) by the returned offset.  And set IEnd to the
       // distance between the kid's end edge to containing block's end edge.
@@ -640,7 +737,7 @@ void nsAbsoluteContainingBlock::ResolveSizeDependentOffsets(
       }
       nscoord offset = OffsetToAlignedStaticPos(
           aKidReflowInput, aKidSize, logicalCBSizeOuterWM, placeholderContainer,
-          outerWM, eLogicalAxisBlock);
+          outerWM, LogicalAxis::Block, Nothing{});
       // Shift BStart from its current position (at start corner of the
       // alignment container) by the returned offset.  And set BEnd to the
       // distance between the kid's end edge to containing block's end edge.
@@ -677,16 +774,23 @@ void nsAbsoluteContainingBlock::ResolveAutoMarginsAfterLayout(
                        offsetsInWM.BStartEnd(wm) - marginInWM.BStartEnd(wm);
 
   const auto& styleMargin = aKidReflowInput.mStyleMargin;
+  const auto positionProperty = aKidReflowInput.mStyleDisplay->mPosition;
   if (wm.IsOrthogonalTo(outerWM)) {
     ReflowInput::ComputeAbsPosInlineAutoMargin(
         availMarginSpace, outerWM,
-        styleMargin->mMargin.GetIStart(outerWM).IsAuto(),
-        styleMargin->mMargin.GetIEnd(outerWM).IsAuto(), aMargin, aOffsets);
+        styleMargin->GetMargin(LogicalSide::IStart, outerWM, positionProperty)
+            ->IsAuto(),
+        styleMargin->GetMargin(LogicalSide::IEnd, outerWM, positionProperty)
+            ->IsAuto(),
+        aMargin, aOffsets);
   } else {
     ReflowInput::ComputeAbsPosBlockAutoMargin(
         availMarginSpace, outerWM,
-        styleMargin->mMargin.GetBStart(outerWM).IsAuto(),
-        styleMargin->mMargin.GetBEnd(outerWM).IsAuto(), aMargin, aOffsets);
+        styleMargin->GetMargin(LogicalSide::BStart, outerWM, positionProperty)
+            ->IsAuto(),
+        styleMargin->GetMargin(LogicalSide::BEnd, outerWM, positionProperty)
+            ->IsAuto(),
+        aMargin, aOffsets);
   }
 
   aKidReflowInput.SetComputedLogicalMargin(outerWM, aMargin);
@@ -696,8 +800,8 @@ void nsAbsoluteContainingBlock::ResolveAutoMarginsAfterLayout(
       aKidReflowInput.mFrame->GetProperty(nsIFrame::UsedMarginProperty());
   // InitOffsets should've created a UsedMarginProperty for us, if any margin is
   // auto.
-  MOZ_ASSERT_IF(styleMargin->HasInlineAxisAuto(outerWM) ||
-                    styleMargin->HasBlockAxisAuto(outerWM),
+  MOZ_ASSERT_IF(styleMargin->HasInlineAxisAuto(outerWM, positionProperty) ||
+                    styleMargin->HasBlockAxisAuto(outerWM, positionProperty),
                 propValue);
   if (propValue) {
     *propValue = aMargin.GetPhysicalMargin(outerWM);
@@ -839,12 +943,76 @@ void nsAbsoluteContainingBlock::ReflowAbsoluteFrame(
                                     margin, offsets);
     }
 
+    // If the inset is constrained as non-auto, we may have a child that does
+    // not fill out the inset-reduced containing block. In this case, we need to
+    // align the child by its margin box:
+    // https://drafts.csswg.org/css-position-3/#abspos-layout
+    const auto* stylePos = aKidFrame->StylePosition();
+    auto positionProperty = aKidFrame->StyleDisplay()->mPosition;
+    const bool iInsetAuto =
+        stylePos
+            ->GetAnchorResolvedInset(LogicalSide::IStart, outerWM,
+                                     positionProperty)
+            ->IsAuto() ||
+        stylePos
+            ->GetAnchorResolvedInset(LogicalSide::IEnd, outerWM,
+                                     positionProperty)
+            ->IsAuto();
+    const bool bInsetAuto =
+        stylePos
+            ->GetAnchorResolvedInset(LogicalSide::BStart, outerWM,
+                                     positionProperty)
+            ->IsAuto() ||
+        stylePos
+            ->GetAnchorResolvedInset(LogicalSide::BEnd, outerWM,
+                                     positionProperty)
+            ->IsAuto();
+    const LogicalSize logicalCBSizeOuterWM(outerWM, aContainingBlock.Size());
+    const LogicalSize kidMarginBox{
+        outerWM, margin.IStartEnd(outerWM) + kidSize.ISize(outerWM),
+        margin.BStartEnd(outerWM) + kidSize.BSize(outerWM)};
+    const auto* placeholderContainer =
+        GetPlaceholderContainer(kidReflowInput.mFrame);
+
+    if (!iInsetAuto) {
+      MOZ_ASSERT(!kidReflowInput.mFlags.mIOffsetsNeedCSSAlign,
+                 "Non-auto inline inset but requires CSS alignment for static "
+                 "position?");
+      auto alignOffset = OffsetToAlignedStaticPos(
+          kidReflowInput, kidMarginBox, logicalCBSizeOuterWM,
+          placeholderContainer, outerWM, LogicalAxis::Inline,
+          Some(NonAutoAlignParams{
+              offsets.IStart(outerWM),
+              offsets.IEnd(outerWM),
+          }));
+
+      offsets.IStart(outerWM) += alignOffset;
+      offsets.IEnd(outerWM) =
+          logicalCBSizeOuterWM.ISize(outerWM) -
+          (offsets.IStart(outerWM) + kidMarginBox.ISize(outerWM));
+    }
+    if (!bInsetAuto) {
+      MOZ_ASSERT(!kidReflowInput.mFlags.mBOffsetsNeedCSSAlign,
+                 "Non-auto block inset but requires CSS alignment for static "
+                 "position?");
+      auto alignOffset = OffsetToAlignedStaticPos(
+          kidReflowInput, kidMarginBox, logicalCBSizeOuterWM,
+          placeholderContainer, outerWM, LogicalAxis::Block,
+          Some(NonAutoAlignParams{
+              offsets.BStart(outerWM),
+              offsets.BEnd(outerWM),
+          }));
+      offsets.BStart(outerWM) += alignOffset;
+      offsets.BEnd(outerWM) =
+          logicalCBSizeOuterWM.BSize(outerWM) -
+          (offsets.BStart(outerWM) + kidMarginBox.BSize(outerWM));
+    }
+
     LogicalRect rect(outerWM,
-                     border.IStart(outerWM) + offsets.IStart(outerWM) +
-                         margin.IStart(outerWM),
-                     border.BStart(outerWM) + offsets.BStart(outerWM) +
-                         margin.BStart(outerWM),
-                     kidSize.ISize(outerWM), kidSize.BSize(outerWM));
+                     border.StartOffset(outerWM) +
+                         offsets.StartOffset(outerWM) +
+                         margin.StartOffset(outerWM),
+                     kidSize);
     nsRect r = rect.GetPhysicalRect(
         outerWM, logicalCBSize.GetPhysicalSize(wm) +
                      border.Size(outerWM).GetPhysicalSize(outerWM));
