@@ -7,7 +7,7 @@
 /* class that a parent frame uses to reflow a block frame */
 
 #include "nsBlockReflowContext.h"
-#include "BlockReflowInput.h"
+#include "BlockReflowState.h"
 #include "nsFloatManager.h"
 #include "nsColumnSetFrame.h"
 #include "nsContainerFrame.h"
@@ -44,7 +44,7 @@ static nsIFrame* DescendIntoBlockLevelFrame(nsIFrame* aFrame) {
 }
 
 bool nsBlockReflowContext::ComputeCollapsedBStartMargin(
-    const ReflowInput& aRI, nsCollapsingMargin* aMargin,
+    const ReflowInput& aRI, CollapsingMargin* aMargin,
     nsIFrame* aClearanceFrame, bool* aMayNeedRetry, bool* aBlockIsEmpty) {
   WritingMode wm = aRI.GetWritingMode();
   WritingMode parentWM = mMetrics.GetWritingMode();
@@ -158,7 +158,7 @@ bool nsBlockReflowContext::ComputeCollapsedBStartMargin(
                                          availSpace);
             // Record that we're being optimistic by assuming the kid
             // has no clearance
-            if (kid->StyleDisplay()->mBreakType != StyleClear::None ||
+            if (kid->StyleDisplay()->mClear != StyleClear::None ||
                 !nsBlockFrame::BlockCanIntersectFloats(kid)) {
               *aMayNeedRetry = true;
             }
@@ -209,17 +209,19 @@ done:
   return dirtiedLine;
 }
 
-void nsBlockReflowContext::ReflowBlock(
-    const LogicalRect& aSpace, bool aApplyBStartMargin,
-    nsCollapsingMargin& aPrevMargin, nscoord aClearance,
-    bool aIsAdjacentWithBStart, nsLineBox* aLine, ReflowInput& aFrameRI,
-    nsReflowStatus& aFrameReflowStatus, BlockReflowInput& aState) {
+void nsBlockReflowContext::ReflowBlock(const LogicalRect& aSpace,
+                                       bool aApplyBStartMargin,
+                                       CollapsingMargin& aPrevMargin,
+                                       nscoord aClearance, nsLineBox* aLine,
+                                       ReflowInput& aFrameRI,
+                                       nsReflowStatus& aFrameReflowStatus,
+                                       BlockReflowState& aState) {
   mFrame = aFrameRI.mFrame;
   mWritingMode = aState.mReflowInput.GetWritingMode();
   mContainerSize = aState.ContainerSize();
   mSpace = aSpace;
 
-  if (!aIsAdjacentWithBStart) {
+  if (!aState.IsAdjacentWithBStart()) {
     aFrameRI.mFlags.mIsTopOfPage = false;  // make sure this is cleared
   }
 
@@ -237,13 +239,13 @@ void nsBlockReflowContext::ReflowBlock(
     // child frame doesn't think it can reflow into its margin area.
     if (mWritingMode.IsOrthogonalTo(mFrame->GetWritingMode())) {
       if (NS_UNCONSTRAINEDSIZE != aFrameRI.AvailableISize()) {
-        aFrameRI.AvailableISize() -= mBStartMargin.get() + aClearance;
-        aFrameRI.AvailableISize() = std::max(0, aFrameRI.AvailableISize());
+        aFrameRI.SetAvailableISize(std::max(
+            0, aFrameRI.AvailableISize() - mBStartMargin.Get() - aClearance));
       }
     } else {
       if (NS_UNCONSTRAINEDSIZE != aFrameRI.AvailableBSize()) {
-        aFrameRI.AvailableBSize() -= mBStartMargin.get() + aClearance;
-        aFrameRI.AvailableBSize() = std::max(0, aFrameRI.AvailableBSize());
+        aFrameRI.SetAvailableBSize(std::max(
+            0, aFrameRI.AvailableBSize() - mBStartMargin.Get() - aClearance));
       }
     }
   } else {
@@ -264,7 +266,7 @@ void nsBlockReflowContext::ReflowBlock(
     // reflow auto inline-start/end margins will have a zero value.
     LogicalMargin usedMargin = aFrameRI.ComputedLogicalMargin(mWritingMode);
     mICoord = mSpace.IStart(mWritingMode) + usedMargin.IStart(mWritingMode);
-    mBCoord = mSpace.BStart(mWritingMode) + mBStartMargin.get() + aClearance;
+    mBCoord = mSpace.BStart(mWritingMode) + mBStartMargin.Get() + aClearance;
 
     LogicalRect space(
         mWritingMode, mICoord, mBCoord,
@@ -273,7 +275,7 @@ void nsBlockReflowContext::ReflowBlock(
     tI = space.LineLeft(mWritingMode, mContainerSize);
     tB = mBCoord;
 
-    if (!mFrame->HasAnyStateBits(NS_BLOCK_FLOAT_MGR)) {
+    if (!mFrame->HasAnyStateBits(NS_BLOCK_BFC)) {
       aFrameRI.mBlockDelta =
           mOuterReflowInput.mBlockDelta + mBCoord - aLine->BStart();
     }
@@ -328,7 +330,9 @@ void nsBlockReflowContext::ReflowBlock(
       // which detaches the placeholder from the float.
       nsOverflowContinuationTracker::AutoFinish fini(aState.mOverflowTracker,
                                                      mFrame);
-      kidNextInFlow->GetParent()->DeleteNextInFlowChild(kidNextInFlow, true);
+      nsIFrame::DestroyContext context(mPresContext->PresShell());
+      kidNextInFlow->GetParent()->DeleteNextInFlowChild(context, kidNextInFlow,
+                                                        true);
     }
   }
 }
@@ -340,7 +344,7 @@ void nsBlockReflowContext::ReflowBlock(
  */
 bool nsBlockReflowContext::PlaceBlock(const ReflowInput& aReflowInput,
                                       bool aForceFit, nsLineBox* aLine,
-                                      nsCollapsingMargin& aBEndMarginResult,
+                                      CollapsingMargin& aBEndMarginResult,
                                       OverflowAreas& aOverflowAreas,
                                       const nsReflowStatus& aReflowStatus) {
   // Compute collapsed block-end margin value.
@@ -401,7 +405,7 @@ bool nsBlockReflowContext::PlaceBlock(const ReflowInput& aReflowInput,
     // because the containing block will place the next line at the
     // line's BEnd, and it must place the next line at a different
     // point from where this empty block will be.
-    backupContainingBlockAdvance = mBStartMargin.get();
+    backupContainingBlockAdvance = mBStartMargin.Get();
   }
 
   // See if the frame fit. If it's the first frame or empty then it
