@@ -14,6 +14,9 @@
 #include "mozilla/StaticPrefs_layout.h"
 #include "mozilla/ToString.h"
 #include "nsCSSRendering.h"
+#include "nsDisplayList.h"
+#include "nsIFrameInlines.h"
+#include "nsLayoutUtils.h"
 
 using namespace mozilla;
 using namespace mozilla::layout;
@@ -23,21 +26,22 @@ static LazyLogModule sColumnSetLog("ColumnSet");
 #define COLUMN_SET_LOG(msg, ...) \
   MOZ_LOG(sColumnSetLog, LogLevel::Debug, (msg, ##__VA_ARGS__))
 
-class nsDisplayColumnRule : public nsPaintedDisplayItem {
+class nsDisplayColumnRule final : public nsPaintedDisplayItem {
  public:
   nsDisplayColumnRule(nsDisplayListBuilder* aBuilder, nsIFrame* aFrame)
       : nsPaintedDisplayItem(aBuilder, aFrame) {
     MOZ_COUNT_CTOR(nsDisplayColumnRule);
   }
-  MOZ_COUNTED_DTOR_OVERRIDE(nsDisplayColumnRule)
 
-  /**
-   * Returns the frame's ink overflow rect instead of the frame's bounds.
-   */
+  MOZ_COUNTED_DTOR_FINAL(nsDisplayColumnRule)
+
   nsRect GetBounds(nsDisplayListBuilder* aBuilder, bool* aSnap) const override {
     *aSnap = false;
-    return static_cast<nsColumnSetFrame*>(mFrame)->CalculateColumnRuleBounds(
-        ToReferenceFrame());
+    // We just return the frame's ink-overflow rect, which is guaranteed to
+    // contain all the column-rule areas.  It's not worth calculating the exact
+    // union of those areas since it would only lead to performance improvements
+    // during painting in rare edge cases.
+    return mFrame->InkOverflowRect() + ToReferenceFrame();
   }
 
   bool CreateWebRenderCommands(
@@ -57,7 +61,7 @@ class nsDisplayColumnRule : public nsPaintedDisplayItem {
 void nsDisplayColumnRule::Paint(nsDisplayListBuilder* aBuilder,
                                 gfxContext* aCtx) {
   static_cast<nsColumnSetFrame*>(mFrame)->CreateBorderRenderers(
-      mBorderRenderers, aCtx, GetPaintRect(), ToReferenceFrame());
+      mBorderRenderers, aCtx, GetPaintRect(aBuilder, aCtx), ToReferenceFrame());
 
   for (auto iter = mBorderRenderers.begin(); iter != mBorderRenderers.end();
        iter++) {
@@ -71,11 +75,16 @@ bool nsDisplayColumnRule::CreateWebRenderCommands(
     const StackingContextHelper& aSc,
     mozilla::layers::RenderRootStateManager* aManager,
     nsDisplayListBuilder* aDisplayListBuilder) {
-  RefPtr<gfxContext> screenRefCtx = gfxContext::CreateOrNull(
-      gfxPlatform::GetPlatform()->ScreenReferenceDrawTarget().get());
+  RefPtr dt = gfxPlatform::GetPlatform()->ScreenReferenceDrawTarget();
+  if (!dt || !dt->IsValid()) {
+    return false;
+  }
+  gfxContext screenRefCtx(dt);
 
+  bool dummy;
   static_cast<nsColumnSetFrame*>(mFrame)->CreateBorderRenderers(
-      mBorderRenderers, screenRefCtx, GetPaintRect(), ToReferenceFrame());
+      mBorderRenderers, &screenRefCtx, GetBounds(aDisplayListBuilder, &dummy),
+      ToReferenceFrame());
 
   if (mBorderRenderers.IsEmpty()) {
     return true;
@@ -88,6 +97,9 @@ bool nsDisplayColumnRule::CreateWebRenderCommands(
   return true;
 }
 
+// The maximum number of columns we support.
+static constexpr int32_t kMaxColumnCount = 1000;
+
 /**
  * Tracking issues:
  *
@@ -116,14 +128,20 @@ void nsColumnSetFrame::ForEachColumnRule(
     const std::function<void(const nsRect& lineRect)>& aSetLineRect,
     const nsPoint& aPt) const {
   nsIFrame* child = mFrames.FirstChild();
-  if (!child) return;  // no columns
+  if (!child) {
+    return;  // no columns
+  }
 
   nsIFrame* nextSibling = child->GetNextSibling();
-  if (!nextSibling) return;  // 1 column only - this means no gap to draw on
+  if (!nextSibling) {
+    return;  // 1 column only - this means no gap to draw on
+  }
 
   const nsStyleColumn* colStyle = StyleColumn();
-  nscoord ruleWidth = colStyle->GetComputedColumnRuleWidth();
-  if (!ruleWidth) return;
+  nscoord ruleWidth = colStyle->GetColumnRuleWidth();
+  if (!ruleWidth) {
+    return;
+  }
 
   WritingMode wm = GetWritingMode();
   bool isVertical = wm.IsVertical();
@@ -164,17 +182,6 @@ void nsColumnSetFrame::ForEachColumnRule(
   }
 }
 
-nsRect nsColumnSetFrame::CalculateColumnRuleBounds(
-    const nsPoint& aOffset) const {
-  nsRect combined;
-  ForEachColumnRule(
-      [&combined](const nsRect& aLineRect) {
-        combined = combined.Union(aLineRect);
-      },
-      aOffset);
-  return combined;
-}
-
 void nsColumnSetFrame::CreateBorderRenderers(
     nsTArray<nsCSSBorderRenderer>& aBorderRenderers, gfxContext* aCtx,
     const nsRect& aDirtyRect, const nsPoint& aPt) {
@@ -184,36 +191,39 @@ void nsColumnSetFrame::CreateBorderRenderers(
   StyleBorderStyle ruleStyle;
 
   // Per spec, inset => ridge and outset => groove
-  if (colStyle->mColumnRuleStyle == StyleBorderStyle::Inset)
+  if (colStyle->mColumnRuleStyle == StyleBorderStyle::Inset) {
     ruleStyle = StyleBorderStyle::Ridge;
-  else if (colStyle->mColumnRuleStyle == StyleBorderStyle::Outset)
+  } else if (colStyle->mColumnRuleStyle == StyleBorderStyle::Outset) {
     ruleStyle = StyleBorderStyle::Groove;
-  else
+  } else {
     ruleStyle = colStyle->mColumnRuleStyle;
+  }
 
-  nscoord ruleWidth = colStyle->GetComputedColumnRuleWidth();
-  if (!ruleWidth) return;
+  nscoord ruleWidth = colStyle->GetColumnRuleWidth();
+  if (!ruleWidth) {
+    return;
+  }
 
   aBorderRenderers.Clear();
   nscolor ruleColor =
       GetVisitedDependentColor(&nsStyleColumn::mColumnRuleColor);
 
-  nsPresContext* presContext = PresContext();
+  nsPresContext* pc = PresContext();
   // In order to re-use a large amount of code, we treat the column rule as a
   // border. We create a new border style object and fill in all the details of
   // the column rule as the left border. PaintBorder() does all the rendering
   // for us, so we not only save an enormous amount of code but we'll support
   // all the line styles that we support on borders!
-  nsStyleBorder border(*presContext->Document());
+  nsStyleBorder border;
   Sides skipSides;
   if (isVertical) {
-    border.SetBorderWidth(eSideTop, ruleWidth);
+    border.SetBorderWidth(eSideTop, ruleWidth, pc->AppUnitsPerDevPixel());
     border.SetBorderStyle(eSideTop, ruleStyle);
     border.mBorderTopColor = StyleColor::FromColor(ruleColor);
     skipSides |= mozilla::SideBits::eLeftRight;
     skipSides |= mozilla::SideBits::eBottom;
   } else {
-    border.SetBorderWidth(eSideLeft, ruleWidth);
+    border.SetBorderWidth(eSideLeft, ruleWidth, pc->AppUnitsPerDevPixel());
     border.SetBorderStyle(eSideLeft, ruleStyle);
     border.mBorderLeftColor = StyleColor::FromColor(ruleColor);
     skipSides |= mozilla::SideBits::eTopBottom;
@@ -238,7 +248,7 @@ void nsColumnSetFrame::CreateBorderRenderers(
         bool borderIsEmpty = false;
         Maybe<nsCSSBorderRenderer> br =
             nsCSSRendering::CreateBorderRendererWithStyleBorder(
-                presContext, dt, this, aDirtyRect, aLineRect, border, Style(),
+                pc, dt, this, aDirtyRect, aLineRect, border, Style(),
                 &borderIsEmpty, skipSides);
         if (br.isSome()) {
           MOZ_ASSERT(!borderIsEmpty);
@@ -271,7 +281,10 @@ nsColumnSetFrame::ReflowConfig nsColumnSetFrame::ChooseColumnStrategy(
   nscoord colBSize = aReflowInput.AvailableBSize();
   nscoord colGap =
       ColumnUtils::GetColumnGap(this, aReflowInput.ComputedISize());
-  int32_t numColumns = colStyle->mColumnCount;
+  int32_t numColumns =
+      colStyle->mColumnCount.IsAuto()
+          ? 0
+          : std::min(colStyle->mColumnCount.AsInteger(), kMaxColumnCount);
 
   // If column-fill is set to 'balance' or we have a column-span sibling, then
   // we want to balance the columns.
@@ -285,6 +298,7 @@ nsColumnSetFrame::ReflowConfig nsColumnSetFrame::ChooseColumnStrategy(
     if (balancingDepth == kMaxNestedColumnBalancingDepth) {
       isBalancing = false;
       numColumns = 1;
+      aForceAuto = true;
     }
   }
 
@@ -304,7 +318,7 @@ nsColumnSetFrame::ReflowConfig nsColumnSetFrame::ChooseColumnStrategy(
       // This expression uses truncated rounding, which is what we
       // want
       int32_t maxColumns =
-          std::min(nscoord(nsStyleColumn::kMaxColumnCount),
+          std::min(nscoord(kMaxColumnCount),
                    (availContentISize + colGap) / (colGap + colISize));
       numColumns = std::max(1, std::min(numColumns, maxColumns));
     }
@@ -333,8 +347,7 @@ nsColumnSetFrame::ReflowConfig nsColumnSetFrame::ChooseColumnStrategy(
       if (colGap + colISize > 0) {
         numColumns = (availContentISize + colGap) / (colGap + colISize);
         // The number of columns should never exceed kMaxColumnCount.
-        numColumns =
-            std::min(nscoord(nsStyleColumn::kMaxColumnCount), numColumns);
+        numColumns = std::min(kMaxColumnCount, numColumns);
       }
       if (numColumns <= 0) {
         numColumns = 1;
@@ -396,26 +409,6 @@ static void MarkPrincipalChildrenDirty(nsIFrame* aFrame) {
   }
 }
 
-nsColumnSetFrame::ColumnBalanceData nsColumnSetFrame::ReflowColumns(
-    ReflowOutput& aDesiredSize, const ReflowInput& aReflowInput,
-    nsReflowStatus& aReflowStatus, ReflowConfig& aConfig,
-    bool aUnboundedLastColumn) {
-  const ColumnBalanceData colData = ReflowChildren(
-      aDesiredSize, aReflowInput, aReflowStatus, aConfig, aUnboundedLastColumn);
-
-  if (!colData.mHasExcessBSize) {
-    return colData;
-  }
-
-  aConfig = ChooseColumnStrategy(aReflowInput, true);
-
-  // We need to reflow our children again one last time, otherwise we might
-  // end up with a stale column block-size for some of our columns, since we
-  // bailed out of balancing.
-  return ReflowChildren(aDesiredSize, aReflowInput, aReflowStatus, aConfig,
-                        aUnboundedLastColumn);
-}
-
 static void MoveChildTo(nsIFrame* aChild, LogicalPoint aOrigin, WritingMode aWM,
                         const nsSize& aContainerSize) {
   if (aChild->GetLogicalPosition(aWM, aContainerSize) == aOrigin) {
@@ -426,15 +419,17 @@ static void MoveChildTo(nsIFrame* aChild, LogicalPoint aOrigin, WritingMode aWM,
   nsContainerFrame::PlaceFrameView(aChild);
 }
 
-nscoord nsColumnSetFrame::GetMinISize(gfxContext* aRenderingContext) {
+nscoord nsColumnSetFrame::IntrinsicISize(const IntrinsicSizeInput& input,
+                                         IntrinsicISizeType aType) {
+  return aType == IntrinsicISizeType::MinISize ? MinISize(input)
+                                               : PrefISize(input);
+}
+
+nscoord nsColumnSetFrame::MinISize(const IntrinsicSizeInput& aInput) {
   nscoord iSize = 0;
-  DISPLAY_MIN_INLINE_SIZE(this, iSize);
 
   if (mFrames.FirstChild()) {
-    // We want to ignore this in the case that we're size contained
-    // because our children should not contribute to our
-    // intrinsic size.
-    iSize = mFrames.FirstChild()->GetMinISize(aRenderingContext);
+    iSize = mFrames.FirstChild()->GetMinISize(aInput);
   }
   const nsStyleColumn* colStyle = StyleColumn();
   if (colStyle->mColumnWidth.IsLength()) {
@@ -445,26 +440,25 @@ nscoord nsColumnSetFrame::GetMinISize(gfxContext* aRenderingContext) {
     // of the child's min-width with any specified column width.
     iSize = std::min(iSize, colISize);
   } else {
-    NS_ASSERTION(colStyle->mColumnCount > 0,
+    NS_ASSERTION(!colStyle->mColumnCount.IsAuto(),
                  "column-count and column-width can't both be auto");
     // As available width reduces to zero, we still have mColumnCount columns,
     // so compute our minimum size based on the number of columns and their gaps
     // and minimum per-column size.
     nscoord colGap = ColumnUtils::GetColumnGap(this, NS_UNCONSTRAINEDSIZE);
-    iSize = ColumnUtils::IntrinsicISize(colStyle->mColumnCount, colGap, iSize);
+    iSize = ColumnUtils::IntrinsicISize(colStyle->mColumnCount.AsInteger(),
+                                        colGap, iSize);
   }
   // XXX count forced column breaks here? Maybe we should return the child's
   // min-width times the minimum number of columns.
   return iSize;
 }
 
-nscoord nsColumnSetFrame::GetPrefISize(gfxContext* aRenderingContext) {
+nscoord nsColumnSetFrame::PrefISize(const IntrinsicSizeInput& aInput) {
   // Our preferred width is our desired column width, if specified, otherwise
   // the child's preferred width, times the number of columns, plus the width
   // of any required column gaps
   // XXX what about forced column breaks here?
-  nscoord result = 0;
-  DISPLAY_PREF_INLINE_SIZE(this, result);
   const nsStyleColumn* colStyle = StyleColumn();
 
   nscoord colISize;
@@ -472,25 +466,19 @@ nscoord nsColumnSetFrame::GetPrefISize(gfxContext* aRenderingContext) {
     colISize =
         ColumnUtils::ClampUsedColumnWidth(colStyle->mColumnWidth.AsLength());
   } else if (mFrames.FirstChild()) {
-    // We want to ignore this in the case that we're size contained
-    // because our children should not contribute to our
-    // intrinsic size.
-    colISize = mFrames.FirstChild()->GetPrefISize(aRenderingContext);
+    colISize = mFrames.FirstChild()->GetPrefISize(aInput);
   } else {
     colISize = 0;
   }
 
   // If column-count is auto, assume one column.
   uint32_t numColumns =
-      colStyle->mColumnCount == nsStyleColumn::kColumnCountAuto
-          ? 1
-          : colStyle->mColumnCount;
+      colStyle->mColumnCount.IsAuto() ? 1 : colStyle->mColumnCount.AsInteger();
   nscoord colGap = ColumnUtils::GetColumnGap(this, NS_UNCONSTRAINEDSIZE);
-  result = ColumnUtils::IntrinsicISize(numColumns, colGap, colISize);
-  return result;
+  return ColumnUtils::IntrinsicISize(numColumns, colGap, colISize);
 }
 
-nsColumnSetFrame::ColumnBalanceData nsColumnSetFrame::ReflowChildren(
+nsColumnSetFrame::ColumnBalanceData nsColumnSetFrame::ReflowColumns(
     ReflowOutput& aDesiredSize, const ReflowInput& aReflowInput,
     nsReflowStatus& aStatus, const ReflowConfig& aConfig,
     bool aUnboundedLastColumn) {
@@ -546,7 +534,7 @@ nsColumnSetFrame::ColumnBalanceData nsColumnSetFrame::ReflowChildren(
   bool reflowNext = false;
 
   while (child) {
-    const bool isMeasuringFeasibleContentBSize =
+    const bool reflowLastColumnWithUnconstrainedAvailBSize =
         aUnboundedLastColumn && colData.mColCount == aConfig.mUsedColCount &&
         aConfig.mIsBalancing;
 
@@ -568,7 +556,7 @@ nsColumnSetFrame::ColumnBalanceData nsColumnSetFrame::ReflowChildren(
         // content from its next sibling (Note that it might be the last column,
         // but not be the last child because the desired number of columns has
         // changed.)
-        isMeasuringFeasibleContentBSize;
+        reflowLastColumnWithUnconstrainedAvailBSize;
 
     // If column-fill is auto (not the default), then we might need to
     // move content between columns for any change in column block-size.
@@ -594,17 +582,17 @@ nsColumnSetFrame::ColumnBalanceData nsColumnSetFrame::ReflowChildren(
     // this is a calculation that affects layout.
     if (!reflowChild && shrinkingBSize) {
       switch (wm.GetBlockDir()) {
-        case WritingMode::eBlockTB:
+        case WritingMode::BlockDir::TB:
           if (child->ScrollableOverflowRect().YMost() > aConfig.mColBSize) {
             reflowChild = true;
           }
           break;
-        case WritingMode::eBlockLR:
+        case WritingMode::BlockDir::LR:
           if (child->ScrollableOverflowRect().XMost() > aConfig.mColBSize) {
             reflowChild = true;
           }
           break;
-        case WritingMode::eBlockRL:
+        case WritingMode::BlockDir::RL:
           // XXX not sure how to handle this, so for now just don't attempt
           // the optimization
           reflowChild = true;
@@ -638,12 +626,12 @@ nsColumnSetFrame::ColumnBalanceData nsColumnSetFrame::ReflowChildren(
                      colData.mColCount, child, ToString(aStatus).c_str());
     } else {
       LogicalSize availSize(wm, aConfig.mColISize, aConfig.mColBSize);
-      if (isMeasuringFeasibleContentBSize) {
+      if (reflowLastColumnWithUnconstrainedAvailBSize) {
         availSize.BSize(wm) = NS_UNCONSTRAINEDSIZE;
 
         COLUMN_SET_LOG(
-            "%s: Measuring content block-size, change available block-size "
-            "from %d to %d",
+            "%s: Reflowing last column with unconstrained block-size. Change "
+            "available block-size from %d to %d",
             __func__, aConfig.mColBSize, availSize.BSize(wm));
       }
 
@@ -654,18 +642,49 @@ nsColumnSetFrame::ColumnBalanceData nsColumnSetFrame::ReflowChildren(
       LogicalSize kidCBSize(wm, availSize.ISize(wm), computedBSize);
       ReflowInput kidReflowInput(PresContext(), aReflowInput, child, availSize,
                                  Some(kidCBSize));
-      kidReflowInput.mFlags.mIsTopOfPage = true;
+      kidReflowInput.mFlags.mIsTopOfPage = [&]() {
+        const bool isNestedMulticolOrInRootPaginatedDoc =
+            aReflowInput.mParentReflowInput->mFrame->HasAnyStateBits(
+                NS_FRAME_HAS_MULTI_COLUMN_ANCESTOR) ||
+            PresContext()->IsRootPaginatedDocument();
+        if (isNestedMulticolOrInRootPaginatedDoc) {
+          if (aConfig.mForceAuto) {
+            // If we are forced to fill columns sequentially, force fit the
+            // content whether we are at top of page or not.
+            return true;
+          }
+          if (aReflowInput.mFlags.mIsTopOfPage) {
+            // If this is the last balancing reflow, we want to force fit
+            // content to avoid infinite loops.
+            return !aConfig.mIsBalancing || aConfig.mIsLastBalancingReflow;
+          }
+          // If we are a not at the top of page, we shouldn't force fit content.
+          // This is because our ColumnSetWrapperFrame can be pushed to the next
+          // column or page and reflowed again with a potentially larger
+          // available block-size.
+          return false;
+        }
+        // We are a top-level multicol in a non-paginated root document or in a
+        // subdocument (regardless of whether the root document is paginated).
+        // Force fit the content only if we are not balancing columns.
+        return !aConfig.mIsBalancing;
+      }();
       kidReflowInput.mFlags.mTableIsSplittable = false;
       kidReflowInput.mFlags.mIsColumnBalancing = aConfig.mIsBalancing;
+      kidReflowInput.mFlags.mIsInLastColumnBalancingReflow =
+          aConfig.mIsLastBalancingReflow;
+      kidReflowInput.mBreakType = ReflowInput::BreakType::Column;
 
       // We need to reflow any float placeholders, even if our column block-size
       // hasn't changed.
       kidReflowInput.mFlags.mMustReflowPlaceholders = !changingBSize;
 
       COLUMN_SET_LOG(
-          "%s: Reflowing child #%d %p: availSize=(%d,%d), kidCBSize=(%d,%d)",
+          "%s: Reflowing child #%d %p: availSize=(%d,%d), kidCBSize=(%d,%d), "
+          "child's mIsTopOfPage=%d",
           __func__, colData.mColCount, child, availSize.ISize(wm),
-          availSize.BSize(wm), kidCBSize.ISize(wm), kidCBSize.BSize(wm));
+          availSize.BSize(wm), kidCBSize.ISize(wm), kidCBSize.BSize(wm),
+          kidReflowInput.mFlags.mIsTopOfPage);
 
       // Note if the column's next in flow is not being changed by this
       // incremental reflow. This may allow the current column to avoid trying
@@ -684,13 +703,19 @@ nsColumnSetFrame::ColumnBalanceData nsColumnSetFrame::ReflowChildren(
       // really big float in the first column and text in following
       // columns would flow around it.
 
-      // Reflow the frame
-      const auto childMargin = kidReflowInput.ComputedLogicalMargin(wm);
-      const LogicalPoint origin(wm, childOrigin.I(wm) + childMargin.IStart(wm),
-                                childOrigin.B(wm) + childMargin.BStart(wm));
+      MOZ_ASSERT(kidReflowInput.ComputedLogicalMargin(wm).IsAllZero(),
+                 "-moz-column-content has no margin!");
       aStatus.Reset();
       ReflowChild(child, PresContext(), kidDesiredSize, kidReflowInput, wm,
-                  origin, containerSize, ReflowChildFlags::Default, aStatus);
+                  childOrigin, containerSize, ReflowChildFlags::Default,
+                  aStatus);
+
+      if (colData.mColCount == 1 && aStatus.IsInlineBreakBefore()) {
+        COLUMN_SET_LOG("%s: Content in the first column reports break-before!",
+                       __func__);
+        allFit = false;
+        break;
+      }
 
       reflowNext = aStatus.NextInFlowNeedsReflow();
 
@@ -730,7 +755,7 @@ nsColumnSetFrame::ColumnBalanceData nsColumnSetFrame::ReflowChildren(
           "childContentBEnd=%d, CarriedOutBEndMargin=%d (ignored)",
           __func__, colData.mColCount, child, ToString(aStatus).c_str(),
           kidDesiredSize.ISize(wm), kidDesiredSize.BSize(wm), childContentBEnd,
-          kidDesiredSize.mCarriedOutBEndMargin.get());
+          kidDesiredSize.mCarriedOutBEndMargin.Get());
     }
 
     contentRect.UnionRect(contentRect, child->GetRect());
@@ -743,7 +768,7 @@ nsColumnSetFrame::ColumnBalanceData nsColumnSetFrame::ReflowChildren(
     // Build a continuation column if necessary
     nsIFrame* kidNextInFlow = child->GetNextInFlow();
 
-    if (aStatus.IsFullyComplete() && !aStatus.IsTruncated()) {
+    if (aStatus.IsFullyComplete()) {
       NS_ASSERTION(!kidNextInFlow, "next in flow should have been deleted");
       child = nullptr;
       break;
@@ -776,14 +801,6 @@ nsColumnSetFrame::ColumnBalanceData nsColumnSetFrame::ReflowChildren(
       kidNextInFlow->RemoveStateBits(NS_FRAME_IS_OVERFLOW_CONTAINER);
     }
 
-    if (contentBEnd > aReflowInput.mCBReflowInput->ComputedMaxBSize() &&
-        aConfig.mIsBalancing) {
-      // We overflowed vertically, but have not exceeded the number of
-      // columns. We're going to go into overflow columns now, so balancing
-      // no longer applies.
-      colData.mHasExcessBSize = true;
-    }
-
     // We have reached the maximum number of columns. If we are balancing, stop
     // this reflow and continue finding the optimal balancing block-size.
     //
@@ -834,7 +851,7 @@ nsColumnSetFrame::ColumnBalanceData nsColumnSetFrame::ReflowChildren(
       kidNextInFlow->MarkSubtreeDirty();
       // Move any of our leftover columns to our overflow list. Our
       // next-in-flow will eventually pick them up.
-      nsFrameList continuationColumns = mFrames.RemoveFramesAfter(child);
+      nsFrameList continuationColumns = mFrames.TakeFramesAfter(child);
       if (continuationColumns.NotEmpty()) {
         SetOverflowFrames(std::move(continuationColumns));
       }
@@ -895,7 +912,7 @@ nsColumnSetFrame::ColumnBalanceData nsColumnSetFrame::ReflowChildren(
 
     // If a) our parent ColumnSetWrapper has constrained block-size
     // (nsBlockFrame::ReflowBlockFrame() applies the block-size constraint
-    // when creating BlockReflowInput for ColumnSetFrame); and b) we are the
+    // when creating a ReflowInput for ColumnSetFrame child); and b) we are the
     // sole ColumnSet or the last ColumnSet continuation split by column-spans
     // in a ColumnSetWrapper, extend our block-size to consume the available
     // block-size so that the column-rules are drawn to the content block-end
@@ -930,8 +947,7 @@ nsColumnSetFrame::ColumnBalanceData nsColumnSetFrame::ReflowChildren(
     aDesiredSize.UnionOverflowAreasWithDesiredBounds();
   }
 
-  colData.mFeasible =
-      allFit && aStatus.IsFullyComplete() && !aStatus.IsTruncated();
+  colData.mFeasible = allFit && aStatus.IsFullyComplete();
 
   COLUMN_SET_LOG(
       "%s: Done column reflow pass: %s, mMaxBSize=%d, mSumBSize=%d, "
@@ -953,7 +969,7 @@ void nsColumnSetFrame::DrainOverflowColumns() {
     if (overflows) {
       nsContainerFrame::ReparentFrameViewList(*overflows, prev, this);
 
-      mFrames.InsertFrames(this, nullptr, *overflows);
+      mFrames.InsertFrames(this, nullptr, std::move(*overflows));
     }
   }
 
@@ -963,7 +979,7 @@ void nsColumnSetFrame::DrainOverflowColumns() {
   if (overflows) {
     // We're already the parent for these frames, so no need to set
     // their parent again.
-    mFrames.AppendFrames(nullptr, *overflows);
+    mFrames.AppendFrames(nullptr, std::move(*overflows));
   }
 }
 
@@ -974,6 +990,9 @@ void nsColumnSetFrame::FindBestBalanceBSize(const ReflowInput& aReflowInput,
                                             ReflowOutput& aDesiredSize,
                                             bool aUnboundedLastColumn,
                                             nsReflowStatus& aStatus) {
+  MOZ_ASSERT(aConfig.mIsBalancing,
+             "Why are we here if we are not balancing columns?");
+
   const nscoord availableContentBSize = aReflowInput.AvailableBSize();
 
   // Termination of the algorithm below is guaranteed because
@@ -996,7 +1015,7 @@ void nsColumnSetFrame::FindBestBalanceBSize(const ReflowInput& aReflowInput,
   // default line-height 1140/2 = 570 app units as the minimum value. Otherwise
   // we might take more than necessary iterations before finding a feasible
   // block-size.
-  nscoord extraBlockSize = std::max(570, aReflowInput.CalcLineHeight() / 2);
+  nscoord extraBlockSize = std::max(570, aReflowInput.GetLineHeight() / 2);
 
   // We use divide-by-N to estimate the optimal column block-size only if the
   // last column's available block-size is unbounded.
@@ -1092,8 +1111,8 @@ void nsColumnSetFrame::FindBestBalanceBSize(const ReflowInput& aReflowInput,
       // extraBlockSize to try to make it on the feasible side.
       nextGuess = aColData.mSumBSize / aConfig.mUsedColCount + extraBlockSize;
       // Sanitize it
-      nextGuess = clamped(nextGuess, aConfig.mKnownInfeasibleBSize + 1,
-                          aConfig.mKnownFeasibleBSize - 1);
+      nextGuess = std::clamp(nextGuess, aConfig.mKnownInfeasibleBSize + 1,
+                             aConfig.mKnownFeasibleBSize - 1);
       // We keep doubling extraBlockSize in every iteration until we find a
       // feasible guess.
       extraBlockSize *= 2;
@@ -1126,16 +1145,9 @@ void nsColumnSetFrame::FindBestBalanceBSize(const ReflowInput& aReflowInput,
     if (!foundFeasibleBSizeCloserToBest && aColData.mFeasible) {
       foundFeasibleBSizeCloserToBest = true;
     }
-
-    if (!aConfig.mIsBalancing) {
-      // Looks like we had excess block-size when balancing, so we gave up on
-      // trying to balance.
-      break;
-    }
   }
 
-  if (aConfig.mIsBalancing && !aColData.mFeasible &&
-      !aPresContext->HasPendingInterrupt()) {
+  if (!aColData.mFeasible && !aPresContext->HasPendingInterrupt()) {
     // We need to reflow one more time at the feasible block-size to
     // get a valid layout.
     if (aConfig.mKnownInfeasibleBSize >= availableContentBSize) {
@@ -1179,6 +1191,8 @@ void nsColumnSetFrame::FindBestBalanceBSize(const ReflowInput& aReflowInput,
     // allowed to have arbitrary block-size here, even though we were
     // balancing. Otherwise we'd have to split, and it's not clear what we'd
     // do with that.
+    COLUMN_SET_LOG("%s: Last attempt to call ReflowColumns", __func__);
+    aConfig.mIsLastBalancingReflow = true;
     const bool forceUnboundedLastColumn =
         aReflowInput.mParentReflowInput->AvailableBSize() ==
         NS_UNCONSTRAINEDSIZE;
@@ -1197,7 +1211,6 @@ void nsColumnSetFrame::Reflow(nsPresContext* aPresContext,
   nsPresContext::InterruptPreventer noInterrupts(aPresContext);
 
   DO_GLOBAL_REFLOW_COUNT("nsColumnSetFrame");
-  DISPLAY_REFLOW(aPresContext, this, aReflowInput, aDesiredSize, aStatus);
   MOZ_ASSERT(aStatus.IsEmpty(), "Caller should pass a fresh reflow status!");
 
   MOZ_ASSERT(aReflowInput.mCBReflowInput->mFrame->StyleColumn()
@@ -1209,13 +1222,19 @@ void nsColumnSetFrame::Reflow(nsPresContext* aPresContext,
       aReflowInput.ComputedLogicalBorderPadding(aReflowInput.GetWritingMode())
           .IsAllZero(),
       "Only the column container can have border and padding!");
-  MOZ_ASSERT(GetChildList(kOverflowContainersList).IsEmpty() &&
-                 GetChildList(kExcessOverflowContainersList).IsEmpty(),
-             "ColumnSetFrame should store overflow containers in principal "
-             "child list!");
+  MOZ_ASSERT(
+      GetChildList(FrameChildListID::OverflowContainers).IsEmpty() &&
+          GetChildList(FrameChildListID::ExcessOverflowContainers).IsEmpty(),
+      "ColumnSetFrame should store overflow containers in principal "
+      "child list!");
 
   //------------ Handle Incremental Reflow -----------------
 
+  COLUMN_SET_LOG("%s: Begin Reflow: this=%p, is nested multicol=%d", __func__,
+                 this,
+                 aReflowInput.mParentReflowInput->mFrame->HasAnyStateBits(
+                     NS_FRAME_HAS_MULTI_COLUMN_ANCESTOR));
+
   // If inline size is unconstrained, set aForceAuto to true to allow
   // the columns to expand in the inline direction. (This typically
   // happens in orthogonal flows where the inline direction is the
@@ -1258,7 +1277,7 @@ void nsColumnSetFrame::Reflow(nsPresContext* aPresContext,
              "ColumnSetWrapperFrame should be the abs.pos container!");
   FinishAndStoreOverflow(&aDesiredSize, aReflowInput.mStyleDisplay);
 
-  NS_FRAME_SET_TRUNCATION(aStatus, aReflowInput, aDesiredSize);
+  COLUMN_SET_LOG("%s: End Reflow: this=%p", __func__, this);
 }
 
 void nsColumnSetFrame::BuildDisplayList(nsDisplayListBuilder* aBuilder,
@@ -1271,8 +1290,8 @@ void nsColumnSetFrame::BuildDisplayList(nsDisplayListBuilder* aBuilder,
   }
 
   // Our children won't have backgrounds so it doesn't matter where we put them.
-  for (nsFrameList::Enumerator e(mFrames); !e.AtEnd(); e.Next()) {
-    BuildDisplayListForChild(aBuilder, e.get(), aLists);
+  for (nsIFrame* f : mFrames) {
+    BuildDisplayListForChild(aBuilder, f, aLists);
   }
 }
 
@@ -1291,26 +1310,54 @@ void nsColumnSetFrame::AppendDirectlyOwnedAnonBoxes(
   aResult.AppendElement(OwnedAnonBox(column));
 }
 
+Maybe<nscoord> nsColumnSetFrame::GetNaturalBaselineBOffset(
+    WritingMode aWM, BaselineSharingGroup aBaselineGroup,
+    BaselineExportContext aExportContext) const {
+  Maybe<nscoord> result;
+  for (const auto* kid : mFrames) {
+    auto kidBaseline =
+        kid->GetNaturalBaselineBOffset(aWM, aBaselineGroup, aExportContext);
+    if (!kidBaseline) {
+      continue;
+    }
+    // The kid frame may not necessarily be aligned with the columnset frame.
+    LogicalRect kidRect{aWM, kid->GetLogicalNormalPosition(aWM, GetSize()),
+                        kid->GetLogicalSize(aWM)};
+    if (aBaselineGroup == BaselineSharingGroup::First) {
+      *kidBaseline += kidRect.BStart(aWM);
+    } else {
+      *kidBaseline += (GetLogicalSize().BSize(aWM) - kidRect.BEnd(aWM));
+    }
+    // Take the smallest of the baselines (i.e. Closest to border-block-start
+    // for `BaselineSharingGroup::First`, border-block-end for
+    // `BaselineSharingGroup::Last`)
+    if (!result || *kidBaseline < *result) {
+      result = kidBaseline;
+    }
+  }
+  return result;
+}
+
 #ifdef DEBUG
 void nsColumnSetFrame::SetInitialChildList(ChildListID aListID,
-                                           nsFrameList& aChildList) {
-  MOZ_ASSERT(aListID != kPrincipalList || aChildList.OnlyChild(),
+                                           nsFrameList&& aChildList) {
+  MOZ_ASSERT(aListID != FrameChildListID::Principal || aChildList.OnlyChild(),
              "initial principal child list must have exactly one child");
-  nsContainerFrame::SetInitialChildList(aListID, aChildList);
+  nsContainerFrame::SetInitialChildList(aListID, std::move(aChildList));
 }
 
 void nsColumnSetFrame::AppendFrames(ChildListID aListID,
-                                    nsFrameList& aFrameList) {
+                                    nsFrameList&& aFrameList) {
   MOZ_CRASH("unsupported operation");
 }
 
 void nsColumnSetFrame::InsertFrames(ChildListID aListID, nsIFrame* aPrevFrame,
                                     const nsLineList::iterator* aPrevFrameLine,
-                                    nsFrameList& aFrameList) {
+                                    nsFrameList&& aFrameList) {
   MOZ_CRASH("unsupported operation");
 }
 
-void nsColumnSetFrame::RemoveFrame(ChildListID aListID, nsIFrame* aOldFrame) {
+void nsColumnSetFrame::RemoveFrame(DestroyContext&, ChildListID, nsIFrame*) {
   MOZ_CRASH("unsupported operation");
 }
 #endif
