@@ -89,7 +89,7 @@ class nsHTMLFramesetBorderFrame final : public nsLeafFrame {
                                WidgetGUIEvent* aEvent,
                                nsEventStatus* aEventStatus) override;
 
-  Maybe<Cursor> GetCursor(const nsPoint&) override;
+  Cursor GetCursor(const nsPoint&) override;
 
   virtual void BuildDisplayList(nsDisplayListBuilder* aBuilder,
                                 const nsDisplayListSet& aLists) override;
@@ -108,8 +108,6 @@ class nsHTMLFramesetBorderFrame final : public nsLeafFrame {
   nsHTMLFramesetBorderFrame(ComputedStyle*, nsPresContext*, int32_t aWidth,
                             bool aVertical, bool aVisible);
   virtual ~nsHTMLFramesetBorderFrame();
-  virtual nscoord GetIntrinsicISize() override;
-  virtual nscoord GetIntrinsicBSize() override;
 
   // the prev and next neighbors are indexes into the row (for a horizontal
   // border) or col (for a vertical border) of nsHTMLFramesetFrames or
@@ -150,8 +148,6 @@ class nsHTMLFramesetBlankFrame final : public nsLeafFrame {
       : nsLeafFrame(aStyle, aPresContext, kClassID) {}
 
   virtual ~nsHTMLFramesetBlankFrame();
-  virtual nscoord GetIntrinsicISize() override;
-  virtual nscoord GetIntrinsicBSize() override;
 
   friend class nsHTMLFramesetFrame;
   friend class nsHTMLFrameset;
@@ -238,12 +234,16 @@ void nsHTMLFramesetFrame::Init(nsIContent* aContent, nsContainerFrame* aParent,
   mVerBorders = MakeUnique<nsHTMLFramesetBorderFrame*[]>(
       mNumCols);  // 1 more than number of ver borders
 
-  for (int verX = 0; verX < mNumCols; verX++) mVerBorders[verX] = nullptr;
+  for (int verX = 0; verX < mNumCols; verX++) {
+    mVerBorders[verX] = nullptr;
+  }
 
   mHorBorders = MakeUnique<nsHTMLFramesetBorderFrame*[]>(
       mNumRows);  // 1 more than number of hor borders
 
-  for (int horX = 0; horX < mNumRows; horX++) mHorBorders[horX] = nullptr;
+  for (int horX = 0; horX < mNumRows; horX++) {
+    mHorBorders[horX] = nullptr;
+  }
 
   static_assert(NS_MAX_FRAMESET_SPEC_COUNT <
                     UINT_MAX / sizeof(int32_t) / NS_MAX_FRAMESET_SPEC_COUNT,
@@ -336,16 +336,16 @@ void nsHTMLFramesetFrame::Init(nsIContent* aContent, nsContainerFrame* aParent,
 }
 
 void nsHTMLFramesetFrame::SetInitialChildList(ChildListID aListID,
-                                              nsFrameList& aChildList) {
+                                              nsFrameList&& aChildList) {
   // We do this weirdness where we create our child frames in Init().  On the
   // other hand, we're going to get a SetInitialChildList() with an empty list
   // and null list name after the frame constructor is done creating us.  So
   // just ignore that call.
-  if (aListID == kPrincipalList && aChildList.IsEmpty()) {
+  if (aListID == FrameChildListID::Principal && aChildList.IsEmpty()) {
     return;
   }
 
-  nsContainerFrame::SetInitialChildList(aListID, aChildList);
+  nsContainerFrame::SetInitialChildList(aListID, std::move(aChildList));
 }
 
 // XXX should this try to allocate twips based on an even pixel boundary?
@@ -498,7 +498,9 @@ void nsHTMLFramesetFrame::GenerateRowCol(nsPresContext* aPresContext,
   int32_t i;
 
   for (i = 0; i < aNumSpecs; i++) {
-    if (!aNewAttr.IsEmpty()) aNewAttr.Append(char16_t(','));
+    if (!aNewAttr.IsEmpty()) {
+      aNewAttr.Append(char16_t(','));
+    }
 
     switch (aSpecs[i].mUnit) {
       case eFramesetUnit_Fixed:
@@ -630,13 +632,13 @@ nsresult nsHTMLFramesetFrame::HandleEvent(nsPresContext* aPresContext,
   return NS_OK;
 }
 
-Maybe<nsIFrame::Cursor> nsHTMLFramesetFrame::GetCursor(const nsPoint&) {
+nsIFrame::Cursor nsHTMLFramesetFrame::GetCursor(const nsPoint&) {
   auto kind = StyleCursorKind::Default;
   if (mDragger) {
     kind = mDragger->mVertical ? StyleCursorKind::EwResize
                                : StyleCursorKind::NsResize;
   }
-  return Some(Cursor{kind, AllowCustomCursorImage::No});
+  return Cursor{kind, AllowCustomCursorImage::No};
 }
 
 void nsHTMLFramesetFrame::BuildDisplayList(nsDisplayListBuilder* aBuilder,
@@ -682,13 +684,13 @@ static nsFrameborder GetFrameBorderHelper(nsGenericHTMLElement* aContent) {
   if (nullptr != aContent) {
     const nsAttrValue* attr = aContent->GetParsedAttr(nsGkAtoms::frameborder);
     if (attr && attr->Type() == nsAttrValue::eEnum) {
-      switch (attr->GetEnumValue()) {
-        case NS_STYLE_FRAME_YES:
-        case NS_STYLE_FRAME_1:
+      switch (static_cast<FrameBorderProperty>(attr->GetEnumValue())) {
+        case FrameBorderProperty::Yes:
+        case FrameBorderProperty::One:
           return eFrameborder_Yes;
 
-        case NS_STYLE_FRAME_NO:
-        case NS_STYLE_FRAME_0:
+        case FrameBorderProperty::No:
+        case FrameBorderProperty::Zero:
           return eFrameborder_No;
       }
     }
@@ -760,7 +762,6 @@ void nsHTMLFramesetFrame::Reflow(nsPresContext* aPresContext,
                                  nsReflowStatus& aStatus) {
   MarkInReflow();
   DO_GLOBAL_REFLOW_COUNT("nsHTMLFramesetFrame");
-  DISPLAY_REFLOW(aPresContext, this, aReflowInput, aDesiredSize, aStatus);
   MOZ_ASSERT(aStatus.IsEmpty(), "Caller should pass a fresh reflow status!");
 
   mozilla::PresShell* presShell = aPresContext->PresShell();
@@ -794,10 +795,14 @@ void nsHTMLFramesetFrame::Reflow(nsPresContext* aPresContext,
   int32_t borderWidth = GetBorderWidth(aPresContext, true);
 
   width -= (mNumCols - 1) * borderWidth;
-  if (width < 0) width = 0;
+  if (width < 0) {
+    width = 0;
+  }
 
   height -= (mNumRows - 1) * borderWidth;
-  if (height < 0) height = 0;
+  if (height < 0) {
+    height = 0;
+  }
 
   HTMLFrameSetElement* ourContent = HTMLFrameSetElement::FromNode(mContent);
   NS_ASSERTION(ourContent, "Someone gave us a broken frameset element!");
@@ -811,7 +816,6 @@ void nsHTMLFramesetFrame::Reflow(nsPresContext* aPresContext,
   // will be re-created.
   if (mNumRows != rows || mNumCols != cols) {
     mDrag.UnSet();
-    NS_FRAME_SET_TRUNCATION(aStatus, aReflowInput, aDesiredSize);
     return;
   }
 
@@ -1050,8 +1054,6 @@ void nsHTMLFramesetFrame::Reflow(nsPresContext* aPresContext,
 
   aDesiredSize.SetOverflowAreasToDesiredBounds();
   FinishAndStoreOverflow(&aDesiredSize);
-
-  NS_FRAME_SET_TRUNCATION(aStatus, aReflowInput, aDesiredSize);
 }
 
 #ifdef DEBUG_FRAME_DUMP
@@ -1086,7 +1088,7 @@ bool nsHTMLFramesetFrame::GetNoResize(nsIFrame* aChildFrame) {
   nsIContent* content = aChildFrame->GetContent();
 
   return content && content->IsElement() &&
-         content->AsElement()->HasAttr(kNameSpaceID_None, nsGkAtoms::noresize);
+         content->AsElement()->HasAttr(nsGkAtoms::noresize);
 }
 
 bool nsHTMLFramesetFrame::CanChildResize(bool aVertical, bool aLeft,
@@ -1294,16 +1296,6 @@ nsHTMLFramesetBorderFrame::~nsHTMLFramesetBorderFrame() {
 
 NS_IMPL_FRAMEARENA_HELPERS(nsHTMLFramesetBorderFrame)
 
-nscoord nsHTMLFramesetBorderFrame::GetIntrinsicISize() {
-  // No intrinsic width
-  return 0;
-}
-
-nscoord nsHTMLFramesetBorderFrame::GetIntrinsicBSize() {
-  // No intrinsic height
-  return 0;
-}
-
 void nsHTMLFramesetBorderFrame::SetVisibility(bool aVisibility) {
   mVisibility = aVisibility;
 }
@@ -1315,24 +1307,25 @@ void nsHTMLFramesetBorderFrame::Reflow(nsPresContext* aPresContext,
                                        const ReflowInput& aReflowInput,
                                        nsReflowStatus& aStatus) {
   DO_GLOBAL_REFLOW_COUNT("nsHTMLFramesetBorderFrame");
-  DISPLAY_REFLOW(aPresContext, this, aReflowInput, aDesiredSize, aStatus);
   MOZ_ASSERT(aStatus.IsEmpty(), "Caller should pass a fresh reflow status!");
 
   // Override Reflow(), since we don't want to deal with what our
-  // computed values are.
-  SizeToAvailSize(aReflowInput, aDesiredSize);
-
+  // computed values are. We just size to our available size.
+  aDesiredSize.SetSize(aReflowInput.GetWritingMode(),
+                       aReflowInput.AvailableSize());
   aDesiredSize.SetOverflowAreasToDesiredBounds();
+  FinishAndStoreOverflow(&aDesiredSize, aReflowInput.mStyleDisplay);
 }
 
-class nsDisplayFramesetBorder : public nsPaintedDisplayItem {
+class nsDisplayFramesetBorder final : public nsPaintedDisplayItem {
  public:
   nsDisplayFramesetBorder(nsDisplayListBuilder* aBuilder,
                           nsHTMLFramesetBorderFrame* aFrame)
       : nsPaintedDisplayItem(aBuilder, aFrame) {
     MOZ_COUNT_CTOR(nsDisplayFramesetBorder);
   }
-  MOZ_COUNTED_DTOR_OVERRIDE(nsDisplayFramesetBorder)
+
+  MOZ_COUNTED_DTOR_FINAL(nsDisplayFramesetBorder)
 
   // REVIEW: see old GetFrameForPoint
   // Receives events in its bounds
@@ -1361,19 +1354,21 @@ void nsHTMLFramesetBorderFrame::PaintBorder(DrawTarget* aDrawTarget,
   nscoord widthInPixels = nsPresContext::AppUnitsToIntCSSPixels(mWidth);
   nscoord pixelWidth = nsPresContext::CSSPixelsToAppUnits(1);
 
-  if (widthInPixels <= 0) return;
+  if (widthInPixels <= 0) {
+    return;
+  }
 
   ColorPattern bgColor(ToDeviceColor(LookAndFeel::Color(
-      LookAndFeel::ColorID::WidgetBackground, this, NS_RGB(200, 200, 200))));
+      LookAndFeel::ColorID::Window, this, NS_RGB(200, 200, 200))));
 
   ColorPattern fgColor(ToDeviceColor(LookAndFeel::Color(
-      LookAndFeel::ColorID::WidgetForeground, this, NS_RGB(0, 0, 0))));
+      LookAndFeel::ColorID::Windowtext, this, NS_RGB(0, 0, 0))));
 
   ColorPattern hltColor(ToDeviceColor(LookAndFeel::Color(
-      LookAndFeel::ColorID::Widget3DHighlight, this, NS_RGB(255, 255, 255))));
+      LookAndFeel::ColorID::Threedhighlight, this, NS_RGB(255, 255, 255))));
 
   ColorPattern sdwColor(ToDeviceColor(LookAndFeel::Color(
-      LookAndFeel::ColorID::Widget3DShadow, this, NS_RGB(128, 128, 128))));
+      LookAndFeel::ColorID::Threedshadow, this, NS_RGB(128, 128, 128))));
 
   ColorPattern color(ToDeviceColor(NS_RGB(255, 255, 255)));  // default to white
   if (mVisibility) {
@@ -1405,7 +1400,9 @@ void nsHTMLFramesetBorderFrame::PaintBorder(DrawTarget* aDrawTarget,
     }
   }
 
-  if (!mVisibility) return;
+  if (!mVisibility) {
+    return;
+  }
 
   if (widthInPixels >= 5) {
     start.x = (mVertical) ? pixelWidth : 0;
@@ -1457,12 +1454,12 @@ nsresult nsHTMLFramesetBorderFrame::HandleEvent(nsPresContext* aPresContext,
   return NS_OK;
 }
 
-Maybe<nsIFrame::Cursor> nsHTMLFramesetBorderFrame::GetCursor(const nsPoint&) {
+nsIFrame::Cursor nsHTMLFramesetBorderFrame::GetCursor(const nsPoint&) {
   auto kind = StyleCursorKind::Default;
   if (mCanResize) {
     kind = mVertical ? StyleCursorKind::EwResize : StyleCursorKind::NsResize;
   }
-  return Some(Cursor{kind, AllowCustomCursorImage::No});
+  return Cursor{kind, AllowCustomCursorImage::No};
 }
 
 #ifdef DEBUG_FRAME_DUMP
@@ -1485,16 +1482,6 @@ nsHTMLFramesetBlankFrame::~nsHTMLFramesetBlankFrame() {
   // printf("nsHTMLFramesetBlankFrame destructor %p \n", this);
 }
 
-nscoord nsHTMLFramesetBlankFrame::GetIntrinsicISize() {
-  // No intrinsic width
-  return 0;
-}
-
-nscoord nsHTMLFramesetBlankFrame::GetIntrinsicBSize() {
-  // No intrinsic height
-  return 0;
-}
-
 void nsHTMLFramesetBlankFrame::Reflow(nsPresContext* aPresContext,
                                       ReflowOutput& aDesiredSize,
                                       const ReflowInput& aReflowInput,
@@ -1503,19 +1490,21 @@ void nsHTMLFramesetBlankFrame::Reflow(nsPresContext* aPresContext,
   MOZ_ASSERT(aStatus.IsEmpty(), "Caller should pass a fresh reflow status!");
 
   // Override Reflow(), since we don't want to deal with what our
-  // computed values are.
-  SizeToAvailSize(aReflowInput, aDesiredSize);
-
+  // computed values are. We just size to our available size.
+  aDesiredSize.SetSize(aReflowInput.GetWritingMode(),
+                       aReflowInput.AvailableSize());
   aDesiredSize.SetOverflowAreasToDesiredBounds();
+  FinishAndStoreOverflow(&aDesiredSize, aReflowInput.mStyleDisplay);
 }
 
-class nsDisplayFramesetBlank : public nsPaintedDisplayItem {
+class nsDisplayFramesetBlank final : public nsPaintedDisplayItem {
  public:
   nsDisplayFramesetBlank(nsDisplayListBuilder* aBuilder, nsIFrame* aFrame)
       : nsPaintedDisplayItem(aBuilder, aFrame) {
     MOZ_COUNT_CTOR(nsDisplayFramesetBlank);
   }
-  MOZ_COUNTED_DTOR_OVERRIDE(nsDisplayFramesetBlank)
+
+  MOZ_COUNTED_DTOR_FINAL(nsDisplayFramesetBlank)
 
   virtual void Paint(nsDisplayListBuilder* aBuilder, gfxContext* aCtx) override;
   NS_DISPLAY_DECL_NAME("FramesetBlank", TYPE_FRAMESET_BLANK)
@@ -1525,8 +1514,8 @@ void nsDisplayFramesetBlank::Paint(nsDisplayListBuilder* aBuilder,
                                    gfxContext* aCtx) {
   DrawTarget* drawTarget = aCtx->GetDrawTarget();
   int32_t appUnitsPerDevPixel = mFrame->PresContext()->AppUnitsPerDevPixel();
-  Rect rect =
-      NSRectToSnappedRect(GetPaintRect(), appUnitsPerDevPixel, *drawTarget);
+  Rect rect = NSRectToSnappedRect(GetPaintRect(aBuilder, aCtx),
+                                  appUnitsPerDevPixel, *drawTarget);
   ColorPattern white(ToDeviceColor(sRGBColor::OpaqueWhite()));
   drawTarget->FillRect(rect, white);
 }
