@@ -46,38 +46,41 @@ NS_QUERYFRAME_TAIL_INHERITING(nsIFrame)
 #endif
 
 /* virtual */
-void nsPlaceholderFrame::AddInlineMinISize(
-    gfxContext* aRenderingContext, nsIFrame::InlineMinISizeData* aData) {
-  // Override AddInlineMinWith so that *nothing* happens.  In
+void nsPlaceholderFrame::AddInlineMinISize(const IntrinsicSizeInput& aInput,
+                                           InlineMinISizeData* aData) {
+  // Override AddInlineMinISize so that *nothing* happens. In
   // particular, we don't want to zero out |aData->mTrailingWhitespace|,
   // since nsLineLayout skips placeholders when trimming trailing
   // whitespace, and we don't want to set aData->mSkipWhitespace to
   // false.
 
-  // ...but push floats onto the list
-  if (mOutOfFlowFrame->IsFloating()) {
-    nscoord floatWidth = nsLayoutUtils::IntrinsicForContainer(
-        aRenderingContext, mOutOfFlowFrame, IntrinsicISizeType::MinISize);
-    aData->mFloats.AppendElement(
-        InlineIntrinsicISizeData::FloatInfo(mOutOfFlowFrame, floatWidth));
-  }
+  // ...but push floats onto aData's list.
+  AddFloatToIntrinsicISizeData(aInput, IntrinsicISizeType::MinISize, aData);
 }
 
 /* virtual */
-void nsPlaceholderFrame::AddInlinePrefISize(
-    gfxContext* aRenderingContext, nsIFrame::InlinePrefISizeData* aData) {
-  // Override AddInlinePrefWith so that *nothing* happens.  In
+void nsPlaceholderFrame::AddInlinePrefISize(const IntrinsicSizeInput& aInput,
+                                            InlinePrefISizeData* aData) {
+  // Override AddInlinePrefISize so that *nothing* happens. In
   // particular, we don't want to zero out |aData->mTrailingWhitespace|,
   // since nsLineLayout skips placeholders when trimming trailing
   // whitespace, and we don't want to set aData->mSkipWhitespace to
   // false.
 
-  // ...but push floats onto the list
+  // ...but push floats onto aData's list.
+  AddFloatToIntrinsicISizeData(aInput, IntrinsicISizeType::PrefISize, aData);
+}
+
+void nsPlaceholderFrame::AddFloatToIntrinsicISizeData(
+    const IntrinsicSizeInput& aInput, IntrinsicISizeType aType,
+    InlineIntrinsicISizeData* aData) const {
   if (mOutOfFlowFrame->IsFloating()) {
-    nscoord floatWidth = nsLayoutUtils::IntrinsicForContainer(
-        aRenderingContext, mOutOfFlowFrame, IntrinsicISizeType::PrefISize);
-    aData->mFloats.AppendElement(
-        InlineIntrinsicISizeData::FloatInfo(mOutOfFlowFrame, floatWidth));
+    const IntrinsicSizeInput floatInput(
+        aInput, mOutOfFlowFrame->GetWritingMode(), GetWritingMode());
+    const nscoord floatISize = nsLayoutUtils::IntrinsicForContainer(
+        floatInput.mContext, mOutOfFlowFrame, aType,
+        floatInput.mPercentageBasisForChildren);
+    aData->mFloats.EmplaceBack(mOutOfFlowFrame, floatISize);
   }
 }
 
@@ -98,9 +101,15 @@ void nsPlaceholderFrame::Reflow(nsPresContext* aPresContext,
   // the placeholder, so they don't have this requirement (and this condition
   // doesn't hold anyways because the default popupgroup goes before than the
   // default tooltip, for example).
+  //
+  // We also have an exception if the out-of-flow created an orthogonal flow,
+  // because in this case we may have needed to do a measuring reflow during
+  // intrinsic size computation. That's OK because it does not depend on the
+  // placeholder being reflowed first.
   if (HasAnyStateBits(NS_FRAME_FIRST_REFLOW) &&
       !mOutOfFlowFrame->IsMenuPopupFrame() &&
-      !mOutOfFlowFrame->HasAnyStateBits(NS_FRAME_FIRST_REFLOW)) {
+      !mOutOfFlowFrame->HasAnyStateBits(NS_FRAME_FIRST_REFLOW) &&
+      !mOutOfFlowFrame->GetWritingMode().IsOrthogonalTo(GetWritingMode())) {
     // Unfortunately, this can currently happen when the placeholder is in a
     // later continuation or later IB-split sibling than its out-of-flow (as
     // is the case in some of our existing unit tests). So for now, in that
@@ -124,7 +133,6 @@ void nsPlaceholderFrame::Reflow(nsPresContext* aPresContext,
 
   MarkInReflow();
   DO_GLOBAL_REFLOW_COUNT("nsPlaceholderFrame");
-  DISPLAY_REFLOW(aPresContext, this, aReflowInput, aDesiredSize, aStatus);
   MOZ_ASSERT(aStatus.IsEmpty(), "Caller should pass a fresh reflow status!");
   aDesiredSize.ClearSize();
 }
@@ -142,7 +150,7 @@ static FrameChildListID ChildListIDForOutOfFlow(nsFrameState aPlaceholderState,
   if (aPlaceholderState & PLACEHOLDER_FOR_ABSPOS) {
     return FrameChildListID::Absolute;
   }
-  MOZ_DIAGNOSTIC_ASSERT(false, "unknown list");
+  MOZ_DIAGNOSTIC_CRASH("unknown list");
   return FrameChildListID::Float;
 }
 
@@ -221,7 +229,8 @@ void nsPlaceholderFrame::List(FILE* out, const char* aPrefix,
 
   if (mOutOfFlowFrame) {
     str += " outOfFlowFrame=";
-    str += mOutOfFlowFrame->ListTag();
+    str += mOutOfFlowFrame->ListTag(
+        aFlags.contains(ListFlag::OnlyListDeterministicInfo));
   }
   fprintf_stderr(out, "%s\n", str.get());
 }
