@@ -14,6 +14,7 @@
 #include "mozilla/PresShell.h"
 #include "mozilla/StaticPrefs_layout.h"
 #include "mozilla/WritingModes.h"
+#include "nsLayoutUtils.h"
 #include "nsLineLayout.h"
 #include "nsPresContext.h"
 #include "nsContainerFrameInlines.h"
@@ -43,14 +44,6 @@ nsContainerFrame* NS_NewRubyFrame(PresShell* aPresShell,
 // nsRubyFrame Method Implementations
 // ==================================
 
-/* virtual */
-bool nsRubyFrame::IsFrameOfType(uint32_t aFlags) const {
-  if (aFlags & eBidiInlineContainer) {
-    return false;
-  }
-  return nsInlineFrame::IsFrameOfType(aFlags);
-}
-
 #ifdef DEBUG_FRAME_DUMP
 nsresult nsRubyFrame::GetFrameName(nsAString& aResult) const {
   return MakeFrameName(u"Ruby"_ns, aResult);
@@ -58,24 +51,30 @@ nsresult nsRubyFrame::GetFrameName(nsAString& aResult) const {
 #endif
 
 /* virtual */
-void nsRubyFrame::AddInlineMinISize(gfxContext* aRenderingContext,
-                                    nsIFrame::InlineMinISizeData* aData) {
-  auto handleChildren = [aRenderingContext](auto frame, auto data) {
+void nsRubyFrame::AddInlineMinISize(const IntrinsicSizeInput& aInput,
+                                    InlineMinISizeData* aData) {
+  auto handleChildren = [&](auto frame, auto data) {
+    // Ruby frames shouldn't have percentage block sizes that require a
+    // percentage basis for resolution.
+    const IntrinsicSizeInput input(aInput.mContext, Nothing(), Nothing());
     for (RubySegmentEnumerator e(static_cast<nsRubyFrame*>(frame)); !e.AtEnd();
          e.Next()) {
-      e.GetBaseContainer()->AddInlineMinISize(aRenderingContext, data);
+      e.GetBaseContainer()->AddInlineMinISize(input, data);
     }
   };
   DoInlineIntrinsicISize(aData, handleChildren);
 }
 
 /* virtual */
-void nsRubyFrame::AddInlinePrefISize(gfxContext* aRenderingContext,
-                                     nsIFrame::InlinePrefISizeData* aData) {
-  auto handleChildren = [aRenderingContext](auto frame, auto data) {
+void nsRubyFrame::AddInlinePrefISize(const IntrinsicSizeInput& aInput,
+                                     InlinePrefISizeData* aData) {
+  auto handleChildren = [&](auto frame, auto data) {
+    // Ruby frames shouldn't have percentage block sizes that require a
+    // percentage basis for resolution.
+    const IntrinsicSizeInput input(aInput.mContext, Nothing(), Nothing());
     for (RubySegmentEnumerator e(static_cast<nsRubyFrame*>(frame)); !e.AtEnd();
          e.Next()) {
-      e.GetBaseContainer()->AddInlinePrefISize(aRenderingContext, data);
+      e.GetBaseContainer()->AddInlinePrefISize(input, data);
     }
   };
   DoInlineIntrinsicISize(aData, handleChildren);
@@ -85,7 +84,7 @@ void nsRubyFrame::AddInlinePrefISize(gfxContext* aRenderingContext,
 static nsRubyBaseContainerFrame* FindRubyBaseContainerAncestor(
     nsIFrame* aFrame) {
   for (nsIFrame* ancestor = aFrame->GetParent();
-       ancestor && ancestor->IsFrameOfType(nsIFrame::eLineParticipant);
+       ancestor && ancestor->IsLineParticipant();
        ancestor = ancestor->GetParent()) {
     if (ancestor->IsRubyBaseContainerFrame()) {
       return static_cast<nsRubyBaseContainerFrame*>(ancestor);
@@ -101,7 +100,6 @@ void nsRubyFrame::Reflow(nsPresContext* aPresContext,
                          nsReflowStatus& aStatus) {
   MarkInReflow();
   DO_GLOBAL_REFLOW_COUNT("nsRubyFrame");
-  DISPLAY_REFLOW(aPresContext, this, aReflowInput, aDesiredSize, aStatus);
   MOZ_ASSERT(aStatus.IsEmpty(), "Caller should pass a fresh reflow status!");
 
   if (!aReflowInput.mLineLayout) {
@@ -253,15 +251,15 @@ void nsRubyFrame::ReflowSegment(nsPresContext* aPresContext,
       PushChildrenToOverflow(lastChild->GetNextSibling(), lastChild);
       aReflowInput.mLineLayout->SetDirtyNextLine();
     }
-  } else {
+  } else if (rtcCount) {
+    DestroyContext context(PresShell());
     // If the ruby base container is reflowed completely, the line
     // layout will remove the next-in-flows of that frame. But the
     // line layout is not aware of the ruby text containers, hence
     // it is necessary to remove them here.
     for (uint32_t i = 0; i < rtcCount; i++) {
-      nsIFrame* nextRTC = textContainers[i]->GetNextInFlow();
-      if (nextRTC) {
-        nextRTC->GetParent()->DeleteNextInFlowChild(nextRTC, true);
+      if (nsIFrame* nextRTC = textContainers[i]->GetNextInFlow()) {
+        nextRTC->GetParent()->DeleteNextInFlowChild(context, nextRTC, true);
       }
     }
   }
@@ -308,25 +306,25 @@ void nsRubyFrame::ReflowSegment(nsPresContext* aPresContext,
     Maybe<LineRelativeDir> lineSide;
     switch (textContainer->StyleText()->mRubyPosition) {
       case StyleRubyPosition::Over:
-        lineSide.emplace(eLineRelativeDirOver);
+        lineSide.emplace(LineRelativeDir::Over);
         break;
       case StyleRubyPosition::Under:
-        lineSide.emplace(eLineRelativeDirUnder);
+        lineSide.emplace(LineRelativeDir::Under);
         break;
       case StyleRubyPosition::AlternateOver:
         if (lastLineSide.isSome() &&
-            lastLineSide.value() == eLineRelativeDirOver) {
-          lineSide.emplace(eLineRelativeDirUnder);
+            lastLineSide.value() == LineRelativeDir::Over) {
+          lineSide.emplace(LineRelativeDir::Under);
         } else {
-          lineSide.emplace(eLineRelativeDirOver);
+          lineSide.emplace(LineRelativeDir::Over);
         }
         break;
       case StyleRubyPosition::AlternateUnder:
         if (lastLineSide.isSome() &&
-            lastLineSide.value() == eLineRelativeDirUnder) {
-          lineSide.emplace(eLineRelativeDirOver);
+            lastLineSide.value() == LineRelativeDir::Under) {
+          lineSide.emplace(LineRelativeDir::Over);
         } else {
-          lineSide.emplace(eLineRelativeDirUnder);
+          lineSide.emplace(LineRelativeDir::Under);
         }
         break;
       default:
@@ -341,7 +339,7 @@ void nsRubyFrame::ReflowSegment(nsPresContext* aPresContext,
           lineWM.LogicalSideForLineRelativeDir(lineSide.value());
       if (StaticPrefs::layout_css_ruby_intercharacter_enabled() &&
           rtcWM.IsVerticalRL() &&
-          lineWM.GetInlineDir() == WritingMode::eInlineLTR) {
+          lineWM.GetInlineDir() == WritingMode::InlineDir::LTR) {
         // Inter-character ruby annotations are only supported for vertical-rl
         // in ltr horizontal writing. Fall back to non-inter-character behavior
         // otherwise.
@@ -352,11 +350,11 @@ void nsRubyFrame::ReflowSegment(nsPresContext* aPresContext,
                 : 0);
         position = offsetRect.Origin(lineWM) + offset;
         aReflowInput.mLineLayout->AdvanceICoord(size.ISize(lineWM));
-      } else if (logicalSide == eLogicalSideBStart) {
+      } else if (logicalSide == LogicalSide::BStart) {
         offsetRect.BStart(lineWM) -= size.BSize(lineWM);
         offsetRect.BSize(lineWM) += size.BSize(lineWM);
         position = offsetRect.Origin(lineWM);
-      } else if (logicalSide == eLogicalSideBEnd) {
+      } else if (logicalSide == LogicalSide::BEnd) {
         position = offsetRect.Origin(lineWM) +
                    LogicalPoint(lineWM, 0, offsetRect.BSize(lineWM));
         offsetRect.BSize(lineWM) += size.BSize(lineWM);
