@@ -20,8 +20,6 @@
 #include "nsGenericHTMLElement.h"
 #include "nsPresContext.h"
 #include "nsContentCreatorFunctions.h"
-#include "nsBoxLayoutState.h"
-#include "nsBoxFrame.h"
 #include "nsIContentInlines.h"
 #include "nsImageFrame.h"
 #include "nsIImageLoadingContent.h"
@@ -40,15 +38,28 @@ nsIFrame* NS_NewHTMLVideoFrame(PresShell* aPresShell, ComputedStyle* aStyle) {
   return new (aPresShell) nsVideoFrame(aStyle, aPresShell->GetPresContext());
 }
 
+nsIFrame* NS_NewHTMLAudioFrame(PresShell* aPresShell, ComputedStyle* aStyle) {
+  return new (aPresShell) nsAudioFrame(aStyle, aPresShell->GetPresContext());
+}
+
 NS_IMPL_FRAMEARENA_HELPERS(nsVideoFrame)
+NS_QUERYFRAME_HEAD(nsVideoFrame)
+  NS_QUERYFRAME_ENTRY(nsVideoFrame)
+  NS_QUERYFRAME_ENTRY(nsIAnonymousContentCreator)
+NS_QUERYFRAME_TAIL_INHERITING(nsContainerFrame)
+
+NS_IMPL_FRAMEARENA_HELPERS(nsAudioFrame)
+NS_QUERYFRAME_HEAD(nsAudioFrame)
+  NS_QUERYFRAME_ENTRY(nsAudioFrame)
+NS_QUERYFRAME_TAIL_INHERITING(nsVideoFrame)
 
 // A matrix to obtain a correct-rotated video frame.
 static Matrix ComputeRotationMatrix(gfxFloat aRotatedWidth,
                                     gfxFloat aRotatedHeight,
-                                    VideoInfo::Rotation aDegrees) {
+                                    VideoRotation aDegrees) {
   Matrix shiftVideoCenterToOrigin;
-  if (aDegrees == VideoInfo::Rotation::kDegree_90 ||
-      aDegrees == VideoInfo::Rotation::kDegree_270) {
+  if (aDegrees == VideoRotation::kDegree_90 ||
+      aDegrees == VideoRotation::kDegree_270) {
     shiftVideoCenterToOrigin =
         Matrix::Translation(-aRotatedHeight / 2.0, -aRotatedWidth / 2.0);
   } else {
@@ -56,33 +67,36 @@ static Matrix ComputeRotationMatrix(gfxFloat aRotatedWidth,
         Matrix::Translation(-aRotatedWidth / 2.0, -aRotatedHeight / 2.0);
   }
 
-  Matrix rotation = Matrix::Rotation(gfx::Float(aDegrees / 180.0 * M_PI));
+  auto angle = static_cast<double>(aDegrees) / 180.0 * M_PI;
+  Matrix rotation = Matrix::Rotation(static_cast<gfx::Float>(angle));
   Matrix shiftLeftTopToOrigin =
       Matrix::Translation(aRotatedWidth / 2.0, aRotatedHeight / 2.0);
   return shiftVideoCenterToOrigin * rotation * shiftLeftTopToOrigin;
 }
 
 static void SwapScaleWidthHeightForRotation(IntSize& aSize,
-                                            VideoInfo::Rotation aDegrees) {
-  if (aDegrees == VideoInfo::Rotation::kDegree_90 ||
-      aDegrees == VideoInfo::Rotation::kDegree_270) {
+                                            VideoRotation aDegrees) {
+  if (aDegrees == VideoRotation::kDegree_90 ||
+      aDegrees == VideoRotation::kDegree_270) {
     int32_t tmpWidth = aSize.width;
     aSize.width = aSize.height;
     aSize.height = tmpWidth;
   }
 }
 
-nsVideoFrame::nsVideoFrame(ComputedStyle* aStyle, nsPresContext* aPresContext)
-    : nsContainerFrame(aStyle, aPresContext, kClassID) {
+nsVideoFrame::nsVideoFrame(ComputedStyle* aStyle, nsPresContext* aPc,
+                           ClassID aClassID)
+    : nsContainerFrame(aStyle, aPc, aClassID),
+      mIsAudio(aClassID == nsAudioFrame::kClassID) {
   EnableVisibilityTracking();
 }
 
 nsVideoFrame::~nsVideoFrame() = default;
 
-NS_QUERYFRAME_HEAD(nsVideoFrame)
-  NS_QUERYFRAME_ENTRY(nsVideoFrame)
-  NS_QUERYFRAME_ENTRY(nsIAnonymousContentCreator)
-NS_QUERYFRAME_TAIL_INHERITING(nsContainerFrame)
+nsAudioFrame::nsAudioFrame(ComputedStyle* aStyle, nsPresContext* aPc)
+    : nsVideoFrame(aStyle, aPc, kClassID) {}
+
+nsAudioFrame::~nsAudioFrame() = default;
 
 nsresult nsVideoFrame::CreateAnonymousContent(
     nsTArray<ContentInfo>& aElements) {
@@ -99,18 +113,6 @@ nsresult nsVideoFrame::CreateAnonymousContent(
     NS_ENSURE_TRUE(nodeInfo, NS_ERROR_OUT_OF_MEMORY);
     mPosterImage = NS_NewHTMLImageElement(nodeInfo.forget());
     NS_ENSURE_TRUE(mPosterImage, NS_ERROR_OUT_OF_MEMORY);
-
-    // Set the nsImageLoadingContent::ImageState() to 0. This means that the
-    // image will always report its state as 0, so it will never be reframed
-    // to show frames for loading or the broken image icon. This is important,
-    // as the image is native anonymous, and so can't be reframed (currently).
-    HTMLImageElement* imgContent = HTMLImageElement::FromNode(mPosterImage);
-    NS_ENSURE_TRUE(imgContent, NS_ERROR_FAILURE);
-
-    imgContent->ForceImageState(true, 0);
-    // And now have it update its internal state
-    mPosterImage->UpdateState(false);
-
     UpdatePosterSource(false);
 
     // XXX(Bug 1631371) Check if this should use a fallible operation as it
@@ -160,14 +162,13 @@ nsIContent* nsVideoFrame::GetVideoControls() const {
   return mContent->GetShadowRoot()->GetFirstChild();
 }
 
-void nsVideoFrame::DestroyFrom(nsIFrame* aDestructRoot,
-                               PostDestroyData& aPostDestroyData) {
+void nsVideoFrame::Destroy(DestroyContext& aContext) {
   if (mReflowCallbackPosted) {
     PresShell()->CancelReflowCallback(this);
   }
-  aPostDestroyData.AddAnonymousContent(mCaptionDiv.forget());
-  aPostDestroyData.AddAnonymousContent(mPosterImage.forget());
-  nsContainerFrame::DestroyFrom(aDestructRoot, aPostDestroyData);
+  aContext.AddAnonymousContent(mCaptionDiv.forget());
+  aContext.AddAnonymousContent(mPosterImage.forget());
+  nsContainerFrame::Destroy(aContext);
 }
 
 class DispatchResizeEvent : public Runnable {
@@ -225,14 +226,13 @@ void nsVideoFrame::Reflow(nsPresContext* aPresContext, ReflowOutput& aMetrics,
                           nsReflowStatus& aStatus) {
   MarkInReflow();
   DO_GLOBAL_REFLOW_COUNT("nsVideoFrame");
-  DISPLAY_REFLOW(aPresContext, this, aReflowInput, aMetrics, aStatus);
   MOZ_ASSERT(aStatus.IsEmpty(), "Caller should pass a fresh reflow status!");
   NS_FRAME_TRACE(
       NS_FRAME_TRACE_CALLS,
       ("enter nsVideoFrame::Reflow: availSize=%d,%d",
-       aReflowInput.AvailableWidth(), aReflowInput.AvailableHeight()));
+       aReflowInput.AvailableISize(), aReflowInput.AvailableBSize()));
 
-  MOZ_ASSERT(mState & NS_FRAME_IN_REFLOW, "frame is not in reflow");
+  MOZ_ASSERT(HasAnyStateBits(NS_FRAME_IN_REFLOW), "frame is not in reflow");
 
   const WritingMode myWM = aReflowInput.GetWritingMode();
   nscoord contentBoxBSize = aReflowInput.ComputedBSize();
@@ -246,8 +246,6 @@ void nsVideoFrame::Reflow(nsPresContext* aPresContext, ReflowOutput& aMetrics,
     borderBoxBSize = contentBoxBSize + logicalBP.BStartEnd(myWM);
   }
 
-  nsMargin borderPadding = aReflowInput.ComputedPhysicalBorderPadding();
-
   nsIContent* videoControlsDiv = GetVideoControls();
 
   // Reflow the child frames. We may have up to three: an image
@@ -256,53 +254,40 @@ void nsVideoFrame::Reflow(nsPresContext* aPresContext, ReflowOutput& aMetrics,
   for (nsIFrame* child : mFrames) {
     nsSize oldChildSize = child->GetSize();
     nsReflowStatus childStatus;
+    const WritingMode childWM = child->GetWritingMode();
+    LogicalSize availableSize = aReflowInput.ComputedSize(childWM);
+    availableSize.BSize(childWM) = NS_UNCONSTRAINEDSIZE;
+    ReflowInput kidReflowInput(aPresContext, aReflowInput, child,
+                               availableSize);
+    ReflowOutput kidDesiredSize(myWM);
+    const nsSize containerSize =
+        aReflowInput.ComputedSizeAsContainerIfConstrained();
 
     if (child->GetContent() == mPosterImage) {
       // Reflow the poster frame.
-      nsImageFrame* imageFrame = static_cast<nsImageFrame*>(child);
-      ReflowOutput kidDesiredSize(aReflowInput);
-      WritingMode wm = imageFrame->GetWritingMode();
-      LogicalSize availableSize = aReflowInput.AvailableSize(wm);
-      availableSize.BSize(wm) = NS_UNCONSTRAINEDSIZE;
+      const LogicalPoint childOrigin = logicalBP.StartOffset(myWM);
+      const LogicalSize posterRenderSize = aReflowInput.ComputedSize(childWM);
+      kidReflowInput.SetComputedISize(posterRenderSize.ISize(childWM));
+      kidReflowInput.SetComputedBSize(posterRenderSize.BSize(childWM));
 
-      LogicalSize cbSize = aMetrics.Size(aMetrics.GetWritingMode())
-                               .ConvertTo(wm, aMetrics.GetWritingMode());
-      ReflowInput kidReflowInput(aPresContext, aReflowInput, imageFrame,
-                                 availableSize, Some(cbSize));
-
-      nsRect posterRenderRect;
-      if (ShouldDisplayPoster()) {
-        posterRenderRect =
-            nsRect(nsPoint(borderPadding.left, borderPadding.top),
-                   nsSize(aReflowInput.ComputedWidth(),
-                          aReflowInput.ComputedHeight()));
-      }
-      kidReflowInput.SetComputedWidth(posterRenderRect.width);
-      kidReflowInput.SetComputedHeight(posterRenderRect.height);
-      ReflowChild(imageFrame, aPresContext, kidDesiredSize, kidReflowInput,
-                  posterRenderRect.x, posterRenderRect.y,
-                  ReflowChildFlags::Default, childStatus);
+      ReflowChild(child, aPresContext, kidDesiredSize, kidReflowInput, myWM,
+                  childOrigin, containerSize, ReflowChildFlags::Default,
+                  childStatus);
       MOZ_ASSERT(childStatus.IsFullyComplete(),
                  "We gave our child unconstrained available block-size, "
                  "so it should be complete!");
 
-      FinishReflowChild(imageFrame, aPresContext, kidDesiredSize,
-                        &kidReflowInput, posterRenderRect.x, posterRenderRect.y,
+      FinishReflowChild(child, aPresContext, kidDesiredSize, &kidReflowInput,
+                        myWM, childOrigin, containerSize,
                         ReflowChildFlags::Default);
 
     } else if (child->GetContent() == mCaptionDiv ||
                child->GetContent() == videoControlsDiv) {
       // Reflow the caption and control bar frames.
-      WritingMode wm = child->GetWritingMode();
-      LogicalSize availableSize = aReflowInput.ComputedSize(wm);
-      availableSize.BSize(wm) = NS_UNCONSTRAINEDSIZE;
-
-      ReflowInput kidReflowInput(aPresContext, aReflowInput, child,
-                                 availableSize);
-      ReflowOutput kidDesiredSize(kidReflowInput);
-      ReflowChild(child, aPresContext, kidDesiredSize, kidReflowInput,
-                  borderPadding.left, borderPadding.top,
-                  ReflowChildFlags::Default, childStatus);
+      const LogicalPoint childOrigin = logicalBP.StartOffset(myWM);
+      ReflowChild(child, aPresContext, kidDesiredSize, kidReflowInput, myWM,
+                  childOrigin, containerSize, ReflowChildFlags::Default,
+                  childStatus);
       MOZ_ASSERT(childStatus.IsFullyComplete(),
                  "We gave our child unconstrained available block-size, "
                  "so it should be complete!");
@@ -311,16 +296,15 @@ void nsVideoFrame::Reflow(nsPresContext* aPresContext, ReflowOutput& aMetrics,
         // Resolve our own BSize based on the controls' size in the
         // same axis. Unless we're size-contained, in which case we
         // have to behave as if we have an intrinsic size of 0.
-        if (aReflowInput.mStyleDisplay->GetContainSizeAxes().mBContained) {
+        if (GetContainSizeAxes().mBContained) {
           contentBoxBSize = 0;
         } else {
-          contentBoxBSize = myWM.IsOrthogonalTo(wm) ? kidDesiredSize.ISize(wm)
-                                                    : kidDesiredSize.BSize(wm);
+          contentBoxBSize = kidDesiredSize.BSize(myWM);
         }
       }
 
       FinishReflowChild(child, aPresContext, kidDesiredSize, &kidReflowInput,
-                        borderPadding.left, borderPadding.top,
+                        myWM, childOrigin, containerSize,
                         ReflowChildFlags::Default);
 
       if (child->GetSize() != oldChildSize) {
@@ -345,9 +329,7 @@ void nsVideoFrame::Reflow(nsPresContext* aPresContext, ReflowOutput& aMetrics,
       // get one from our controls. Just use BSize of 0.
       contentBoxBSize = 0;
     }
-    contentBoxBSize =
-        NS_CSS_MINMAX(contentBoxBSize, aReflowInput.ComputedMinBSize(),
-                      aReflowInput.ComputedMaxBSize());
+    contentBoxBSize = aReflowInput.ApplyMinMaxBSize(contentBoxBSize);
     borderBoxBSize = contentBoxBSize + logicalBP.BStartEnd(myWM);
   }
 
@@ -362,7 +344,6 @@ void nsVideoFrame::Reflow(nsPresContext* aPresContext, ReflowOutput& aMetrics,
                                         aMetrics.Width(), aMetrics.Height()));
 
   MOZ_ASSERT(aStatus.IsEmpty(), "This type of frame can't be split.");
-  NS_FRAME_SET_TRUNCATION(aStatus, aReflowInput, aMetrics);
 }
 
 #ifdef ACCESSIBILITY
@@ -392,20 +373,10 @@ nsIFrame::SizeComputationResult nsVideoFrame::ComputeSize(
           AspectRatioUsage::None};
 }
 
-nscoord nsVideoFrame::GetMinISize(gfxContext* aRenderingContext) {
-  nscoord result;
-  // Bind the result variable to a RAII-based debug object - the variable
-  // therefore must match the function's return value.
-  DISPLAY_MIN_INLINE_SIZE(this, result);
-  // This call handles size-containment
-  nsSize size = GetVideoIntrinsicSize();
-  result = GetWritingMode().IsVertical() ? size.height : size.width;
-  return result;
-}
-
-nscoord nsVideoFrame::GetPrefISize(gfxContext* aRenderingContext) {
+nscoord nsVideoFrame::IntrinsicISize(const IntrinsicSizeInput& aInput,
+                                     IntrinsicISizeType aType) {
   // <audio> / <video> has the same min / pref ISize.
-  return GetMinISize(aRenderingContext);
+  return GetIntrinsicSize().ISize(GetWritingMode()).valueOr(0);
 }
 
 Maybe<nsSize> nsVideoFrame::PosterImageSize() const {
@@ -421,11 +392,11 @@ AspectRatio nsVideoFrame::GetIntrinsicRatio() const {
   }
 
   // 'contain:[inline-]size' replaced elements have no intrinsic ratio.
-  if (StyleDisplay()->GetContainSizeAxes().IsAny()) {
+  if (GetContainSizeAxes().IsAny()) {
     return AspectRatio();
   }
 
-  HTMLVideoElement* element = static_cast<HTMLVideoElement*>(GetContent());
+  auto* element = static_cast<HTMLVideoElement*>(GetContent());
   if (Maybe<CSSIntSize> size = element->GetVideoSize()) {
     return AspectRatio::FromSize(*size);
   }
@@ -436,14 +407,22 @@ AspectRatio nsVideoFrame::GetIntrinsicRatio() const {
     }
   }
 
+  if (StylePosition()->mAspectRatio.HasRatio()) {
+    return AspectRatio();
+  }
+
   return AspectRatio::FromSize(kFallbackIntrinsicSizeInPixels);
 }
 
 bool nsVideoFrame::ShouldDisplayPoster() const {
-  if (!HasVideoElement()) return false;
+  if (!HasVideoElement()) {
+    return false;
+  }
 
-  HTMLVideoElement* element = static_cast<HTMLVideoElement*>(GetContent());
-  if (element->GetPlayedOrSeeked() && HasVideoData()) return false;
+  auto* element = static_cast<HTMLVideoElement*>(GetContent());
+  if (element->GetPlayedOrSeeked() && HasVideoData()) {
+    return false;
+  }
 
   nsCOMPtr<nsIImageLoadingContent> imgContent = do_QueryInterface(mPosterImage);
   NS_ENSURE_TRUE(imgContent, false);
@@ -457,49 +436,60 @@ bool nsVideoFrame::ShouldDisplayPoster() const {
 
   uint32_t status = 0;
   res = request->GetImageStatus(&status);
-  if (NS_FAILED(res) || (status & imgIRequest::STATUS_ERROR)) return false;
+  if (NS_FAILED(res) || (status & imgIRequest::STATUS_ERROR)) {
+    return false;
+  }
 
   return true;
 }
 
-nsSize nsVideoFrame::GetVideoIntrinsicSize() const {
-  const auto containAxes = StyleDisplay()->GetContainSizeAxes();
+IntrinsicSize nsVideoFrame::GetIntrinsicSize() {
+  const auto containAxes = GetContainSizeAxes();
   const auto isVideo = HasVideoElement();
-  // Intrinsic size will be 0,0 if the element is:
-  // - Size-contained, or
-  // - An audio element with no "controls" attribute, distinguished by the last
-  // and only child being the control.
-  if (containAxes.IsBoth() || (!isVideo && !mFrames.LastChild())) {
-    return {};
+  // Intrinsic size will be given by contain-intrinsic-size if the element is
+  // size-contained. If both axes have containment, FinishIntrinsicSize() will
+  // ignore the fallback size argument, so we can just pass no intrinsic size,
+  // or whatever.
+  if (containAxes.IsBoth()) {
+    return FinishIntrinsicSize(containAxes, {});
   }
 
-  auto wm = GetWritingMode();
   if (!isVideo) {
-    return containAxes.ContainSize(kFallbackIntrinsicSize, wm);
+    // An audio element with no "controls" attribute, distinguished by the last
+    // and only child being the control, falls back to no intrinsic size.
+    if (!mFrames.LastChild()) {
+      return FinishIntrinsicSize(containAxes, {});
+    }
+
+    return FinishIntrinsicSize(containAxes,
+                               IntrinsicSize(kFallbackIntrinsicSize));
   }
 
-  HTMLVideoElement* element = static_cast<HTMLVideoElement*>(GetContent());
+  auto* element = static_cast<HTMLVideoElement*>(GetContent());
   if (Maybe<CSSIntSize> size = element->GetVideoSize()) {
-    return containAxes.ContainSize(CSSPixel::ToAppUnits(*size), wm);
+    return FinishIntrinsicSize(containAxes,
+                               IntrinsicSize(CSSPixel::ToAppUnits(*size)));
   }
 
   if (ShouldDisplayPoster()) {
     if (Maybe<nsSize> imgSize = PosterImageSize()) {
-      return containAxes.ContainSize(*imgSize, wm);
+      return FinishIntrinsicSize(containAxes, IntrinsicSize(*imgSize));
     }
   }
-  return containAxes.ContainSize(kFallbackIntrinsicSize, wm);
-}
 
-IntrinsicSize nsVideoFrame::GetIntrinsicSize() {
-  return IntrinsicSize(GetVideoIntrinsicSize());
+  if (StylePosition()->mAspectRatio.HasRatio()) {
+    return {};
+  }
+
+  return FinishIntrinsicSize(containAxes,
+                             IntrinsicSize(kFallbackIntrinsicSize));
 }
 
 void nsVideoFrame::UpdatePosterSource(bool aNotify) {
   NS_ASSERTION(HasVideoElement(), "Only call this on <video> elements.");
   HTMLVideoElement* element = static_cast<HTMLVideoElement*>(GetContent());
 
-  if (element->HasAttr(kNameSpaceID_None, nsGkAtoms::poster) &&
+  if (element->HasAttr(nsGkAtoms::poster) &&
       !element->AttrValueIs(kNameSpaceID_None, nsGkAtoms::poster,
                             nsGkAtoms::_empty, eIgnoreCase)) {
     nsAutoString posterStr;
@@ -535,10 +525,6 @@ void nsVideoFrame::OnVisibilityChange(
   nsContainerFrame::OnVisibilityChange(aNewVisibility, aNonvisibleAction);
 }
 
-bool nsVideoFrame::HasVideoElement() const {
-  return static_cast<HTMLMediaElement*>(GetContent())->IsVideo();
-}
-
 bool nsVideoFrame::HasVideoData() const {
   if (!HasVideoElement()) {
     return false;
@@ -553,14 +539,14 @@ void nsVideoFrame::UpdateTextTrack() {
 
 namespace mozilla {
 
-class nsDisplayVideo : public nsPaintedDisplayItem {
+class nsDisplayVideo final : public nsPaintedDisplayItem {
  public:
   nsDisplayVideo(nsDisplayListBuilder* aBuilder, nsVideoFrame* aFrame)
       : nsPaintedDisplayItem(aBuilder, aFrame) {
     MOZ_COUNT_CTOR(nsDisplayVideo);
   }
 
-  MOZ_COUNTED_DTOR_OVERRIDE(nsDisplayVideo)
+  MOZ_COUNTED_DTOR_FINAL(nsDisplayVideo)
 
   NS_DISPLAY_DECL_NAME("Video", TYPE_VIDEO)
 
@@ -654,7 +640,7 @@ class nsDisplayVideo : public nsPaintedDisplayItem {
       return;
     }
 
-    VideoInfo::Rotation rotationDeg = element->RotationDegrees();
+    VideoRotation rotationDeg = element->RotationDegrees();
     Matrix preTransform = ComputeRotationMatrix(
         destGFXRect.Width(), destGFXRect.Height(), rotationDeg);
     Matrix transform =
@@ -698,13 +684,15 @@ class nsDisplayVideo : public nsPaintedDisplayItem {
 
 void nsVideoFrame::BuildDisplayList(nsDisplayListBuilder* aBuilder,
                                     const nsDisplayListSet& aLists) {
-  if (!IsVisibleForPainting()) return;
+  if (!IsVisibleForPainting()) {
+    return;
+  }
 
   DO_GLOBAL_REFLOW_COUNT_DSP("nsVideoFrame");
 
   DisplayBorderBackgroundOutline(aBuilder, aLists);
 
-  if (IsContentHidden()) {
+  if (HidesContent()) {
     return;
   }
 
@@ -732,8 +720,7 @@ void nsVideoFrame::BuildDisplayList(nsDisplayListBuilder* aBuilder,
   // but only want to draw mPosterImage conditionally. Others we
   // always add to the display list.
   for (nsIFrame* child : mFrames) {
-    if (child->GetContent() != mPosterImage || shouldDisplayPoster ||
-        child->IsBoxFrame()) {
+    if (child->GetContent() != mPosterImage || shouldDisplayPoster) {
       nsDisplayListBuilder::AutoBuildingDisplayList buildingForChild(
           aBuilder, child,
           aBuilder->GetVisibleRect() - child->GetOffsetTo(this),
