@@ -8,22 +8,23 @@
 
 #include "mozilla/dom/Document.h"
 #include "nsIContent.h"
-#include "nsIContentViewer.h"
 #include "nsPIDOMWindow.h"
-#include "nsIDocShell.h"
 #include "nsPresContext.h"
 #include "mozilla/EventStateManager.h"
 #include "mozilla/dom/DocumentInlines.h"
 #include "mozilla/dom/Element.h"
 
 using namespace mozilla;
+using namespace mozilla::dom;
 
 ///////////////////////////////////////////////////////////////////////////////
 
 EventStateManager* inLayoutUtils::GetEventStateManagerFor(Element& aElement) {
   Document* doc = aElement.OwnerDoc();
   nsPresContext* presContext = doc->GetPresContext();
-  if (!presContext) return nullptr;
+  if (!presContext) {
+    return nullptr;
+  }
 
   return presContext->EventStateManager();
 }
