@@ -6,11 +6,16 @@
 
 #include "nsMathMLmmultiscriptsFrame.h"
 
+#include "mozilla/dom/Document.h"
+#include "mozilla/dom/Element.h"
 #include "mozilla/PresShell.h"
+#include "mozilla/StaticPrefs_mathml.h"
+#include "nsLayoutUtils.h"
 #include "nsPresContext.h"
 #include <algorithm>
 #include "gfxContext.h"
 #include "gfxMathTable.h"
+#include "gfxTextRun.h"
 
 using namespace mozilla;
 
@@ -29,13 +34,15 @@ nsIFrame* NS_NewMathMLmmultiscriptsFrame(PresShell* aPresShell,
 
 NS_IMPL_FRAMEARENA_HELPERS(nsMathMLmmultiscriptsFrame)
 
-nsMathMLmmultiscriptsFrame::~nsMathMLmmultiscriptsFrame() {}
+nsMathMLmmultiscriptsFrame::~nsMathMLmmultiscriptsFrame() = default;
 
 uint8_t nsMathMLmmultiscriptsFrame::ScriptIncrement(nsIFrame* aFrame) {
-  if (!aFrame) return 0;
+  if (!aFrame) {
+    return 0;
+  }
   if (mFrames.ContainsFrame(aFrame)) {
     if (mFrames.FirstChild() == aFrame ||
-        aFrame->GetContent()->IsMathMLElement(nsGkAtoms::mprescripts_)) {
+        aFrame->GetContent()->IsMathMLElement(nsGkAtoms::mprescripts)) {
       return 0;  // No script increment for base frames or prescript markers
     }
     return 1;
@@ -54,12 +61,12 @@ nsMathMLmmultiscriptsFrame::TransmitAutomaticData() {
   // the compression flag in them.
 
   int32_t count = 0;
-  bool isSubScript = !mContent->IsMathMLElement(nsGkAtoms::msup_);
+  bool isSubScript = !mContent->IsMathMLElement(nsGkAtoms::msup);
 
   AutoTArray<nsIFrame*, 8> subScriptFrames;
   nsIFrame* childFrame = mFrames.FirstChild();
   while (childFrame) {
-    if (childFrame->GetContent()->IsMathMLElement(nsGkAtoms::mprescripts_)) {
+    if (childFrame->GetContent()->IsMathMLElement(nsGkAtoms::mprescripts)) {
       // mprescripts frame
     } else if (0 == count) {
       // base frame
@@ -88,74 +95,36 @@ nsMathMLmmultiscriptsFrame::TransmitAutomaticData() {
 
 /* virtual */
 nsresult nsMathMLmmultiscriptsFrame::Place(DrawTarget* aDrawTarget,
-                                           bool aPlaceOrigin,
+                                           const PlaceFlags& aFlags,
                                            ReflowOutput& aDesiredSize) {
   nscoord subScriptShift = 0;
   nscoord supScriptShift = 0;
   float fontSizeInflation = nsLayoutUtils::FontSizeInflationFor(this);
 
-  // subscriptshift
-  //
-  // "Specifies the minimum amount to shift the baseline of subscript down; the
-  // default is for the rendering agent to use its own positioning rules."
-  //
-  // values: length
-  // default: automatic
-  //
-  // We use 0 as the default value so unitless values can be ignored.
-  // As a minimum, negative values can be ignored.
-  //
-  nsAutoString value;
-  if (!mContent->IsMathMLElement(nsGkAtoms::msup_)) {
-    mContent->AsElement()->GetAttr(kNameSpaceID_None,
-                                   nsGkAtoms::subscriptshift_, value);
-    if (!value.IsEmpty()) {
-      ParseNumericValue(value, &subScriptShift, 0, PresContext(),
-                        mComputedStyle, fontSizeInflation);
-    }
-  }
-  // superscriptshift
-  //
-  // "Specifies the minimum amount to shift the baseline of superscript up; the
-  // default is for the rendering agent to use its own positioning rules."
-  //
-  // values: length
-  // default: automatic
-  //
-  // We use 0 as the default value so unitless values can be ignored.
-  // As a minimum, negative values can be ignored.
-  //
-  if (!mContent->IsMathMLElement(nsGkAtoms::msub_)) {
-    mContent->AsElement()->GetAttr(kNameSpaceID_None,
-                                   nsGkAtoms::superscriptshift_, value);
-    if (!value.IsEmpty()) {
-      ParseNumericValue(value, &supScriptShift, 0, PresContext(),
-                        mComputedStyle, fontSizeInflation);
-    }
-  }
-  return PlaceMultiScript(PresContext(), aDrawTarget, aPlaceOrigin,
-                          aDesiredSize, this, subScriptShift, supScriptShift,
+  return PlaceMultiScript(PresContext(), aDrawTarget, aFlags, aDesiredSize,
+                          this, subScriptShift, supScriptShift,
                           fontSizeInflation);
 }
 
 // exported routine that both munderover and mmultiscripts share.
 // munderover uses this when movablelimits is set.
 nsresult nsMathMLmmultiscriptsFrame::PlaceMultiScript(
-    nsPresContext* aPresContext, DrawTarget* aDrawTarget, bool aPlaceOrigin,
-    ReflowOutput& aDesiredSize, nsMathMLContainerFrame* aFrame,
-    nscoord aUserSubScriptShift, nscoord aUserSupScriptShift,
-    float aFontSizeInflation) {
+    nsPresContext* aPresContext, DrawTarget* aDrawTarget,
+    const PlaceFlags& aFlags, ReflowOutput& aDesiredSize,
+    nsMathMLContainerFrame* aFrame, nscoord aUserSubScriptShift,
+    nscoord aUserSupScriptShift, float aFontSizeInflation) {
   nsAtom* tag = aFrame->GetContent()->NodeInfo()->NameAtom();
 
   // This function deals with both munderover etc. as well as msubsup etc.
   // As the former behaves identically to the later, we treat it as such
   // to avoid additional checks later.
-  if (aFrame->GetContent()->IsMathMLElement(nsGkAtoms::mover_))
-    tag = nsGkAtoms::msup_;
-  else if (aFrame->GetContent()->IsMathMLElement(nsGkAtoms::munder_))
-    tag = nsGkAtoms::msub_;
-  else if (aFrame->GetContent()->IsMathMLElement(nsGkAtoms::munderover_))
-    tag = nsGkAtoms::msubsup_;
+  if (aFrame->GetContent()->IsMathMLElement(nsGkAtoms::mover)) {
+    tag = nsGkAtoms::msup;
+  } else if (aFrame->GetContent()->IsMathMLElement(nsGkAtoms::munder)) {
+    tag = nsGkAtoms::msub;
+  } else if (aFrame->GetContent()->IsMathMLElement(nsGkAtoms::munderover)) {
+    tag = nsGkAtoms::msubsup;
+  }
 
   nsBoundingMetrics bmFrame;
 
@@ -169,11 +138,12 @@ nsresult nsMathMLmmultiscriptsFrame::PlaceMultiScript(
   nsIFrame* baseFrame = aFrame->PrincipalChildList().FirstChild();
 
   if (!baseFrame) {
-    if (tag == nsGkAtoms::mmultiscripts_)
+    if (tag == nsGkAtoms::mmultiscripts) {
       aFrame->ReportErrorToConsole("NoBase");
-    else
+    } else {
       aFrame->ReportChildCountError();
-    return aFrame->ReflowError(aDrawTarget, aDesiredSize);
+    }
+    return aFrame->PlaceAsMrow(aDrawTarget, aFlags, aDesiredSize);
   }
 
   // get x-height (an ex)
@@ -184,7 +154,7 @@ nsresult nsMathMLmmultiscriptsFrame::PlaceMultiScript(
   nscoord xHeight = fm->XHeight();
 
   nscoord oneDevPixel = fm->AppUnitsPerDevPixel();
-  gfxFont* mathFont = fm->GetThebesFontGroup()->GetFirstMathFont();
+  RefPtr<gfxFont> mathFont = fm->GetThebesFontGroup()->GetFirstMathFont();
   // scriptspace from TeX for extra spacing after sup/subscript
   nscoord scriptSpace;
   if (mathFont) {
@@ -224,7 +194,7 @@ nsresult nsMathMLmmultiscriptsFrame::PlaceMultiScript(
     nscoord subScriptShift1, subScriptShift2;
     // Get subScriptShift{1,2} default from font
     GetSubScriptShifts(fm, subScriptShift1, subScriptShift2);
-    if (tag == nsGkAtoms::msub_) {
+    if (tag == nsGkAtoms::msub) {
       subScriptShift = subScriptShift1;
     } else {
       subScriptShift = std::max(subScriptShift1, subScriptShift2);
@@ -263,8 +233,7 @@ nsresult nsMathMLmmultiscriptsFrame::PlaceMultiScript(
 
     // get sup script shift depending on current script level and display style
     // Rule 18c, App. G, TeXbook
-    if (font->mScriptLevel == 0 &&
-        font->mMathDisplay == NS_MATHML_DISPLAYSTYLE_BLOCK &&
+    if (font->mMathDepth == 0 && font->mMathStyle == StyleMathStyle::Normal &&
         !NS_MATHML_IS_COMPRESSED(presentationData.flags)) {
       // Style D in TeXbook
       supScriptShift = supScriptShift1;
@@ -304,6 +273,7 @@ nsresult nsMathMLmmultiscriptsFrame::PlaceMultiScript(
 
   bool firstPrescriptsPair = false;
   nsBoundingMetrics bmBase, bmSubScript, bmSupScript, bmMultiSub, bmMultiSup;
+  nsMargin baseMargin, subScriptMargin, supScriptMargin;
   multiSubSize.SetBlockStartAscent(-0x7FFFFFFF);
   multiSupSize.SetBlockStartAscent(-0x7FFFFFFF);
   bmMultiSub.ascent = bmMultiSup.ascent = -0x7FFFFFFF;
@@ -316,56 +286,44 @@ nsresult nsMathMLmmultiscriptsFrame::PlaceMultiScript(
   aDesiredSize.Width() = aDesiredSize.Height() = 0;
 
   int32_t count = 0;
-  bool foundNoneTag = false;
 
   // Boolean to determine whether the current child is a subscript.
   // Note that only msup starts with a superscript.
-  bool isSubScript = (tag != nsGkAtoms::msup_);
+  bool isSubScript = (tag != nsGkAtoms::msup);
 
   nsIFrame* childFrame = aFrame->PrincipalChildList().FirstChild();
   while (childFrame) {
-    if (childFrame->GetContent()->IsMathMLElement(nsGkAtoms::mprescripts_)) {
-      if (tag != nsGkAtoms::mmultiscripts_) {
-        if (aPlaceOrigin) {
-          aFrame->ReportInvalidChildError(nsGkAtoms::mprescripts_);
+    if (childFrame->GetContent()->IsMathMLElement(nsGkAtoms::mprescripts)) {
+      if (tag != nsGkAtoms::mmultiscripts) {
+        if (!aFlags.contains(PlaceFlag::MeasureOnly)) {
+          aFrame->ReportInvalidChildError(nsGkAtoms::mprescripts);
         }
-        return aFrame->ReflowError(aDrawTarget, aDesiredSize);
+        return aFrame->PlaceAsMrow(aDrawTarget, aFlags, aDesiredSize);
       }
       if (prescriptsFrame) {
         // duplicate <mprescripts/> found
         // report an error, encourage people to get their markups in order
-        if (aPlaceOrigin) {
+        if (!aFlags.contains(PlaceFlag::MeasureOnly)) {
           aFrame->ReportErrorToConsole("DuplicateMprescripts");
         }
-        return aFrame->ReflowError(aDrawTarget, aDesiredSize);
+        return aFrame->PlaceAsMrow(aDrawTarget, aFlags, aDesiredSize);
       }
       if (!isSubScript) {
-        if (aPlaceOrigin) {
+        if (!aFlags.contains(PlaceFlag::MeasureOnly)) {
           aFrame->ReportErrorToConsole("SubSupMismatch");
         }
-        return aFrame->ReflowError(aDrawTarget, aDesiredSize);
+        return aFrame->PlaceAsMrow(aDrawTarget, aFlags, aDesiredSize);
       }
 
       prescriptsFrame = childFrame;
       firstPrescriptsPair = true;
     } else if (0 == count) {
       // base
-
-      if (childFrame->GetContent()->IsMathMLElement(nsGkAtoms::none)) {
-        if (tag == nsGkAtoms::mmultiscripts_) {
-          if (aPlaceOrigin) {
-            aFrame->ReportErrorToConsole("NoBase");
-          }
-          return aFrame->ReflowError(aDrawTarget, aDesiredSize);
-        } else {
-          // A different error message is triggered later for the other tags
-          foundNoneTag = true;
-        }
-      }
       baseFrame = childFrame;
       GetReflowAndBoundingMetricsFor(baseFrame, baseSize, bmBase);
+      baseMargin = GetMarginForPlace(aFlags, baseFrame);
 
-      if (tag != nsGkAtoms::msub_) {
+      if (tag != nsGkAtoms::msub) {
         // Apply italics correction if there is the potential for a
         // postsupscript.
         GetItalicCorrection(bmBase, italicCorrection);
@@ -377,39 +335,44 @@ nsresult nsMathMLmmultiscriptsFrame::PlaceMultiScript(
 
       // we update boundingMetrics.{ascent,descent} with that
       // of the baseFrame only after processing all the sup/sub pairs
-      boundingMetrics.width = bmBase.width;
-      boundingMetrics.rightBearing = bmBase.rightBearing;
+      boundingMetrics.width = bmBase.width + baseMargin.LeftRight();
+      boundingMetrics.rightBearing =
+          bmBase.rightBearing + baseMargin.LeftRight();
       boundingMetrics.leftBearing = bmBase.leftBearing;  // until overwritten
     } else {
       // super/subscript block
-      if (childFrame->GetContent()->IsMathMLElement(nsGkAtoms::none)) {
-        foundNoneTag = true;
-      }
-
       if (isSubScript) {
         // subscript
         subScriptFrame = childFrame;
         GetReflowAndBoundingMetricsFor(subScriptFrame, subScriptSize,
                                        bmSubScript);
+        subScriptMargin = GetMarginForPlace(aFlags, subScriptFrame);
+
         if (!mathFont) {
           // get the subdrop from the subscript font
           GetSubDropFromChild(subScriptFrame, subDrop, aFontSizeInflation);
         }
 
         // parameter v, Rule 18a, App. G, TeXbook
-        minSubScriptShift = bmBase.descent + subDrop;
+        minSubScriptShift = bmBase.descent + baseMargin.bottom + subDrop;
         trySubScriptShift = std::max(minSubScriptShift, subScriptShift);
-        multiSubSize.SetBlockStartAscent(std::max(
-            multiSubSize.BlockStartAscent(), subScriptSize.BlockStartAscent()));
-        bmMultiSub.ascent = std::max(bmMultiSub.ascent, bmSubScript.ascent);
-        bmMultiSub.descent = std::max(bmMultiSub.descent, bmSubScript.descent);
+        multiSubSize.SetBlockStartAscent(
+            std::max(multiSubSize.BlockStartAscent(),
+                     subScriptSize.BlockStartAscent() + subScriptMargin.top));
+        bmMultiSub.ascent = std::max(bmMultiSub.ascent,
+                                     bmSubScript.ascent + subScriptMargin.top);
+        bmMultiSub.descent = std::max(
+            bmMultiSub.descent, bmSubScript.descent + subScriptMargin.bottom);
         multiSubSize.Height() =
             std::max(multiSubSize.Height(),
-                     subScriptSize.Height() - subScriptSize.BlockStartAscent());
-        if (bmSubScript.width) width = bmSubScript.width + scriptSpace;
-        rightBearing = bmSubScript.rightBearing;
+                     subScriptSize.Height() - subScriptSize.BlockStartAscent() +
+                         subScriptMargin.bottom);
+        if (bmSubScript.width) {
+          width = bmSubScript.width + subScriptMargin.LeftRight() + scriptSpace;
+        }
+        rightBearing = bmSubScript.rightBearing + subScriptMargin.LeftRight();
 
-        if (tag == nsGkAtoms::msub_) {
+        if (tag == nsGkAtoms::msub) {
           boundingMetrics.rightBearing = boundingMetrics.width + rightBearing;
           boundingMetrics.width += width;
 
@@ -422,7 +385,8 @@ nsresult nsMathMLmmultiscriptsFrame::PlaceMultiScript(
             // = h(x) - 4/5 * sigma_5, Rule 18b, App. G, TeXbook
             subscriptTopMax = NSToCoordRound((4.0f / 5.0f) * xHeight);
           }
-          nscoord minShiftFromXHeight = bmSubScript.ascent - subscriptTopMax;
+          nscoord minShiftFromXHeight =
+              bmSubScript.ascent + subScriptMargin.top - subscriptTopMax;
           maxSubScriptShift = std::max(trySubScriptShift, minShiftFromXHeight);
 
           maxSubScriptShift = std::max(maxSubScriptShift, trySubScriptShift);
@@ -433,12 +397,13 @@ nsresult nsMathMLmmultiscriptsFrame::PlaceMultiScript(
         supScriptFrame = childFrame;
         GetReflowAndBoundingMetricsFor(supScriptFrame, supScriptSize,
                                        bmSupScript);
+        supScriptMargin = GetMarginForPlace(aFlags, supScriptFrame);
         if (!mathFont) {
           // get the supdrop from the supscript font
           GetSupDropFromChild(supScriptFrame, supDrop, aFontSizeInflation);
         }
         // parameter u, Rule 18a, App. G, TeXbook
-        minSupScriptShift = bmBase.ascent - supDrop;
+        minSupScriptShift = bmBase.ascent + baseMargin.top - supDrop;
         nscoord superscriptBottomMin;
         if (mathFont) {
           superscriptBottomMin = mathFont->MathTable()->Constant(
@@ -448,23 +413,32 @@ nsresult nsMathMLmmultiscriptsFrame::PlaceMultiScript(
           // = d(x) + 1/4 * sigma_5, Rule 18c, App. G, TeXbook
           superscriptBottomMin = NSToCoordRound((1.0f / 4.0f) * xHeight);
         }
-        minShiftFromXHeight = bmSupScript.descent + superscriptBottomMin;
+        minShiftFromXHeight =
+            bmSupScript.descent + supScriptMargin.bottom + superscriptBottomMin;
         trySupScriptShift = std::max(
             minSupScriptShift, std::max(minShiftFromXHeight, supScriptShift));
-        multiSupSize.SetBlockStartAscent(std::max(
-            multiSupSize.BlockStartAscent(), supScriptSize.BlockStartAscent()));
-        bmMultiSup.ascent = std::max(bmMultiSup.ascent, bmSupScript.ascent);
-        bmMultiSup.descent = std::max(bmMultiSup.descent, bmSupScript.descent);
+        multiSupSize.SetBlockStartAscent(
+            std::max(multiSupSize.BlockStartAscent(),
+                     supScriptSize.BlockStartAscent() + supScriptMargin.top));
+        bmMultiSup.ascent = std::max(bmMultiSup.ascent,
+                                     bmSupScript.ascent + supScriptMargin.top);
+        bmMultiSup.descent = std::max(
+            bmMultiSup.descent, bmSupScript.descent + supScriptMargin.bottom);
         multiSupSize.Height() =
             std::max(multiSupSize.Height(),
-                     supScriptSize.Height() - supScriptSize.BlockStartAscent());
+                     supScriptSize.Height() - supScriptSize.BlockStartAscent() +
+                         supScriptMargin.bottom);
 
-        if (bmSupScript.width)
-          width = std::max(width, bmSupScript.width + scriptSpace);
+        if (bmSupScript.width) {
+          width =
+              std::max(width, bmSupScript.width + supScriptMargin.LeftRight() +
+                                  scriptSpace);
+        }
 
         if (!prescriptsFrame) {  // we are still looping over base & postscripts
           rightBearing = std::max(rightBearing,
-                                  italicCorrection + bmSupScript.rightBearing);
+                                  italicCorrection + bmSupScript.rightBearing +
+                                      supScriptMargin.LeftRight());
           boundingMetrics.rightBearing = boundingMetrics.width + rightBearing;
           boundingMetrics.width += width;
         } else {
@@ -480,7 +454,7 @@ nsresult nsMathMLmmultiscriptsFrame::PlaceMultiScript(
         // negotiate between the various shifts so that
         // there is enough gap between the sup and subscripts
         // Rule 18e, App. G, TeXbook
-        if (tag == nsGkAtoms::mmultiscripts_ || tag == nsGkAtoms::msubsup_) {
+        if (tag == nsGkAtoms::mmultiscripts || tag == nsGkAtoms::msubsup) {
           nscoord subSuperscriptGapMin;
           if (mathFont) {
             subSuperscriptGapMin = mathFont->MathTable()->Constant(
@@ -490,8 +464,10 @@ nsresult nsMathMLmmultiscriptsFrame::PlaceMultiScript(
             GetRuleThickness(aDrawTarget, fm, ruleSize);
             subSuperscriptGapMin = 4 * ruleSize;
           }
-          nscoord gap = (trySupScriptShift - bmSupScript.descent) -
-                        (bmSubScript.ascent - trySubScriptShift);
+          nscoord gap =
+              (trySupScriptShift - bmSupScript.descent -
+               supScriptMargin.bottom) -
+              (subScriptMargin.top + bmSubScript.ascent - trySubScriptShift);
           if (gap < subSuperscriptGapMin) {
             // adjust trySubScriptShift to get a gap of subSuperscriptGapMin
             trySubScriptShift += subSuperscriptGapMin - gap;
@@ -508,7 +484,8 @@ nsresult nsMathMLmmultiscriptsFrame::PlaceMultiScript(
                 NSToCoordRound((4.0f / 5.0f) * xHeight);
           }
           gap = superscriptBottomMaxWithSubscript -
-                (trySupScriptShift - bmSupScript.descent);
+                (trySupScriptShift - bmSupScript.descent -
+                 supScriptMargin.bottom);
           if (gap > 0) {
             trySupScriptShift += gap;
             trySubScriptShift -= gap;
@@ -529,25 +506,21 @@ nsresult nsMathMLmmultiscriptsFrame::PlaceMultiScript(
   }
 
   // NoBase error may also have been reported above
-  if ((count != 2 && (tag == nsGkAtoms::msup_ || tag == nsGkAtoms::msub_)) ||
-      (count != 3 && tag == nsGkAtoms::msubsup_) || !baseFrame ||
-      (foundNoneTag && tag != nsGkAtoms::mmultiscripts_) ||
-      (!isSubScript && tag == nsGkAtoms::mmultiscripts_)) {
+  if ((count != 2 && (tag == nsGkAtoms::msup || tag == nsGkAtoms::msub)) ||
+      (count != 3 && tag == nsGkAtoms::msubsup) || !baseFrame ||
+      (!isSubScript && tag == nsGkAtoms::mmultiscripts)) {
     // report an error, encourage people to get their markups in order
-    if (aPlaceOrigin) {
-      if ((count != 2 &&
-           (tag == nsGkAtoms::msup_ || tag == nsGkAtoms::msub_)) ||
-          (count != 3 && tag == nsGkAtoms::msubsup_)) {
+    if (!aFlags.contains(PlaceFlag::MeasureOnly)) {
+      if ((count != 2 && (tag == nsGkAtoms::msup || tag == nsGkAtoms::msub)) ||
+          (count != 3 && tag == nsGkAtoms::msubsup)) {
         aFrame->ReportChildCountError();
-      } else if (foundNoneTag && tag != nsGkAtoms::mmultiscripts_) {
-        aFrame->ReportInvalidChildError(nsGkAtoms::none);
       } else if (!baseFrame) {
         aFrame->ReportErrorToConsole("NoBase");
       } else {
         aFrame->ReportErrorToConsole("SubSupMismatch");
       }
     }
-    return aFrame->ReflowError(aDrawTarget, aDesiredSize);
+    return aFrame->PlaceAsMrow(aDrawTarget, aFlags, aDesiredSize);
   }
 
   // we left out the width of prescripts, so ...
@@ -556,39 +529,54 @@ nsresult nsMathMLmmultiscriptsFrame::PlaceMultiScript(
 
   // Zero out the shifts in where a frame isn't present to avoid the potential
   // for overflow.
-  if (!subScriptFrame) maxSubScriptShift = 0;
-  if (!supScriptFrame) maxSupScriptShift = 0;
+  if (!subScriptFrame) {
+    maxSubScriptShift = 0;
+  }
+  if (!supScriptFrame) {
+    maxSupScriptShift = 0;
+  }
 
   // we left out the base during our bounding box updates, so ...
-  if (tag == nsGkAtoms::msub_) {
-    boundingMetrics.ascent =
-        std::max(bmBase.ascent, bmMultiSub.ascent - maxSubScriptShift);
+  if (tag == nsGkAtoms::msub) {
+    boundingMetrics.ascent = std::max(bmBase.ascent + baseMargin.top,
+                                      bmMultiSub.ascent - maxSubScriptShift);
   } else {
-    boundingMetrics.ascent =
-        std::max(bmBase.ascent, (bmMultiSup.ascent + maxSupScriptShift));
+    boundingMetrics.ascent = std::max(bmBase.ascent + baseMargin.top,
+                                      (bmMultiSup.ascent + maxSupScriptShift));
   }
-  if (tag == nsGkAtoms::msup_) {
-    boundingMetrics.descent =
-        std::max(bmBase.descent, bmMultiSup.descent - maxSupScriptShift);
+  if (tag == nsGkAtoms::msup) {
+    boundingMetrics.descent = std::max(bmBase.descent + baseMargin.bottom,
+                                       bmMultiSup.descent - maxSupScriptShift);
   } else {
     boundingMetrics.descent =
-        std::max(bmBase.descent, (bmMultiSub.descent + maxSubScriptShift));
+        std::max(bmBase.descent + baseMargin.bottom,
+                 (bmMultiSub.descent + maxSubScriptShift));
   }
-  aFrame->SetBoundingMetrics(boundingMetrics);
 
   // get the reflow metrics ...
   aDesiredSize.SetBlockStartAscent(
-      std::max(baseSize.BlockStartAscent(),
+      std::max(baseSize.BlockStartAscent() + baseMargin.top,
                std::max(multiSubSize.BlockStartAscent() - maxSubScriptShift,
                         multiSupSize.BlockStartAscent() + maxSupScriptShift)));
   aDesiredSize.Height() =
       aDesiredSize.BlockStartAscent() +
-      std::max(baseSize.Height() - baseSize.BlockStartAscent(),
-               std::max(multiSubSize.Height() + maxSubScriptShift,
-                        multiSupSize.Height() - maxSupScriptShift));
+      std::max(
+          baseSize.Height() - baseSize.BlockStartAscent() + baseMargin.bottom,
+          std::max(multiSubSize.Height() + maxSubScriptShift,
+                   multiSupSize.Height() - maxSupScriptShift));
   aDesiredSize.Width() = boundingMetrics.width;
   aDesiredSize.mBoundingMetrics = boundingMetrics;
 
+  // Apply width/height to math content box.
+  auto sizes = aFrame->GetWidthAndHeightForPlaceAdjustment(aFlags);
+  aFrame->ApplyAdjustmentForWidthAndHeight(aFlags, sizes, aDesiredSize,
+                                           boundingMetrics);
+
+  // Add padding+border.
+  auto borderPadding = aFrame->GetBorderPaddingForPlace(aFlags);
+  InflateReflowAndBoundingMetrics(borderPadding, aDesiredSize, boundingMetrics);
+
+  aFrame->SetBoundingMetrics(boundingMetrics);
   aFrame->SetReference(nsPoint(0, aDesiredSize.BlockStartAscent()));
 
   //////////////////
@@ -598,16 +586,20 @@ nsresult nsMathMLmmultiscriptsFrame::PlaceMultiScript(
   // The list of frames is in the order: {base} {postscripts} {prescripts}
   // We go over the list in a circular manner, starting at <prescripts/>
 
-  if (aPlaceOrigin) {
-    nscoord dx = 0, dy = 0;
+  if (!aFlags.contains(PlaceFlag::MeasureOnly)) {
+    const bool isRTL =
+        aFrame->StyleVisibility()->mDirection == StyleDirection::Rtl;
+    nscoord dx = isRTL ? borderPadding.right : borderPadding.left;
+    nscoord dy = 0;
 
     // With msub and msup there is only one element and
     // subscriptFrame/supScriptFrame have already been set above where
     // relevant.  In these cases we skip to the reflow part.
-    if (tag == nsGkAtoms::msub_ || tag == nsGkAtoms::msup_)
+    if (tag == nsGkAtoms::msub || tag == nsGkAtoms::msup) {
       count = 1;
-    else
+    } else {
       count = 0;
+    }
     childFrame = prescriptsFrame;
     bool isPreScript = true;
     do {
@@ -616,52 +608,80 @@ nsresult nsMathMLmmultiscriptsFrame::PlaceMultiScript(
         // place the base ...
         childFrame = baseFrame;
         dy = aDesiredSize.BlockStartAscent() - baseSize.BlockStartAscent();
-        FinishReflowChild(
-            baseFrame, aPresContext, baseSize, nullptr,
-            aFrame->MirrorIfRTL(aDesiredSize.Width(), baseSize.Width(), dx), dy,
-            0);
-        dx += bmBase.width;
-      } else if (prescriptsFrame == childFrame) {
-        // Clear reflow flags of prescripts frame.
-        prescriptsFrame->DidReflow(aPresContext, nullptr);
-      } else {
+        baseMargin = GetMarginForPlace(aFlags, baseFrame);
+        nscoord dx_base = dx + (isRTL ? baseMargin.right : baseMargin.left);
+        FinishReflowChild(baseFrame, aPresContext, baseSize, nullptr,
+                          aFrame->MirrorIfRTL(aDesiredSize.Width(),
+                                              baseSize.Width(), dx_base),
+                          dy, ReflowChildFlags::Default);
+        if (prescriptsFrame) {
+          // place the <mprescripts/>
+          ReflowOutput prescriptsSize(wm);
+          nsBoundingMetrics unusedBm;
+          GetReflowAndBoundingMetricsFor(prescriptsFrame, prescriptsSize,
+                                         unusedBm);
+          nsMargin prescriptsMargin =
+              GetMarginForPlace(aFlags, prescriptsFrame);
+          nscoord dx_prescripts =
+              dx + (isRTL ? prescriptsMargin.right : prescriptsMargin.left);
+          dy = aDesiredSize.BlockStartAscent() -
+               prescriptsSize.BlockStartAscent();
+          FinishReflowChild(
+              prescriptsFrame, aPresContext, prescriptsSize, nullptr,
+              aFrame->MirrorIfRTL(aDesiredSize.Width(), prescriptsSize.Width(),
+                                  dx_prescripts),
+              dy, ReflowChildFlags::Default);
+        }
+        dx += bmBase.width + baseMargin.LeftRight();
+      } else if (childFrame != prescriptsFrame) {
         // process each sup/sub pair
         if (0 == count) {
           subScriptFrame = childFrame;
           count = 1;
         } else if (1 == count) {
-          if (tag != nsGkAtoms::msub_) supScriptFrame = childFrame;
+          if (tag != nsGkAtoms::msub) {
+            supScriptFrame = childFrame;
+          }
           count = 0;
 
           // get the ascent/descent of sup/subscripts stored in their rects
           // rect.x = descent, rect.y = ascent
-          if (subScriptFrame)
+          if (subScriptFrame) {
             GetReflowAndBoundingMetricsFor(subScriptFrame, subScriptSize,
                                            bmSubScript);
-          if (supScriptFrame)
+            subScriptMargin = GetMarginForPlace(aFlags, subScriptFrame);
+          }
+          if (supScriptFrame) {
             GetReflowAndBoundingMetricsFor(supScriptFrame, supScriptSize,
                                            bmSupScript);
+            supScriptMargin = GetMarginForPlace(aFlags, supScriptFrame);
+          }
 
-          width = std::max(subScriptSize.Width(), supScriptSize.Width());
+          width = std::max(subScriptSize.Width() + subScriptMargin.LeftRight(),
+                           supScriptSize.Width() + supScriptMargin.LeftRight());
 
           if (subScriptFrame) {
-            nscoord x = dx;
+            nscoord x =
+                dx + (isRTL ? subScriptMargin.right : subScriptMargin.left);
             // prescripts should be right aligned
             // https://bugzilla.mozilla.org/show_bug.cgi?id=928675
-            if (isPreScript) x += width - subScriptSize.Width();
+            if (isPreScript) {
+              x += width - subScriptSize.Width() - subScriptMargin.LeftRight();
+            }
             dy = aDesiredSize.BlockStartAscent() -
                  subScriptSize.BlockStartAscent() + maxSubScriptShift;
             FinishReflowChild(subScriptFrame, aPresContext, subScriptSize,
                               nullptr,
                               aFrame->MirrorIfRTL(aDesiredSize.Width(),
                                                   subScriptSize.Width(), x),
-                              dy, 0);
+                              dy, ReflowChildFlags::Default);
           }
 
           if (supScriptFrame) {
-            nscoord x = dx;
+            nscoord x =
+                dx + (isRTL ? supScriptMargin.right : supScriptMargin.left);
             if (isPreScript) {
-              x += width - supScriptSize.Width();
+              x += width - supScriptSize.Width() - supScriptMargin.LeftRight();
             } else {
               // post superscripts are shifted by the italic correction value
               x += italicCorrection;
@@ -672,7 +692,7 @@ nsresult nsMathMLmmultiscriptsFrame::PlaceMultiScript(
                               nullptr,
                               aFrame->MirrorIfRTL(aDesiredSize.Width(),
                                                   supScriptSize.Width(), x),
-                              dy, 0);
+                              dy, ReflowChildFlags::Default);
           }
           dx += width + scriptSpace;
         }
