@@ -10,8 +10,9 @@
 #include "nsStyleConsts.h"
 #include "nsNameSpaceManager.h"
 #include "nsCSSRendering.h"
-#include "nsMathMLElement.h"
+#include "mozilla/dom/MathMLElement.h"
 
+#include "nsCRT.h"
 #include "nsTArray.h"
 #include "nsTableFrame.h"
 #include "celldata.h"
@@ -22,9 +23,11 @@
 
 #include "nsIScriptError.h"
 #include "nsContentUtils.h"
+#include "nsLayoutUtils.h"
 
 using namespace mozilla;
 using namespace mozilla::image;
+using mozilla::dom::Element;
 
 //
 // <mtable> -- table or matrix - implementation
@@ -32,34 +35,41 @@ using namespace mozilla::image;
 
 static int8_t ParseStyleValue(nsAtom* aAttribute,
                               const nsAString& aAttributeValue) {
-  if (aAttribute == nsGkAtoms::rowalign_) {
-    if (aAttributeValue.EqualsLiteral("top"))
+  if (aAttribute == nsGkAtoms::rowalign) {
+    if (aAttributeValue.EqualsLiteral("top")) {
       return static_cast<int8_t>(StyleVerticalAlignKeyword::Top);
-    else if (aAttributeValue.EqualsLiteral("bottom"))
+    }
+    if (aAttributeValue.EqualsLiteral("bottom")) {
       return static_cast<int8_t>(StyleVerticalAlignKeyword::Bottom);
-    else if (aAttributeValue.EqualsLiteral("center"))
+    }
+    if (aAttributeValue.EqualsLiteral("center")) {
       return static_cast<int8_t>(StyleVerticalAlignKeyword::Middle);
-    else
-      return static_cast<int8_t>(StyleVerticalAlignKeyword::Baseline);
-  } else if (aAttribute == nsGkAtoms::columnalign_) {
-    if (aAttributeValue.EqualsLiteral("left"))
-      return NS_STYLE_TEXT_ALIGN_LEFT;
-    else if (aAttributeValue.EqualsLiteral("right"))
-      return NS_STYLE_TEXT_ALIGN_RIGHT;
-    else
-      return NS_STYLE_TEXT_ALIGN_CENTER;
-  } else if (aAttribute == nsGkAtoms::rowlines_ ||
-             aAttribute == nsGkAtoms::columnlines_) {
-    if (aAttributeValue.EqualsLiteral("solid"))
-      return static_cast<int8_t>(StyleBorderStyle::Solid);
-    else if (aAttributeValue.EqualsLiteral("dashed"))
-      return static_cast<int8_t>(StyleBorderStyle::Dashed);
-    else
-      return static_cast<int8_t>(StyleBorderStyle::None);
-  } else {
-    MOZ_CRASH("Unrecognized attribute.");
+    }
+    return static_cast<int8_t>(StyleVerticalAlignKeyword::Baseline);
   }
 
+  if (aAttribute == nsGkAtoms::columnalign) {
+    if (aAttributeValue.EqualsLiteral("left")) {
+      return int8_t(StyleTextAlign::Left);
+    }
+    if (aAttributeValue.EqualsLiteral("right")) {
+      return int8_t(StyleTextAlign::Right);
+    }
+    return int8_t(StyleTextAlign::Center);
+  }
+
+  if (aAttribute == nsGkAtoms::rowlines ||
+      aAttribute == nsGkAtoms::columnlines) {
+    if (aAttributeValue.EqualsLiteral("solid")) {
+      return static_cast<int8_t>(StyleBorderStyle::Solid);
+    }
+    if (aAttributeValue.EqualsLiteral("dashed")) {
+      return static_cast<int8_t>(StyleBorderStyle::Dashed);
+    }
+    return static_cast<int8_t>(StyleBorderStyle::None);
+  }
+
+  MOZ_CRASH("Unrecognized attribute.");
   return -1;
 }
 
@@ -89,7 +99,9 @@ static nsTArray<int8_t>* ExtractStyleValues(const nsAString& aString,
 
     // Grab the value found and process it.
     if (count > 0) {
-      if (!styleArray) styleArray = new nsTArray<int8_t>();
+      if (!styleArray) {
+        styleArray = new nsTArray<int8_t>();
+      }
 
       // We want to return a null array if an attribute gives multiple values,
       // but multiple values aren't allowed.
@@ -113,13 +125,14 @@ static nsresult ReportParseError(nsIFrame* aFrame, const char16_t* aAttribute,
                                  const char16_t* aValue) {
   nsIContent* content = aFrame->GetContent();
 
-  const char16_t* params[] = {
-      aValue, aAttribute, content->NodeInfo()->NameAtom()->GetUTF16String()};
+  AutoTArray<nsString, 3> params;
+  params.AppendElement(aValue);
+  params.AppendElement(aAttribute);
+  params.AppendElement(nsDependentAtomString(content->NodeInfo()->NameAtom()));
 
   return nsContentUtils::ReportToConsole(
-      nsIScriptError::errorFlag, NS_LITERAL_CSTRING("Layout: MathML"),
-      content->OwnerDoc(), nsContentUtils::eMATHML_PROPERTIES,
-      "AttributeParsingError", params, 3);
+      nsIScriptError::errorFlag, "Layout: MathML"_ns, content->OwnerDoc(),
+      nsContentUtils::eMATHML_PROPERTIES, "AttributeParsingError", params);
 }
 
 // Each rowalign='top bottom' or columnalign='left right center' (from
@@ -134,10 +147,16 @@ NS_DECLARE_FRAME_PROPERTY_DELETABLE(ColumnLinesProperty, nsTArray<int8_t>)
 
 static const FramePropertyDescriptor<nsTArray<int8_t>>* AttributeToProperty(
     nsAtom* aAttribute) {
-  if (aAttribute == nsGkAtoms::rowalign_) return RowAlignProperty();
-  if (aAttribute == nsGkAtoms::rowlines_) return RowLinesProperty();
-  if (aAttribute == nsGkAtoms::columnalign_) return ColumnAlignProperty();
-  NS_ASSERTION(aAttribute == nsGkAtoms::columnlines_, "Invalid attribute");
+  if (aAttribute == nsGkAtoms::rowalign) {
+    return RowAlignProperty();
+  }
+  if (aAttribute == nsGkAtoms::rowlines) {
+    return RowLinesProperty();
+  }
+  if (aAttribute == nsGkAtoms::columnalign) {
+    return ColumnAlignProperty();
+  }
+  NS_ASSERTION(aAttribute == nsGkAtoms::columnlines, "Invalid attribute");
   return ColumnLinesProperty();
 }
 
@@ -157,10 +176,11 @@ static nsTArray<int8_t>* FindCellProperty(
     propertyData = currentFrame->GetProperty(aFrameProperty);
     bool frameIsTable = (currentFrame->IsTableFrame());
 
-    if (propertyData || frameIsTable)
+    if (propertyData || frameIsTable) {
       currentFrame = nullptr;  // A null frame pointer exits the loop
-    else
+    } else {
       currentFrame = currentFrame->GetParent();  // Go to the parent frame
+    }
   }
 
   return propertyData;
@@ -178,6 +198,8 @@ static void ApplyBorderToStyle(const nsMathMLmtdFrame* aFrame,
   nsTArray<int8_t>* columnLinesList =
       FindCellProperty(aFrame, ColumnLinesProperty());
 
+  const auto a2d = aFrame->PresContext()->AppUnitsPerDevPixel();
+
   // We don't place a row line on top of the first row
   if (rowIndex > 0 && rowLinesList) {
     // If the row number is greater than the number of provided rowline
@@ -192,7 +214,7 @@ static void ApplyBorderToStyle(const nsMathMLmtdFrame* aFrame,
                                   static_cast<StyleBorderStyle>(
                                       rowLinesList->ElementAt(listLength - 1)));
     }
-    aStyleBorder.SetBorderWidth(eSideTop, borderWidth);
+    aStyleBorder.SetBorderWidth(eSideTop, borderWidth, a2d);
   }
 
   // We don't place a column line on the left of the first column.
@@ -209,7 +231,7 @@ static void ApplyBorderToStyle(const nsMathMLmtdFrame* aFrame,
           eSideLeft, static_cast<StyleBorderStyle>(
                          columnLinesList->ElementAt(listLength - 1)));
     }
-    aStyleBorder.SetBorderWidth(eSideLeft, borderWidth);
+    aStyleBorder.SetBorderWidth(eSideLeft, borderWidth, a2d);
   }
 }
 
@@ -253,29 +275,7 @@ class nsDisplaymtdBorder final : public nsDisplayBorder {
   nsDisplaymtdBorder(nsDisplayListBuilder* aBuilder, nsMathMLmtdFrame* aFrame)
       : nsDisplayBorder(aBuilder, aFrame) {}
 
-  nsDisplayItemGeometry* AllocateGeometry(
-      nsDisplayListBuilder* aBuilder) override {
-    return new nsDisplayItemGenericImageGeometry(this, aBuilder);
-  }
-
-  void ComputeInvalidationRegion(nsDisplayListBuilder* aBuilder,
-                                 const nsDisplayItemGeometry* aGeometry,
-                                 nsRegion* aInvalidRegion) const override {
-    auto geometry =
-        static_cast<const nsDisplayItemGenericImageGeometry*>(aGeometry);
-
-    if (aBuilder->ShouldSyncDecodeImages() &&
-        geometry->ShouldInvalidateToSyncDecodeImages()) {
-      bool snap;
-      aInvalidRegion->Or(*aInvalidRegion, GetBounds(aBuilder, &snap));
-    }
-
-    nsDisplayItem::ComputeInvalidationRegion(aBuilder, aGeometry,
-                                             aInvalidRegion);
-  }
-
-  virtual nsRect GetBounds(nsDisplayListBuilder* aBuilder,
-                           bool* aSnap) const override {
+  nsRect GetBounds(nsDisplayListBuilder* aBuilder, bool* aSnap) const override {
     *aSnap = true;
     nsStyleBorder styleBorder = *mFrame->StyleBorder();
     nsMathMLmtdFrame* frame = static_cast<nsMathMLmtdFrame*>(mFrame);
@@ -286,8 +286,7 @@ class nsDisplaymtdBorder final : public nsDisplayBorder {
     return bounds;
   }
 
-  virtual void Paint(nsDisplayListBuilder* aBuilder,
-                     gfxContext* aCtx) override {
+  void Paint(nsDisplayListBuilder* aBuilder, gfxContext* aCtx) override {
     nsStyleBorder styleBorder = *mFrame->StyleBorder();
     nsMathMLmtdFrame* frame = static_cast<nsMathMLmtdFrame*>(mFrame);
     ApplyBorderToStyle(frame, styleBorder);
@@ -300,11 +299,9 @@ class nsDisplaymtdBorder final : public nsDisplayBorder {
                                  ? PaintBorderFlags::SyncDecodeImages
                                  : PaintBorderFlags();
 
-    ImgDrawResult result = nsCSSRendering::PaintBorderWithStyleBorder(
-        mFrame->PresContext(), *aCtx, mFrame, GetPaintRect(), bounds,
-        styleBorder, mFrame->Style(), flags, mFrame->GetSkipSides());
-
-    nsDisplayItemGenericImageGeometry::UpdateDrawResult(this, result);
+    Unused << nsCSSRendering::PaintBorderWithStyleBorder(
+        mFrame->PresContext(), *aCtx, mFrame, GetPaintRect(aBuilder, aCtx),
+        bounds, styleBorder, mFrame->Style(), flags, mFrame->GetSkipSides());
   }
 
   bool CreateWebRenderCommands(
@@ -315,6 +312,8 @@ class nsDisplaymtdBorder final : public nsDisplayBorder {
       nsDisplayListBuilder* aDisplayListBuilder) override {
     return false;
   }
+
+  bool IsInvisibleInRect(const nsRect& aRect) const override { return false; }
 };
 
 #ifdef DEBUG
@@ -331,7 +330,7 @@ static void ParseFrameAttribute(nsIFrame* aFrame, nsAtom* aAttribute,
   nsAutoString attrValue;
 
   Element* frameElement = aFrame->GetContent()->AsElement();
-  frameElement->GetAttr(kNameSpaceID_None, aAttribute, attrValue);
+  frameElement->GetAttr(aAttribute, attrValue);
 
   if (!attrValue.IsEmpty()) {
     nsTArray<int8_t>* valueList =
@@ -417,9 +416,6 @@ static void ExtractSpacingValues(const nsAString& aString, nsAtom* aAttribute,
                                  nsIFrame* aFrame, nscoord aDefaultValue0,
                                  nscoord aDefaultValue1,
                                  float aFontSizeInflation) {
-  nsPresContext* presContext = aFrame->PresContext();
-  ComputedStyle* computedStyle = aFrame->Style();
-
   const char16_t* start = aString.BeginReading();
   const char16_t* end = aString.EndReading();
 
@@ -446,14 +442,13 @@ static void ExtractSpacingValues(const nsAString& aString, nsAtom* aAttribute,
       nsAutoString valueString;
       valueString.Assign(str);
       nscoord newValue;
-      if (aAttribute == nsGkAtoms::framespacing_ && elementNum) {
+      if (aAttribute == nsGkAtoms::framespacing && elementNum) {
         newValue = aDefaultValue1;
       } else {
         newValue = aDefaultValue0;
       }
-      nsMathMLFrame::ParseNumericValue(
-          valueString, &newValue, nsMathMLElement::PARSE_ALLOW_UNITLESS,
-          presContext, computedStyle, aFontSizeInflation);
+      nsMathMLFrame::ParseAndCalcNumericValue(valueString, &newValue, 0,
+                                              aFontSizeInflation, aFrame);
       aSpacingArray.AppendElement(newValue);
 
       startIndex += count;
@@ -465,18 +460,18 @@ static void ExtractSpacingValues(const nsAString& aString, nsAtom* aAttribute,
 
 static void ParseSpacingAttribute(nsMathMLmtableFrame* aFrame,
                                   nsAtom* aAttribute) {
-  NS_ASSERTION(aAttribute == nsGkAtoms::rowspacing_ ||
-                   aAttribute == nsGkAtoms::columnspacing_ ||
-                   aAttribute == nsGkAtoms::framespacing_,
+  NS_ASSERTION(aAttribute == nsGkAtoms::rowspacing ||
+                   aAttribute == nsGkAtoms::columnspacing ||
+                   aAttribute == nsGkAtoms::framespacing,
                "Non spacing attribute passed");
 
   nsAutoString attrValue;
   Element* frameElement = aFrame->GetContent()->AsElement();
-  frameElement->GetAttr(kNameSpaceID_None, aAttribute, attrValue);
+  frameElement->GetAttr(aAttribute, attrValue);
 
-  if (nsGkAtoms::framespacing_ == aAttribute) {
+  if (nsGkAtoms::framespacing == aAttribute) {
     nsAutoString frame;
-    frameElement->GetAttr(kNameSpaceID_None, nsGkAtoms::frame, frame);
+    frameElement->GetAttr(nsGkAtoms::frame, frame);
     if (frame.IsEmpty() || frame.EqualsLiteral("none")) {
       aFrame->SetFrameSpacing(0, 0);
       return;
@@ -489,10 +484,10 @@ static void ParseSpacingAttribute(nsMathMLmtableFrame* aFrame,
   float fontSizeInflation = nsLayoutUtils::FontSizeInflationFor(aFrame);
   RefPtr<nsFontMetrics> fm =
       nsLayoutUtils::GetFontMetricsForFrame(aFrame, fontSizeInflation);
-  if (nsGkAtoms::rowspacing_ == aAttribute) {
+  if (nsGkAtoms::rowspacing == aAttribute) {
     value = kDefaultRowspacingEx * fm->XHeight();
     value2 = 0;
-  } else if (nsGkAtoms::columnspacing_ == aAttribute) {
+  } else if (nsGkAtoms::columnspacing == aAttribute) {
     value = kDefaultColumnspacingEm * fm->EmHeight();
     value2 = 0;
   } else {
@@ -504,14 +499,14 @@ static void ParseSpacingAttribute(nsMathMLmtableFrame* aFrame,
   ExtractSpacingValues(attrValue, aAttribute, valueList, aFrame, value, value2,
                        fontSizeInflation);
   if (valueList.Length() == 0) {
-    if (frameElement->HasAttr(kNameSpaceID_None, aAttribute)) {
+    if (frameElement->HasAttr(aAttribute)) {
       ReportParseError(aFrame, aAttribute->GetUTF16String(), attrValue.get());
     }
     valueList.AppendElement(value);
   }
-  if (aAttribute == nsGkAtoms::framespacing_) {
+  if (aAttribute == nsGkAtoms::framespacing) {
     if (valueList.Length() == 1) {
-      if (frameElement->HasAttr(kNameSpaceID_None, aAttribute)) {
+      if (frameElement->HasAttr(aAttribute)) {
         ReportParseError(aFrame, aAttribute->GetUTF16String(), attrValue.get());
       }
       valueList.AppendElement(value2);
@@ -520,9 +515,9 @@ static void ParseSpacingAttribute(nsMathMLmtableFrame* aFrame,
     }
   }
 
-  if (aAttribute == nsGkAtoms::rowspacing_) {
+  if (aAttribute == nsGkAtoms::rowspacing) {
     aFrame->SetRowSpacingArray(valueList);
-  } else if (aAttribute == nsGkAtoms::columnspacing_) {
+  } else if (aAttribute == nsGkAtoms::columnspacing) {
     aFrame->SetColSpacingArray(valueList);
   } else {
     aFrame->SetFrameSpacing(valueList.ElementAt(0), valueList.ElementAt(1));
@@ -530,9 +525,9 @@ static void ParseSpacingAttribute(nsMathMLmtableFrame* aFrame,
 }
 
 static void ParseSpacingAttributes(nsMathMLmtableFrame* aTableFrame) {
-  ParseSpacingAttribute(aTableFrame, nsGkAtoms::rowspacing_);
-  ParseSpacingAttribute(aTableFrame, nsGkAtoms::columnspacing_);
-  ParseSpacingAttribute(aTableFrame, nsGkAtoms::framespacing_);
+  ParseSpacingAttribute(aTableFrame, nsGkAtoms::rowspacing);
+  ParseSpacingAttribute(aTableFrame, nsGkAtoms::columnspacing);
+  ParseSpacingAttribute(aTableFrame, nsGkAtoms::framespacing);
   aTableFrame->SetUseCSSSpacing();
 }
 
@@ -540,35 +535,37 @@ static void ParseSpacingAttributes(nsMathMLmtableFrame* aTableFrame) {
 // so it can only happen after they are made ready by the table base class.
 static void MapAllAttributesIntoCSS(nsMathMLmtableFrame* aTableFrame) {
   // Map mtable rowalign & rowlines.
-  ParseFrameAttribute(aTableFrame, nsGkAtoms::rowalign_, true);
-  ParseFrameAttribute(aTableFrame, nsGkAtoms::rowlines_, true);
+  ParseFrameAttribute(aTableFrame, nsGkAtoms::rowalign, true);
+  ParseFrameAttribute(aTableFrame, nsGkAtoms::rowlines, true);
 
   // Map mtable columnalign & columnlines.
-  ParseFrameAttribute(aTableFrame, nsGkAtoms::columnalign_, true);
-  ParseFrameAttribute(aTableFrame, nsGkAtoms::columnlines_, true);
+  ParseFrameAttribute(aTableFrame, nsGkAtoms::columnalign, true);
+  ParseFrameAttribute(aTableFrame, nsGkAtoms::columnlines, true);
 
   // Map mtable rowspacing, columnspacing & framespacing
   ParseSpacingAttributes(aTableFrame);
 
   // mtable is simple and only has one (pseudo) row-group
   nsIFrame* rgFrame = aTableFrame->PrincipalChildList().FirstChild();
-  if (!rgFrame || !rgFrame->IsTableRowGroupFrame()) return;
+  if (!rgFrame || !rgFrame->IsTableRowGroupFrame()) {
+    return;
+  }
 
   for (nsIFrame* rowFrame : rgFrame->PrincipalChildList()) {
     DEBUG_VERIFY_THAT_FRAME_IS(rowFrame, TableRow);
     if (rowFrame->IsTableRowFrame()) {
       // Map row rowalign.
-      ParseFrameAttribute(rowFrame, nsGkAtoms::rowalign_, false);
+      ParseFrameAttribute(rowFrame, nsGkAtoms::rowalign, false);
       // Map row columnalign.
-      ParseFrameAttribute(rowFrame, nsGkAtoms::columnalign_, true);
+      ParseFrameAttribute(rowFrame, nsGkAtoms::columnalign, true);
 
       for (nsIFrame* cellFrame : rowFrame->PrincipalChildList()) {
         DEBUG_VERIFY_THAT_FRAME_IS(cellFrame, TableCell);
-        if (IsTableCell(cellFrame->Type())) {
+        if (cellFrame->IsTableCellFrame()) {
           // Map cell rowalign.
-          ParseFrameAttribute(cellFrame, nsGkAtoms::rowalign_, false);
+          ParseFrameAttribute(cellFrame, nsGkAtoms::rowalign, false);
           // Map row columnalign.
-          ParseFrameAttribute(cellFrame, nsGkAtoms::columnalign_, false);
+          ParseFrameAttribute(cellFrame, nsGkAtoms::columnalign, false);
         }
       }
     }
@@ -605,19 +602,19 @@ static void ParseAlignAttribute(nsString& aValue, eAlign& aAlign,
   // ToInteger ignores the whitespaces around the number
   aValue.CompressWhitespace(true, false);
 
-  if (0 == aValue.Find("top")) {
+  if (0 == aValue.Find(u"top")) {
     len = 3;  // 3 is the length of 'top'
     aAlign = eAlign_top;
-  } else if (0 == aValue.Find("bottom")) {
+  } else if (0 == aValue.Find(u"bottom")) {
     len = 6;  // 6 is the length of 'bottom'
     aAlign = eAlign_bottom;
-  } else if (0 == aValue.Find("center")) {
+  } else if (0 == aValue.Find(u"center")) {
     len = 6;  // 6 is the length of 'center'
     aAlign = eAlign_center;
-  } else if (0 == aValue.Find("baseline")) {
+  } else if (0 == aValue.Find(u"baseline")) {
     len = 8;  // 8 is the length of 'baseline'
     aAlign = eAlign_baseline;
-  } else if (0 == aValue.Find("axis")) {
+  } else if (0 == aValue.Find(u"axis")) {
     len = 4;  // 4 is the length of 'axis'
     aAlign = eAlign_axis;
   }
@@ -625,27 +622,12 @@ static void ParseAlignAttribute(nsString& aValue, eAlign& aAlign,
     nsresult error;
     aValue.Cut(0, len);  // aValue is not a const here
     aRowIndex = aValue.ToInteger(&error);
-    if (NS_FAILED(error)) aRowIndex = 0;
+    if (NS_FAILED(error)) {
+      aRowIndex = 0;
+    }
   }
 }
 
-#ifdef DEBUG_rbs_off
-// call ListMathMLTree(mParent) to get the big picture
-static void ListMathMLTree(nsIFrame* atLeast) {
-  // climb up to <math> or <body> if <math> isn't there
-  nsIFrame* f = atLeast;
-  for (; f; f = f->GetParent()) {
-    nsIContent* c = f->GetContent();
-    if (!c || c->IsMathMLElement(nsGkAtoms::math) ||
-        // XXXbaku which kind of body tag?
-        c->NodeInfo()->NameAtom(nsGkAtoms::body))
-      break;
-  }
-  if (!f) f = atLeast;
-  f->List(stdout, 0);
-}
-#endif
-
 // --------
 // implementation of nsMathMLmtableWrapperFrame
 
@@ -661,7 +643,7 @@ nsContainerFrame* NS_NewMathMLmtableOuterFrame(PresShell* aPresShell,
 
 NS_IMPL_FRAMEARENA_HELPERS(nsMathMLmtableWrapperFrame)
 
-nsMathMLmtableWrapperFrame::~nsMathMLmtableWrapperFrame() {}
+nsMathMLmtableWrapperFrame::~nsMathMLmtableWrapperFrame() = default;
 
 nsresult nsMathMLmtableWrapperFrame::AttributeChanged(int32_t aNameSpaceID,
                                                       nsAtom* aAttribute,
@@ -686,56 +668,49 @@ nsresult nsMathMLmtableWrapperFrame::AttributeChanged(int32_t aNameSpaceID,
   NS_ASSERTION(tableFrame && tableFrame->IsTableFrame(),
                "should always have an inner table frame");
   nsIFrame* rgFrame = tableFrame->PrincipalChildList().FirstChild();
-  if (!rgFrame || !rgFrame->IsTableRowGroupFrame()) return NS_OK;
-
-  // align - just need to issue a dirty (resize) reflow command
-  if (aAttribute == nsGkAtoms::align) {
-    PresShell()->FrameNeedsReflow(this, IntrinsicDirty::Resize,
-                                  NS_FRAME_IS_DIRTY);
+  if (!rgFrame || !rgFrame->IsTableRowGroupFrame()) {
     return NS_OK;
   }
 
-  // displaystyle - may seem innocuous, but it is actually very harsh --
-  // like changing an unit. Blow away and recompute all our automatic
-  // presentational data, and issue a style-changed reflow request
-  if (aAttribute == nsGkAtoms::displaystyle_) {
-    nsMathMLContainerFrame::RebuildAutomaticDataForChildren(GetParent());
-    // Need to reflow the parent, not us, because this can actually
-    // affect siblings.
-    PresShell()->FrameNeedsReflow(GetParent(), IntrinsicDirty::StyleChange,
+  // align - just need to issue a dirty (resize) reflow command
+  if (aNameSpaceID == kNameSpaceID_None && aAttribute == nsGkAtoms::align) {
+    PresShell()->FrameNeedsReflow(this, IntrinsicDirty::None,
                                   NS_FRAME_IS_DIRTY);
     return NS_OK;
   }
 
   // ...and the other attributes affect rows or columns in one way or another
 
-  nsPresContext* presContext = tableFrame->PresContext();
-  if (aAttribute == nsGkAtoms::rowspacing_ ||
-      aAttribute == nsGkAtoms::columnspacing_ ||
-      aAttribute == nsGkAtoms::framespacing_) {
+  if (aNameSpaceID == kNameSpaceID_None &&
+      (aAttribute == nsGkAtoms::rowspacing ||
+       aAttribute == nsGkAtoms::columnspacing ||
+       aAttribute == nsGkAtoms::framespacing)) {
     nsMathMLmtableFrame* mathMLmtableFrame = do_QueryFrame(tableFrame);
     if (mathMLmtableFrame) {
       ParseSpacingAttribute(mathMLmtableFrame, aAttribute);
       mathMLmtableFrame->SetUseCSSSpacing();
     }
-  } else if (aAttribute == nsGkAtoms::rowalign_ ||
-             aAttribute == nsGkAtoms::rowlines_ ||
-             aAttribute == nsGkAtoms::columnalign_ ||
-             aAttribute == nsGkAtoms::columnlines_) {
-    // clear any cached property list for this table
-    tableFrame->DeleteProperty(AttributeToProperty(aAttribute));
-    // Reparse the new attribute on the table.
-    ParseFrameAttribute(tableFrame, aAttribute, true);
-  } else {
-    // Ignore attributes that do not affect layout.
+    PresShell()->FrameNeedsReflow(
+        this, IntrinsicDirty::FrameAncestorsAndDescendants, NS_FRAME_IS_DIRTY);
     return NS_OK;
   }
 
-  // Explicitly request a reflow in our subtree to pick up any changes
-  presContext->PresShell()->FrameNeedsReflow(this, IntrinsicDirty::StyleChange,
-                                             NS_FRAME_IS_DIRTY);
+  if (aNameSpaceID == kNameSpaceID_None &&
+      (aAttribute == nsGkAtoms::rowalign || aAttribute == nsGkAtoms::rowlines ||
+       aAttribute == nsGkAtoms::columnalign ||
+       aAttribute == nsGkAtoms::columnlines)) {
+    // clear any cached property list for this table
+    tableFrame->RemoveProperty(AttributeToProperty(aAttribute));
+    // Reparse the new attribute on the table.
+    ParseFrameAttribute(tableFrame, aAttribute, true);
+    PresShell()->FrameNeedsReflow(
+        this, IntrinsicDirty::FrameAncestorsAndDescendants, NS_FRAME_IS_DIRTY);
+    return NS_OK;
+  }
 
-  return NS_OK;
+  // Skip nsTableWrapperFrame::AttributeChanged, mtable does not share more
+  // attributes with table.
+  return nsContainerFrame::AttributeChanged(aNameSpaceID, aAttribute, aModType);
 }
 
 nsIFrame* nsMathMLmtableWrapperFrame::GetRowFrameAt(int32_t aRowIndex) {
@@ -755,11 +730,15 @@ nsIFrame* nsMathMLmtableWrapperFrame::GetRowFrameAt(int32_t aRowIndex) {
     NS_ASSERTION(tableFrame && tableFrame->IsTableFrame(),
                  "should always have an inner table frame");
     nsIFrame* rgFrame = tableFrame->PrincipalChildList().FirstChild();
-    if (!rgFrame || !rgFrame->IsTableRowGroupFrame()) return nullptr;
+    if (!rgFrame || !rgFrame->IsTableRowGroupFrame()) {
+      return nullptr;
+    }
     for (nsIFrame* rowFrame : rgFrame->PrincipalChildList()) {
       if (aRowIndex == 0) {
         DEBUG_VERIFY_THAT_FRAME_IS(rowFrame, TableRow);
-        if (!rowFrame->IsTableRowFrame()) return nullptr;
+        if (!rowFrame->IsTableRowFrame()) {
+          return nullptr;
+        }
 
         return rowFrame;
       }
@@ -786,7 +765,7 @@ void nsMathMLmtableWrapperFrame::Reflow(nsPresContext* aPresContext,
   // see if the user has set the align attribute on the <mtable>
   int32_t rowIndex = 0;
   eAlign tableAlign = eAlign_axis;
-  mContent->AsElement()->GetAttr(kNameSpaceID_None, nsGkAtoms::align, value);
+  mContent->AsElement()->GetAttr(nsGkAtoms::align, value);
   if (!value.IsEmpty()) {
     ParseAlignAttribute(value, tableAlign, rowIndex);
   }
@@ -875,7 +854,6 @@ void nsMathMLmtableWrapperFrame::Reflow(nsPresContext* aPresContext,
   mBoundingMetrics.rightBearing = aDesiredSize.Width();
 
   aDesiredSize.mBoundingMetrics = mBoundingMetrics;
-  NS_FRAME_SET_TRUNCATION(aStatus, aReflowInput, aDesiredSize);
 }
 
 nsContainerFrame* NS_NewMathMLmtableFrame(PresShell* aPresShell,
@@ -886,11 +864,11 @@ nsContainerFrame* NS_NewMathMLmtableFrame(PresShell* aPresShell,
 
 NS_IMPL_FRAMEARENA_HELPERS(nsMathMLmtableFrame)
 
-nsMathMLmtableFrame::~nsMathMLmtableFrame() {}
+nsMathMLmtableFrame::~nsMathMLmtableFrame() = default;
 
 void nsMathMLmtableFrame::SetInitialChildList(ChildListID aListID,
-                                              nsFrameList& aChildList) {
-  nsTableFrame::SetInitialChildList(aListID, aChildList);
+                                              nsFrameList&& aChildList) {
+  nsTableFrame::SetInitialChildList(aListID, std::move(aChildList));
   MapAllAttributesIntoCSS(this);
 }
 
@@ -1018,12 +996,10 @@ nscoord nsMathMLmtableFrame::GetRowSpacing(int32_t aStartRowIndex,
 }
 
 void nsMathMLmtableFrame::SetUseCSSSpacing() {
-  mUseCSSSpacing = !(mContent->AsElement()->HasAttr(kNameSpaceID_None,
-                                                    nsGkAtoms::rowspacing_) ||
-                     mContent->AsElement()->HasAttr(
-                         kNameSpaceID_None, nsGkAtoms::columnspacing_) ||
+  mUseCSSSpacing = !(mContent->AsElement()->HasAttr(nsGkAtoms::rowspacing) ||
                      mContent->AsElement()->HasAttr(kNameSpaceID_None,
-                                                    nsGkAtoms::framespacing_));
+                                                    nsGkAtoms::columnspacing) ||
+                     mContent->AsElement()->HasAttr(nsGkAtoms::framespacing));
 }
 
 NS_QUERYFRAME_HEAD(nsMathMLmtableFrame)
@@ -1041,7 +1017,7 @@ nsContainerFrame* NS_NewMathMLmtrFrame(PresShell* aPresShell,
 
 NS_IMPL_FRAMEARENA_HELPERS(nsMathMLmtrFrame)
 
-nsMathMLmtrFrame::~nsMathMLmtrFrame() {}
+nsMathMLmtrFrame::~nsMathMLmtrFrame() = default;
 
 nsresult nsMathMLmtrFrame::AttributeChanged(int32_t aNameSpaceID,
                                             nsAtom* aAttribute,
@@ -1051,23 +1027,25 @@ nsresult nsMathMLmtrFrame::AttributeChanged(int32_t aNameSpaceID,
   // rowalign    : Here
   // columnalign : Here
 
-  nsPresContext* presContext = PresContext();
-
-  if (aAttribute != nsGkAtoms::rowalign_ &&
-      aAttribute != nsGkAtoms::columnalign_) {
-    return NS_OK;
+  if (aNameSpaceID != kNameSpaceID_None ||
+      (aAttribute != nsGkAtoms::rowalign &&
+       aAttribute != nsGkAtoms::columnalign)) {
+    // Skip nsTableCellFrame::AttributeChanged, mtr does not share any attribute
+    // with tr.
+    return nsContainerFrame::AttributeChanged(aNameSpaceID, aAttribute,
+                                              aModType);
   }
 
-  DeleteProperty(AttributeToProperty(aAttribute));
+  RemoveProperty(AttributeToProperty(aAttribute));
 
-  bool allowMultiValues = (aAttribute == nsGkAtoms::columnalign_);
+  bool allowMultiValues = (aAttribute == nsGkAtoms::columnalign);
 
   // Reparse the new attribute.
   ParseFrameAttribute(this, aAttribute, allowMultiValues);
 
   // Explicitly request a reflow in our subtree to pick up any changes
-  presContext->PresShell()->FrameNeedsReflow(this, IntrinsicDirty::StyleChange,
-                                             NS_FRAME_IS_DIRTY);
+  PresShell()->FrameNeedsReflow(
+      this, IntrinsicDirty::FrameAncestorsAndDescendants, NS_FRAME_IS_DIRTY);
 
   return NS_OK;
 }
@@ -1083,7 +1061,7 @@ nsContainerFrame* NS_NewMathMLmtdFrame(PresShell* aPresShell,
 
 NS_IMPL_FRAMEARENA_HELPERS(nsMathMLmtdFrame)
 
-nsMathMLmtdFrame::~nsMathMLmtdFrame() {}
+nsMathMLmtdFrame::~nsMathMLmtdFrame() = default;
 
 void nsMathMLmtdFrame::Init(nsIContent* aContent, nsContainerFrame* aParent,
                             nsIFrame* aPrevInFlow) {
@@ -1104,24 +1082,27 @@ nsresult nsMathMLmtdFrame::AttributeChanged(int32_t aNameSpaceID,
   // rowspan     : here
   // columnspan  : here
 
-  if (aAttribute == nsGkAtoms::rowalign_ ||
-      aAttribute == nsGkAtoms::columnalign_) {
-    DeleteProperty(AttributeToProperty(aAttribute));
+  if (aNameSpaceID == kNameSpaceID_None &&
+      (aAttribute == nsGkAtoms::rowalign ||
+       aAttribute == nsGkAtoms::columnalign)) {
+    RemoveProperty(AttributeToProperty(aAttribute));
 
     // Reparse the attribute.
     ParseFrameAttribute(this, aAttribute, false);
     return NS_OK;
   }
 
-  if (aAttribute == nsGkAtoms::rowspan ||
-      aAttribute == nsGkAtoms::columnspan_) {
-    // use the naming expected by the base class
-    if (aAttribute == nsGkAtoms::columnspan_) aAttribute = nsGkAtoms::colspan;
+  if (aNameSpaceID == kNameSpaceID_None &&
+      (aAttribute == nsGkAtoms::rowspan ||
+       aAttribute == nsGkAtoms::columnspan)) {
+    // nsTableCellFrame takes care of renaming columnspan to colspan.
     return nsTableCellFrame::AttributeChanged(aNameSpaceID, aAttribute,
                                               aModType);
   }
 
-  return NS_OK;
+  // Skip nsTableCellFrame::AttributeChanged, mtd does not share more attributes
+  // with td.
+  return nsContainerFrame::AttributeChanged(aNameSpaceID, aAttribute, aModType);
 }
 
 StyleVerticalAlignKeyword nsMathMLmtdFrame::GetVerticalAlign() const {
@@ -1144,11 +1125,10 @@ StyleVerticalAlignKeyword nsMathMLmtdFrame::GetVerticalAlign() const {
   return alignment;
 }
 
-nsresult nsMathMLmtdFrame::ProcessBorders(nsTableFrame* aFrame,
-                                          nsDisplayListBuilder* aBuilder,
-                                          const nsDisplayListSet& aLists) {
+void nsMathMLmtdFrame::ProcessBorders(nsTableFrame* aFrame,
+                                      nsDisplayListBuilder* aBuilder,
+                                      const nsDisplayListSet& aLists) {
   aLists.BorderBackground()->AppendNewToTop<nsDisplaymtdBorder>(aBuilder, this);
-  return NS_OK;
 }
 
 LogicalMargin nsMathMLmtdFrame::GetBorderWidth(WritingMode aWM) const {
@@ -1199,7 +1179,7 @@ void nsMathMLmtdInnerFrame::Reflow(nsPresContext* aPresContext,
 
 const nsStyleText* nsMathMLmtdInnerFrame::StyleTextForLineLayout() {
   // Set the default alignment in case nothing was specified
-  uint8_t alignment = StyleText()->mTextAlign;
+  auto alignment = uint8_t(StyleText()->mTextAlign);
 
   nsTArray<int8_t>* alignmentList =
       FindCellProperty(this, ColumnAlignProperty());
@@ -1210,13 +1190,14 @@ const nsStyleText* nsMathMLmtdInnerFrame::StyleTextForLineLayout() {
 
     // If the column number is greater than the number of provided columalign
     // values, we simply repeat the last value.
-    if (columnIndex < alignmentList->Length())
+    if (columnIndex < alignmentList->Length()) {
       alignment = alignmentList->ElementAt(columnIndex);
-    else
+    } else {
       alignment = alignmentList->ElementAt(alignmentList->Length() - 1);
+    }
   }
 
-  mUniqueStyleText->mTextAlign = alignment;
+  mUniqueStyleText->mTextAlign = StyleTextAlign(alignment);
   return mUniqueStyleText.get();
 }
 
