@@ -12,14 +12,12 @@
 
 using namespace mozilla::css;
 
-namespace mozilla {
-namespace dom {
+namespace mozilla::dom {
 
-CSSSupportsRule::CSSSupportsRule(RefPtr<RawServoSupportsRule> aRawRule,
+CSSSupportsRule::CSSSupportsRule(RefPtr<StyleSupportsRule> aRawRule,
                                  StyleSheet* aSheet, css::Rule* aParentRule,
                                  uint32_t aLine, uint32_t aColumn)
-    : css::ConditionRule(Servo_SupportsRule_GetRules(aRawRule).Consume(),
-                         aSheet, aParentRule, aLine, aColumn),
+    : css::ConditionRule(aSheet, aParentRule, aLine, aColumn),
       mRawRule(std::move(aRawRule)) {}
 
 NS_IMPL_ADDREF_INHERITED(CSSSupportsRule, ConditionRule)
@@ -41,17 +39,12 @@ void CSSSupportsRule::List(FILE* out, int32_t aIndent) const {
 }
 #endif
 
-void CSSSupportsRule::GetConditionText(nsACString& aConditionText) {
-  Servo_SupportsRule_GetConditionText(mRawRule, &aConditionText);
+StyleCssRuleType CSSSupportsRule::Type() const {
+  return StyleCssRuleType::Supports;
 }
 
-void CSSSupportsRule::SetConditionText(const nsACString& aConditionText,
-                                       ErrorResult& aRv) {
-  if (IsReadOnly()) {
-    return;
-  }
-
-  aRv.Throw(NS_ERROR_NOT_IMPLEMENTED);
+void CSSSupportsRule::GetConditionText(nsACString& aConditionText) {
+  Servo_SupportsRule_GetConditionText(mRawRule, &aConditionText);
 }
 
 /* virtual */
@@ -59,6 +52,15 @@ void CSSSupportsRule::GetCssText(nsACString& aCssText) const {
   Servo_SupportsRule_GetCssText(mRawRule, &aCssText);
 }
 
+void CSSSupportsRule::SetRawAfterClone(RefPtr<StyleSupportsRule> aRaw) {
+  mRawRule = std::move(aRaw);
+  css::ConditionRule::DidSetRawAfterClone();
+}
+
+already_AddRefed<StyleLockedCssRules> CSSSupportsRule::GetOrCreateRawRules() {
+  return Servo_SupportsRule_GetRules(mRawRule).Consume();
+}
+
 /* virtual */
 size_t CSSSupportsRule::SizeOfIncludingThis(MallocSizeOf aMallocSizeOf) const {
   // TODO Implement this!
@@ -71,5 +73,4 @@ JSObject* CSSSupportsRule::WrapObject(JSContext* aCx,
   return CSSSupportsRule_Binding::Wrap(aCx, this, aGivenProto);
 }
 
-}  // namespace dom
-}  // namespace mozilla
+}  // namespace mozilla::dom
