@@ -8,6 +8,7 @@
 
 #include "ServoCSSParser.h"
 
+#include "mozilla/AnimatedPropertyID.h"
 #include "mozilla/ServoBindings.h"
 #include "mozilla/ServoStyleSet.h"
 #include "mozilla/dom/Document.h"
@@ -26,17 +27,37 @@ bool ServoCSSParser::ComputeColor(ServoStyleSet* aStyleSet,
                                   const nsACString& aValue,
                                   nscolor* aResultColor, bool* aWasCurrentColor,
                                   css::Loader* aLoader) {
-  return Servo_ComputeColor(aStyleSet ? aStyleSet->RawSet() : nullptr,
+  return Servo_ComputeColor(aStyleSet ? aStyleSet->RawData() : nullptr,
                             aCurrentColor, &aValue, aResultColor,
                             aWasCurrentColor, aLoader);
 }
 
 /* static */
-already_AddRefed<RawServoDeclarationBlock> ServoCSSParser::ParseProperty(
+bool ServoCSSParser::ColorTo(const nsACString& aFromColor,
+                             const nsACString& aToColorSpace,
+                             nsACString* aResultColor,
+                             nsTArray<float>* aResultComponents,
+                             bool* aResultAdjusted, css::Loader* aLoader) {
+  return Servo_ColorTo(&aFromColor, &aToColorSpace, aResultColor,
+                       aResultComponents, aResultAdjusted, aLoader);
+}
+
+/* static */
+already_AddRefed<StyleLockedDeclarationBlock> ServoCSSParser::ParseProperty(
     nsCSSPropertyID aProperty, const nsACString& aValue,
-    const ParsingEnvironment& aParsingEnvironment, ParsingMode aParsingMode) {
+    const ParsingEnvironment& aParsingEnvironment,
+    const StyleParsingMode& aParsingMode) {
+  AnimatedPropertyID property(aProperty);
+  return ParseProperty(property, aValue, aParsingEnvironment, aParsingMode);
+}
+
+/* static */
+already_AddRefed<StyleLockedDeclarationBlock> ServoCSSParser::ParseProperty(
+    const AnimatedPropertyID& aProperty, const nsACString& aValue,
+    const ParsingEnvironment& aParsingEnvironment,
+    const StyleParsingMode& aParsingMode) {
   return Servo_ParseProperty(
-             aProperty, &aValue, aParsingEnvironment.mUrlExtraData,
+             &aProperty, &aValue, aParsingEnvironment.mUrlExtraData,
              aParsingMode, aParsingEnvironment.mCompatMode,
              aParsingEnvironment.mLoader, aParsingEnvironment.mRuleType)
       .Consume();
@@ -44,7 +65,7 @@ already_AddRefed<RawServoDeclarationBlock> ServoCSSParser::ParseProperty(
 
 /* static */
 bool ServoCSSParser::ParseEasing(const nsACString& aValue,
-                                 nsTimingFunction& aResult) {
+                                 StyleComputedTimingFunction& aResult) {
   return Servo_ParseEasing(&aValue, &aResult);
 }
 
@@ -59,23 +80,17 @@ bool ServoCSSParser::ParseTransformIntoMatrix(const nsACString& aValue,
 /* static */
 bool ServoCSSParser::ParseFontShorthandForMatching(
     const nsACString& aValue, URLExtraData* aUrl, StyleFontFamilyList& aList,
-    StyleComputedFontStyleDescriptor& aStyle, float& aStretch, float& aWeight) {
-  return Servo_ParseFontShorthandForMatching(&aValue, aUrl, &aList, &aStyle,
-                                             &aStretch, &aWeight);
+    StyleFontStyle& aStyle, StyleFontStretch& aStretch,
+    StyleFontWeight& aWeight, float* aSize, bool* aSmallCaps) {
+  return Servo_ParseFontShorthandForMatching(
+      &aValue, aUrl, &aList, &aStyle, &aStretch, &aWeight, aSize, aSmallCaps);
 }
 
 /* static */
 already_AddRefed<URLExtraData> ServoCSSParser::GetURLExtraData(
     Document* aDocument) {
   MOZ_ASSERT(aDocument);
-
-  nsCOMPtr<nsIReferrerInfo> referrerInfo =
-      ReferrerInfo::CreateForInternalCSSResources(aDocument);
-
-  // FIXME this is using the wrong base uri (bug 1343919)
-  RefPtr<URLExtraData> url = new URLExtraData(
-      aDocument->GetDocumentURI(), referrerInfo, aDocument->NodePrincipal());
-  return url.forget();
+  return do_AddRef(aDocument->DefaultStyleAttrURLData());
 }
 
 /* static */ ServoCSSParser::ParsingEnvironment
