@@ -9,11 +9,12 @@
 #include "nsICSSDeclaration.h"
 
 #include "nsINode.h"
+#include "mozilla/css/Rule.h"
 
 using mozilla::dom::DocGroup;
 
 DocGroup* nsICSSDeclaration::GetDocGroup() {
-  nsINode* parentNode = GetParentObject();
+  nsINode* parentNode = GetAssociatedNode();
   if (!parentNode) {
     return nullptr;
   }
