@@ -10,17 +10,13 @@
 // Keep others in (case-insensitive) order:
 #include "gfxDrawable.h"
 
-#include "Layers.h"
 #include "nsCSSAnonBoxes.h"
 #include "nsCSSRendering.h"
 #include "nsDisplayList.h"
 #include "nsLayoutUtils.h"
 #include "gfxContext.h"
 #include "SVGPaintServerFrame.h"
-#include "FrameLayerBuilder.h"
-#include "BasicLayers.h"
 #include "mozilla/gfx/Point.h"
-#include "mozilla/gfx/gfxVars.h"
 #include "mozilla/CSSClipPathInstance.h"
 #include "mozilla/FilterInstance.h"
 #include "mozilla/StaticPrefs_layers.h"
@@ -66,7 +62,7 @@ class PreEffectsInkOverflowCollector : public nsLayoutUtils::BoxCallback {
                  "We want the first continuation here");
   }
 
-  virtual void AddBox(nsIFrame* aFrame) override {
+  void AddBox(nsIFrame* aFrame) override {
     nsRect overflow = (aFrame == mCurrentFrame)
                           ? mCurrentFrameOverflowArea
                           : PreEffectsInkOverflowRect(aFrame, mInReflow);
@@ -167,25 +163,6 @@ bool SVGIntegrationUtils::UsingEffectsForFrame(const nsIFrame* aFrame) {
          style->HasClipPath() || style->HasMask();
 }
 
-bool SVGIntegrationUtils::UsingMaskOrClipPathForFrame(const nsIFrame* aFrame) {
-  const nsStyleSVGReset* style = aFrame->StyleSVGReset();
-  return style->HasClipPath() || style->HasMask();
-}
-
-bool SVGIntegrationUtils::UsingSimpleClipPathForFrame(const nsIFrame* aFrame) {
-  const nsStyleSVGReset* style = aFrame->StyleSVGReset();
-  if (!style->HasClipPath() || style->HasMask()) {
-    return false;
-  }
-
-  const auto& clipPath = style->mClipPath;
-  if (!clipPath.IsShape()) {
-    return false;
-  }
-
-  return !clipPath.AsShape()._0->IsPolygon();
-}
-
 nsPoint SVGIntegrationUtils::GetOffsetToBoundingBox(nsIFrame* aFrame) {
   if (aFrame->HasAnyStateBits(NS_FRAME_SVG_LAYOUT)) {
     // Do NOT call GetAllInFlowRectsUnion for SVG - it will get the
@@ -220,7 +197,7 @@ static EffectOffsets ComputeEffectOffset(
   result.offsetToBoundingBox =
       aParams.builder->ToReferenceFrame(aFrame) -
       SVGIntegrationUtils::GetOffsetToBoundingBox(aFrame);
-  if (!aFrame->IsFrameOfType(nsIFrame::eSVG)) {
+  if (!aFrame->IsSVGFrame()) {
     /* Snap the offset if the reference frame is not a SVG frame,
      * since other frames will be snapped to pixel when rendering. */
     result.offsetToBoundingBox =
@@ -286,8 +263,7 @@ gfxPoint SVGIntegrationUtils::GetOffsetToUserSpaceInDevPx(
 
 /* static */
 nsSize SVGIntegrationUtils::GetContinuationUnionSize(nsIFrame* aNonSVGFrame) {
-  NS_ASSERTION(!aNonSVGFrame->IsFrameOfType(nsIFrame::eSVG),
-               "SVG frames should not get here");
+  NS_ASSERTION(!aNonSVGFrame->IsSVGFrame(), "SVG frames should not get here");
   nsIFrame* firstFrame =
       nsLayoutUtils::FirstContinuationOrIBSplitSibling(aNonSVGFrame);
   return nsLayoutUtils::GetAllInFlowRectsUnion(firstFrame, firstFrame).Size();
@@ -295,8 +271,7 @@ nsSize SVGIntegrationUtils::GetContinuationUnionSize(nsIFrame* aNonSVGFrame) {
 
 /* static */ gfx::Size SVGIntegrationUtils::GetSVGCoordContextForNonSVGFrame(
     nsIFrame* aNonSVGFrame) {
-  NS_ASSERTION(!aNonSVGFrame->IsFrameOfType(nsIFrame::eSVG),
-               "SVG frames should not get here");
+  NS_ASSERTION(!aNonSVGFrame->IsSVGFrame(), "SVG frames should not get here");
   nsIFrame* firstFrame =
       nsLayoutUtils::FirstContinuationOrIBSplitSibling(aNonSVGFrame);
   nsRect r = nsLayoutUtils::GetAllInFlowRectsUnion(firstFrame, firstFrame);
@@ -311,8 +286,6 @@ gfxRect SVGIntegrationUtils::GetSVGBBoxForNonSVGFrame(
   // CSS box model rules.
   NS_ASSERTION(!aNonSVGFrame->HasAnyStateBits(NS_FRAME_SVG_LAYOUT),
                "Frames with SVG layout should not get here");
-  MOZ_ASSERT(!aNonSVGFrame->IsFrameOfType(nsIFrame::eSVG) ||
-             aNonSVGFrame->IsSVGOuterSVGFrame());
 
   nsIFrame* firstFrame =
       nsLayoutUtils::FirstContinuationOrIBSplitSibling(aNonSVGFrame);
@@ -372,12 +345,10 @@ nsRect SVGIntegrationUtils::ComputePostEffectsInkOverflowRect(
       nsLayoutUtils::FirstContinuationOrIBSplitSibling(aFrame);
   // Note: we do not return here for eHasNoRefs since we must still handle any
   // CSS filter functions.
-  // TODO: We currently pass nullptr instead of an nsTArray* here, but we
-  // actually should get the filter frames and then pass them into
-  // GetPostFilterBounds below!  See bug 1494263.
   // TODO: we should really return an empty rect for eHasRefsSomeInvalid since
   // in that case we disable painting of the element.
-  if (SVGObserverUtils::GetAndObserveFilters(firstFrame, nullptr) ==
+  nsTArray<SVGFilterFrame*> filterFrames;
+  if (SVGObserverUtils::GetAndObserveFilters(firstFrame, &filterFrames) ==
       SVGObserverUtils::eHasRefsSomeInvalid) {
     return aPreEffectsOverflowRect;
   }
@@ -393,54 +364,17 @@ nsRect SVGIntegrationUtils::ComputePostEffectsInkOverflowRect(
       AppUnitsPerCSSPixel());
   overrideBBox.RoundOut();
 
-  nsRect overflowRect =
-      FilterInstance::GetPostFilterBounds(firstFrame, &overrideBBox);
+  Maybe<nsRect> overflowRect = FilterInstance::GetPostFilterBounds(
+      firstFrame, filterFrames, &overrideBBox);
+  if (!overflowRect) {
+    return aPreEffectsOverflowRect;
+  }
 
   // Return overflowRect relative to aFrame, rather than "user space":
-  return overflowRect -
+  return overflowRect.value() -
          (aFrame->GetOffsetTo(firstFrame) + firstFrameToBoundingBox);
 }
 
-nsIntRegion SVGIntegrationUtils::AdjustInvalidAreaForSVGEffects(
-    nsIFrame* aFrame, const nsPoint& aToReferenceFrame,
-    const nsIntRegion& aInvalidRegion) {
-  if (aInvalidRegion.IsEmpty()) {
-    return nsIntRect();
-  }
-
-  nsIFrame* firstFrame =
-      nsLayoutUtils::FirstContinuationOrIBSplitSibling(aFrame);
-
-  // If we have any filters to observe then we should have started doing that
-  // during reflow/ComputeFrameEffectsRect, so we use GetFiltersIfObserving
-  // here to avoid needless work (or masking bugs by setting up observers at
-  // the wrong time).
-  if (!aFrame->StyleEffects()->HasFilters() ||
-      SVGObserverUtils::GetFiltersIfObserving(firstFrame, nullptr) ==
-          SVGObserverUtils::eHasRefsSomeInvalid) {
-    return aInvalidRegion;
-  }
-
-  int32_t appUnitsPerDevPixel = aFrame->PresContext()->AppUnitsPerDevPixel();
-
-  // Convert aInvalidRegion into bounding box frame space in app units:
-  nsPoint toBoundingBox =
-      aFrame->GetOffsetTo(firstFrame) + GetOffsetToBoundingBox(firstFrame);
-  // The initial rect was relative to the reference frame, so we need to
-  // remove that offset to get a rect relative to the current frame.
-  toBoundingBox -= aToReferenceFrame;
-  nsRegion preEffectsRegion =
-      aInvalidRegion.ToAppUnits(appUnitsPerDevPixel).MovedBy(toBoundingBox);
-
-  // Adjust the dirty area for effects, and shift it back to being relative to
-  // the reference frame.
-  nsRegion result =
-      FilterInstance::GetPostFilterDirtyArea(firstFrame, preEffectsRegion)
-          .MovedBy(-toBoundingBox);
-  // Return the result, in pixels relative to the reference frame.
-  return result.ToOutsidePixels(appUnitsPerDevPixel);
-}
-
 nsRect SVGIntegrationUtils::GetRequiredSourceForInvalidArea(
     nsIFrame* aFrame, const nsRect& aDirtyRect) {
   nsIFrame* firstFrame =
@@ -450,8 +384,9 @@ nsRect SVGIntegrationUtils::GetRequiredSourceForInvalidArea(
   // during reflow/ComputeFrameEffectsRect, so we use GetFiltersIfObserving
   // here to avoid needless work (or masking bugs by setting up observers at
   // the wrong time).
+  nsTArray<SVGFilterFrame*> filterFrames;
   if (!aFrame->StyleEffects()->HasFilters() ||
-      SVGObserverUtils::GetFiltersIfObserving(firstFrame, nullptr) ==
+      SVGObserverUtils::GetFiltersIfObserving(firstFrame, &filterFrames) ==
           SVGObserverUtils::eHasRefsSomeInvalid) {
     return aDirtyRect;
   }
@@ -462,7 +397,8 @@ nsRect SVGIntegrationUtils::GetRequiredSourceForInvalidArea(
   nsRect postEffectsRect = aDirtyRect + toUserSpace;
 
   // Return ther result, relative to aFrame, not in user space:
-  return FilterInstance::GetPreFilterNeededArea(firstFrame, postEffectsRect)
+  return FilterInstance::GetPreFilterNeededArea(firstFrame, filterFrames,
+                                                postEffectsRect)
              .GetBounds() -
          toUserSpace;
 }
@@ -493,11 +429,10 @@ using PaintFramesParams = SVGIntegrationUtils::PaintFramesParams;
  */
 static bool PaintMaskSurface(const PaintFramesParams& aParams,
                              DrawTarget* aMaskDT, float aOpacity,
-                             ComputedStyle* aSC,
+                             const ComputedStyle* aSC,
                              const nsTArray<SVGMaskFrame*>& aMaskFrames,
-                             const Matrix& aMaskSurfaceMatrix,
                              const nsPoint& aOffsetToUserSpace) {
-  MOZ_ASSERT(aMaskFrames.Length() > 0);
+  MOZ_ASSERT(!aMaskFrames.IsEmpty());
   MOZ_ASSERT(aMaskDT->GetFormat() == SurfaceFormat::A8);
   MOZ_ASSERT(aOpacity == 1.0 || aMaskFrames.Length() == 1);
 
@@ -508,9 +443,7 @@ static bool PaintMaskSurface(const PaintFramesParams& aParams,
   gfxPoint devPixelOffsetToUserSpace = nsLayoutUtils::PointToGfxPoint(
       aOffsetToUserSpace, presContext->AppUnitsPerDevPixel());
 
-  RefPtr<gfxContext> maskContext =
-      gfxContext::CreatePreservingTransformOrNull(aMaskDT);
-  MOZ_ASSERT(maskContext);
+  gfxContext maskContext(aMaskDT, /* aPreserveTransform */ true);
 
   bool isMaskComplete = true;
 
@@ -528,8 +461,8 @@ static bool PaintMaskSurface(const PaintFramesParams& aParams,
     // maskFrame == nullptr means we get an image mask.
     if (maskFrame) {
       SVGMaskFrame::MaskParams params(
-          maskContext, aParams.frame, cssPxToDevPxMatrix, aOpacity,
-          svgReset->mMask.mLayers[i].mMaskMode, aParams.imgParams);
+          maskContext.GetDrawTarget(), aParams.frame, cssPxToDevPxMatrix,
+          aOpacity, svgReset->mMask.mLayers[i].mMaskMode, aParams.imgParams);
       RefPtr<SourceSurface> svgMask = maskFrame->GetMaskForMaskedFrame(params);
       if (svgMask) {
         Matrix tmp = aMaskDT->GetTransform();
@@ -540,9 +473,9 @@ static bool PaintMaskSurface(const PaintFramesParams& aParams,
         aMaskDT->SetTransform(tmp);
       }
     } else if (svgReset->mMask.mLayers[i].mImage.IsResolved()) {
-      gfxContextMatrixAutoSaveRestore matRestore(maskContext);
+      gfxContextMatrixAutoSaveRestore matRestore(&maskContext);
 
-      maskContext->Multiply(gfxMatrix::Translation(-devPixelOffsetToUserSpace));
+      maskContext.Multiply(gfxMatrix::Translation(-devPixelOffsetToUserSpace));
       nsCSSRendering::PaintBGParams params =
           nsCSSRendering::PaintBGParams::ForSingleLayer(
               *presContext, aParams.dirtyRect, aParams.borderArea,
@@ -552,7 +485,7 @@ static bool PaintMaskSurface(const PaintFramesParams& aParams,
               i, compositionOp, aOpacity);
 
       aParams.imgParams.result &= nsCSSRendering::PaintStyleImageLayerWithSC(
-          params, *maskContext, aSC, *aParams.frame->StyleBorder());
+          params, maskContext, aSC, *aParams.frame->StyleBorder());
     } else {
       isMaskComplete = false;
     }
@@ -571,22 +504,22 @@ struct MaskPaintResult {
 };
 
 static MaskPaintResult CreateAndPaintMaskSurface(
-    const PaintFramesParams& aParams, float aOpacity, ComputedStyle* aSC,
+    const PaintFramesParams& aParams, float aOpacity, const ComputedStyle* aSC,
     const nsTArray<SVGMaskFrame*>& aMaskFrames,
     const nsPoint& aOffsetToUserSpace) {
   const nsStyleSVGReset* svgReset = aSC->StyleSVGReset();
-  MOZ_ASSERT(aMaskFrames.Length() > 0);
+  MOZ_ASSERT(!aMaskFrames.IsEmpty());
   MaskPaintResult paintResult;
 
   gfxContext& ctx = aParams.ctx;
 
   // Optimization for single SVG mask.
-  if (((aMaskFrames.Length() == 1) && aMaskFrames[0])) {
+  if (aMaskFrames.Length() == 1 && aMaskFrames[0]) {
     gfxMatrix cssPxToDevPxMatrix =
         SVGUtils::GetCSSPxToDevPxMatrix(aParams.frame);
     paintResult.opacityApplied = true;
     SVGMaskFrame::MaskParams params(
-        &ctx, aParams.frame, cssPxToDevPxMatrix, aOpacity,
+        ctx.GetDrawTarget(), aParams.frame, cssPxToDevPxMatrix, aOpacity,
         svgReset->mMask.mLayers[0].mMaskMode, aParams.imgParams);
     paintResult.maskSurface = aMaskFrames[0]->GetMaskForMaskedFrame(params);
     paintResult.maskTransform = ctx.CurrentMatrix();
@@ -598,7 +531,8 @@ static MaskPaintResult CreateAndPaintMaskSurface(
     return paintResult;
   }
 
-  const Rect& maskSurfaceRect = aParams.maskRect.valueOr(Rect());
+  const LayoutDeviceRect& maskSurfaceRect =
+      aParams.maskRect.valueOr(LayoutDeviceRect());
   if (aParams.maskRect.isSome() && maskSurfaceRect.IsEmpty()) {
     // XXX: Is this ever true?
     paintResult.transparentBlackMask = true;
@@ -606,7 +540,7 @@ static MaskPaintResult CreateAndPaintMaskSurface(
   }
 
   RefPtr<DrawTarget> maskDT = ctx.GetDrawTarget()->CreateClippedDrawTarget(
-      maskSurfaceRect, SurfaceFormat::A8);
+      maskSurfaceRect.ToUnknownRect(), SurfaceFormat::A8);
   if (!maskDT || !maskDT->IsValid()) {
     return paintResult;
   }
@@ -623,7 +557,7 @@ static MaskPaintResult CreateAndPaintMaskSurface(
 
   bool isMaskComplete = PaintMaskSurface(
       aParams, maskDT, paintResult.opacityApplied ? aOpacity : 1.0, aSC,
-      aMaskFrames, maskSurfaceMatrix, aOffsetToUserSpace);
+      aMaskFrames, aOffsetToUserSpace);
 
   if (!isMaskComplete ||
       (aParams.imgParams.result != ImgDrawResult::SUCCESS &&
@@ -661,15 +595,11 @@ static MaskPaintResult CreateAndPaintMaskSurface(
 }
 
 static bool ValidateSVGFrame(nsIFrame* aFrame) {
-#ifdef DEBUG
-  NS_ASSERTION(!aFrame->HasAnyStateBits(NS_FRAME_SVG_LAYOUT) ||
-                   (NS_SVGDisplayListPaintingEnabled() &&
-                    !aFrame->HasAnyStateBits(NS_FRAME_IS_NONDISPLAY)),
-               "Should not use SVGIntegrationUtils on this SVG frame");
-#endif
+  NS_ASSERTION(
+      !aFrame->HasAllStateBits(NS_FRAME_SVG_LAYOUT | NS_FRAME_IS_NONDISPLAY),
+      "Should not use SVGIntegrationUtils on this SVG frame");
 
-  bool hasSVGLayout = aFrame->HasAnyStateBits(NS_FRAME_SVG_LAYOUT);
-  if (hasSVGLayout) {
+  if (aFrame->HasAnyStateBits(NS_FRAME_SVG_LAYOUT)) {
 #ifdef DEBUG
     ISVGDisplayableFrame* svgFrame = do_QueryFrame(aFrame);
     MOZ_ASSERT(svgFrame && aFrame->GetContent()->IsSVGElement(),
@@ -686,54 +616,13 @@ static bool ValidateSVGFrame(nsIFrame* aFrame) {
   return true;
 }
 
-bool SVGIntegrationUtils::IsMaskResourceReady(nsIFrame* aFrame) {
-  nsIFrame* firstFrame =
-      nsLayoutUtils::FirstContinuationOrIBSplitSibling(aFrame);
-  nsTArray<SVGMaskFrame*> maskFrames;
-  // XXX check return value?
-  SVGObserverUtils::GetAndObserveMasks(firstFrame, &maskFrames);
-
-  const nsStyleSVGReset* svgReset = firstFrame->StyleSVGReset();
-
-  for (uint32_t i = 0; i < maskFrames.Length(); i++) {
-    // Refers to a valid SVG mask.
-    if (maskFrames[i]) {
-      continue;
-    }
-
-    // Refers to an external resource, which is not ready yet.
-    if (!svgReset->mMask.mLayers[i].mImage.IsComplete()) {
-      return false;
-    }
-  }
-
-  // Either all mask resources are ready, or no mask resource is needed.
-  return true;
-}
-
-class AutoPopGroup {
- public:
-  AutoPopGroup() : mContext(nullptr) {}
-
-  ~AutoPopGroup() {
-    if (mContext) {
-      mContext->PopGroupAndBlend();
-    }
-  }
-
-  void SetContext(gfxContext* aContext) { mContext = aContext; }
-
- private:
-  gfxContext* mContext;
-};
-
 bool SVGIntegrationUtils::PaintMask(const PaintFramesParams& aParams,
                                     bool& aOutIsMaskComplete) {
   aOutIsMaskComplete = true;
 
-  SVGUtils::MaskUsage maskUsage;
-  SVGUtils::DetermineMaskUsage(aParams.frame, aParams.handleOpacity, maskUsage);
-  if (!maskUsage.shouldDoSomething()) {
+  SVGUtils::MaskUsage maskUsage =
+      SVGUtils::DetermineMaskUsage(aParams.frame, aParams.handleOpacity);
+  if (!maskUsage.ShouldDoSomething()) {
     return false;
   }
 
@@ -745,21 +634,14 @@ bool SVGIntegrationUtils::PaintMask(const PaintFramesParams& aParams,
   gfxContext& ctx = aParams.ctx;
   RefPtr<DrawTarget> maskTarget = ctx.GetDrawTarget();
 
-  if (maskUsage.shouldGenerateMaskLayer &&
-      (maskUsage.shouldGenerateClipMaskLayer ||
-       maskUsage.shouldApplyClipPath)) {
+  if (maskUsage.ShouldGenerateMaskLayer() && maskUsage.HasSVGClip()) {
     // We will paint both mask of positioned mask and clip-path into
     // maskTarget.
     //
     // Create one extra draw target for drawing positioned mask, so that we do
     // not have to copy the content of maskTarget before painting
     // clip-path into it.
-    if (!maskTarget->CanCreateSimilarDrawTarget(maskTarget->GetSize(),
-                                                SurfaceFormat::A8)) {
-      return false;
-    }
-    maskTarget = maskTarget->CreateSimilarDrawTarget(maskTarget->GetSize(),
-                                                     SurfaceFormat::A8);
+    maskTarget = maskTarget->CreateClippedDrawTarget(Rect(), SurfaceFormat::A8);
   }
 
   nsIFrame* firstFrame =
@@ -768,50 +650,55 @@ bool SVGIntegrationUtils::PaintMask(const PaintFramesParams& aParams,
   // XXX check return value?
   SVGObserverUtils::GetAndObserveMasks(firstFrame, &maskFrames);
 
-  AutoPopGroup autoPop;
-  bool shouldPushOpacity =
-      (maskUsage.opacity != 1.0) && (maskFrames.Length() != 1);
+  gfxGroupForBlendAutoSaveRestore autoPop(&ctx);
+  bool shouldPushOpacity = !maskUsage.IsOpaque() && maskFrames.Length() != 1;
   if (shouldPushOpacity) {
-    ctx.PushGroupForBlendBack(gfxContentType::COLOR_ALPHA, maskUsage.opacity);
-    autoPop.SetContext(&ctx);
+    autoPop.PushGroupForBlendBack(gfxContentType::COLOR_ALPHA,
+                                  maskUsage.Opacity());
   }
 
   gfxContextMatrixAutoSaveRestore matSR;
 
   // Paint clip-path-basic-shape onto ctx
   gfxContextAutoSaveRestore basicShapeSR;
-  if (maskUsage.shouldApplyBasicShapeOrPath) {
+  if (maskUsage.ShouldApplyBasicShapeOrPath()) {
     matSR.SetContext(&ctx);
 
     MoveContextOriginToUserSpace(firstFrame, aParams);
 
     basicShapeSR.SetContext(&ctx);
-    CSSClipPathInstance::ApplyBasicShapeOrPathClip(
-        ctx, frame, SVGUtils::GetCSSPxToDevPxMatrix(frame));
-    if (!maskUsage.shouldGenerateMaskLayer) {
+    gfxMatrix mat = SVGUtils::GetCSSPxToDevPxMatrix(frame);
+    if (!maskUsage.ShouldGenerateMaskLayer()) {
       // Only have basic-shape clip-path effect. Fill clipped region by
       // opaque white.
       ctx.SetDeviceColor(DeviceColor::MaskOpaqueWhite());
-      ctx.Fill();
+      RefPtr<Path> path = CSSClipPathInstance::CreateClipPathForFrame(
+          ctx.GetDrawTarget(), frame, mat);
+      if (path) {
+        ctx.SetPath(path);
+        ctx.Fill();
+      }
 
       return true;
     }
+    CSSClipPathInstance::ApplyBasicShapeOrPathClip(ctx, frame, mat);
   }
 
-  // Paint mask onto ctx.
-  if (maskUsage.shouldGenerateMaskLayer) {
+  // Paint mask into maskTarget.
+  if (maskUsage.ShouldGenerateMaskLayer()) {
     matSR.Restore();
     matSR.SetContext(&ctx);
 
-    EffectOffsets offsets = MoveContextOriginToUserSpace(frame, aParams);
+    EffectOffsets offsets = ComputeEffectOffset(frame, aParams);
+    maskTarget->SetTransform(maskTarget->GetTransform().PreTranslate(
+        ToPoint(offsets.offsetToUserSpaceInDevPx)));
     aOutIsMaskComplete = PaintMaskSurface(
-        aParams, maskTarget, shouldPushOpacity ? 1.0 : maskUsage.opacity,
-        firstFrame->Style(), maskFrames, ctx.CurrentMatrix(),
-        offsets.offsetToUserSpace);
+        aParams, maskTarget, shouldPushOpacity ? 1.0f : maskUsage.Opacity(),
+        firstFrame->Style(), maskFrames, offsets.offsetToUserSpace);
   }
 
   // Paint clip-path onto ctx.
-  if (maskUsage.shouldGenerateClipMaskLayer || maskUsage.shouldApplyClipPath) {
+  if (maskUsage.HasSVGClip()) {
     matSR.Restore();
     matSR.SetContext(&ctx);
 
@@ -823,9 +710,8 @@ bool SVGIntegrationUtils::PaintMask(const PaintFramesParams& aParams,
     // XXX check return value?
     SVGObserverUtils::GetAndObserveClipPath(firstFrame, &clipPathFrame);
     RefPtr<SourceSurface> maskSurface =
-        maskUsage.shouldGenerateMaskLayer ? maskTarget->Snapshot() : nullptr;
-    clipPathFrame->PaintClipMask(ctx, frame, cssPxToDevPxMatrix, maskSurface,
-                                 ctx.CurrentMatrix());
+        maskUsage.ShouldGenerateMaskLayer() ? maskTarget->Snapshot() : nullptr;
+    clipPathFrame->PaintClipMask(ctx, frame, cssPxToDevPxMatrix, maskSurface);
   }
 
   return true;
@@ -834,9 +720,12 @@ bool SVGIntegrationUtils::PaintMask(const PaintFramesParams& aParams,
 template <class T>
 void PaintMaskAndClipPathInternal(const PaintFramesParams& aParams,
                                   const T& aPaintChild) {
-  MOZ_ASSERT(SVGIntegrationUtils::UsingMaskOrClipPathForFrame(aParams.frame),
+#ifdef DEBUG
+  const nsStyleSVGReset* style = aParams.frame->StyleSVGReset();
+  MOZ_ASSERT(style->HasClipPath() || style->HasMask(),
              "Should not use this method when no mask or clipPath effect"
              "on this frame");
+#endif
 
   /* SVG defines the following rendering model:
    *
@@ -856,10 +745,10 @@ void PaintMaskAndClipPathInternal(const PaintFramesParams& aParams,
     return;
   }
 
-  SVGUtils::MaskUsage maskUsage;
-  SVGUtils::DetermineMaskUsage(aParams.frame, aParams.handleOpacity, maskUsage);
+  SVGUtils::MaskUsage maskUsage =
+      SVGUtils::DetermineMaskUsage(aParams.frame, aParams.handleOpacity);
 
-  if (maskUsage.opacity == 0.0f) {
+  if (maskUsage.IsTransparent()) {
     return;
   }
 
@@ -879,21 +768,19 @@ void PaintMaskAndClipPathInternal(const PaintFramesParams& aParams,
 
   gfxMatrix cssPxToDevPxMatrix = SVGUtils::GetCSSPxToDevPxMatrix(frame);
 
-  bool shouldGenerateMask =
-      (maskUsage.opacity != 1.0f || maskUsage.shouldGenerateClipMaskLayer ||
-       maskUsage.shouldGenerateMaskLayer);
   bool shouldPushMask = false;
 
+  gfxGroupForBlendAutoSaveRestore autoGroupForBlend(&context);
+
   /* Check if we need to do additional operations on this child's
    * rendering, which necessitates rendering into another surface. */
-  if (shouldGenerateMask) {
+  if (maskUsage.ShouldGenerateMask()) {
     gfxContextMatrixAutoSaveRestore matSR;
 
-    Matrix maskTransform;
     RefPtr<SourceSurface> maskSurface;
     bool opacityApplied = false;
 
-    if (maskUsage.shouldGenerateMaskLayer) {
+    if (maskUsage.ShouldGenerateMaskLayer()) {
       matSR.SetContext(&context);
 
       // For css-mask, we want to generate a mask for each continuation frame,
@@ -901,7 +788,7 @@ void PaintMaskAndClipPathInternal(const PaintFramesParams& aParams,
       // instead of the first continuation frame.
       EffectOffsets offsets = MoveContextOriginToUserSpace(frame, aParams);
       MaskPaintResult paintResult = CreateAndPaintMaskSurface(
-          aParams, maskUsage.opacity, firstFrame->Style(), maskFrames,
+          aParams, maskUsage.Opacity(), firstFrame->Style(), maskFrames,
           offsets.offsetToUserSpace);
 
       if (paintResult.transparentBlackMask) {
@@ -911,29 +798,21 @@ void PaintMaskAndClipPathInternal(const PaintFramesParams& aParams,
       maskSurface = paintResult.maskSurface;
       if (maskSurface) {
         shouldPushMask = true;
-        // We want the mask to be untransformed so use the inverse of the
-        // current transform as the maskTransform to compensate.
-        maskTransform = context.CurrentMatrix();
-        maskTransform.Invert();
 
         opacityApplied = paintResult.opacityApplied;
       }
     }
 
-    if (maskUsage.shouldGenerateClipMaskLayer) {
+    if (maskUsage.ShouldGenerateClipMaskLayer()) {
       matSR.Restore();
       matSR.SetContext(&context);
 
       MoveContextOriginToUserSpace(firstFrame, aParams);
       RefPtr<SourceSurface> clipMaskSurface = clipPathFrame->GetClipMask(
-          context, frame, cssPxToDevPxMatrix, maskSurface, maskTransform);
+          context, frame, cssPxToDevPxMatrix, maskSurface);
 
       if (clipMaskSurface) {
         maskSurface = clipMaskSurface;
-        // We want the mask to be untransformed so use the inverse of the
-        // current transform as the maskTransform to compensate.
-        maskTransform = context.CurrentMatrix();
-        maskTransform.Invert();
       } else {
         // Either entire surface is clipped out, or gfx buffer allocation
         // failure in SVGClipPathFrame::GetClipMask.
@@ -944,9 +823,8 @@ void PaintMaskAndClipPathInternal(const PaintFramesParams& aParams,
     }
 
     // opacity != 1.0f.
-    if (!maskUsage.shouldGenerateClipMaskLayer &&
-        !maskUsage.shouldGenerateMaskLayer) {
-      MOZ_ASSERT(maskUsage.opacity != 1.0f);
+    if (!maskUsage.ShouldGenerateLayer()) {
+      MOZ_ASSERT(!maskUsage.IsOpaque());
 
       matSR.SetContext(&context);
       MoveContextOriginToUserSpace(firstFrame, aParams);
@@ -954,32 +832,30 @@ void PaintMaskAndClipPathInternal(const PaintFramesParams& aParams,
     }
 
     if (shouldPushMask) {
-      if (aParams.layerManager &&
-          aParams.layerManager->GetRoot()->GetContentFlags() &
-              Layer::CONTENT_COMPONENT_ALPHA) {
-        context.PushGroupAndCopyBackground(
-            gfxContentType::COLOR_ALPHA,
-            opacityApplied ? 1.0 : maskUsage.opacity, maskSurface,
-            maskTransform);
-      } else {
-        context.PushGroupForBlendBack(gfxContentType::COLOR_ALPHA,
-                                      opacityApplied ? 1.0 : maskUsage.opacity,
-                                      maskSurface, maskTransform);
-      }
+      // We want the mask to be untransformed so use the inverse of the
+      // current transform as the maskTransform to compensate.
+      Matrix maskTransform = context.CurrentMatrix();
+      maskTransform.Invert();
+
+      autoGroupForBlend.PushGroupForBlendBack(
+          gfxContentType::COLOR_ALPHA,
+          opacityApplied ? 1.0f : maskUsage.Opacity(), maskSurface,
+          maskTransform);
     }
   }
 
   /* If this frame has only a trivial clipPath, set up cairo's clipping now so
    * we can just do normal painting and get it clipped appropriately.
    */
-  if (maskUsage.shouldApplyClipPath || maskUsage.shouldApplyBasicShapeOrPath) {
+  if (maskUsage.ShouldApplyClipPath() ||
+      maskUsage.ShouldApplyBasicShapeOrPath()) {
     gfxContextMatrixAutoSaveRestore matSR(&context);
 
     MoveContextOriginToUserSpace(firstFrame, aParams);
 
-    MOZ_ASSERT(!maskUsage.shouldApplyClipPath ||
-               !maskUsage.shouldApplyBasicShapeOrPath);
-    if (maskUsage.shouldApplyClipPath) {
+    MOZ_ASSERT(!maskUsage.ShouldApplyClipPath() ||
+               !maskUsage.ShouldApplyBasicShapeOrPath());
+    if (maskUsage.ShouldApplyClipPath()) {
       clipPathFrame->ApplyClipPath(context, frame, cssPxToDevPxMatrix);
     } else {
       CSSClipPathInstance::ApplyBasicShapeOrPathClip(context, frame,
@@ -999,14 +875,13 @@ void PaintMaskAndClipPathInternal(const PaintFramesParams& aParams,
         aParams.borderArea, frame->PresContext()->AppUnitsPerDevPixel());
     context.SnappedRectangle(drawingRect);
     sRGBColor overlayColor(0.0f, 0.0f, 0.0f, 0.8f);
-    if (maskUsage.shouldGenerateMaskLayer) {
+    if (maskUsage.ShouldGenerateMaskLayer()) {
       overlayColor.r = 1.0f;  // red represents css positioned mask.
     }
-    if (maskUsage.shouldApplyClipPath ||
-        maskUsage.shouldGenerateClipMaskLayer) {
+    if (maskUsage.HasSVGClip()) {
       overlayColor.g = 1.0f;  // green represents clip-path:<clip-source>.
     }
-    if (maskUsage.shouldApplyBasicShapeOrPath) {
+    if (maskUsage.ShouldApplyBasicShapeOrPath()) {
       overlayColor.b = 1.0f;  // blue represents
                               // clip-path:<basic-shape>||<geometry-box>.
     }
@@ -1015,26 +890,10 @@ void PaintMaskAndClipPathInternal(const PaintFramesParams& aParams,
     context.Fill();
   }
 
-  if (maskUsage.shouldApplyClipPath || maskUsage.shouldApplyBasicShapeOrPath) {
+  if (maskUsage.ShouldApplyClipPath() ||
+      maskUsage.ShouldApplyBasicShapeOrPath()) {
     context.PopClip();
   }
-
-  if (shouldPushMask) {
-    context.PopGroupAndBlend();
-  }
-}
-
-void SVGIntegrationUtils::PaintMaskAndClipPath(
-    const PaintFramesParams& aParams) {
-  PaintMaskAndClipPathInternal(aParams, [&] {
-    gfxContext& context = aParams.ctx;
-    BasicLayerManager* basic = aParams.layerManager->AsBasicLayerManager();
-    RefPtr<gfxContext> oldCtx = basic->GetTarget();
-    basic->SetTarget(&context);
-    aParams.layerManager->EndTransaction(FrameLayerBuilder::DrawPaintedLayer,
-                                         aParams.builder);
-    basic->SetTarget(oldCtx);
-  });
 }
 
 void SVGIntegrationUtils::PaintMaskAndClipPath(
@@ -1044,10 +903,11 @@ void SVGIntegrationUtils::PaintMaskAndClipPath(
 }
 
 void SVGIntegrationUtils::PaintFilter(const PaintFramesParams& aParams,
+                                      Span<const StyleFilter> aFilters,
                                       const SVGFilterPaintCallback& aCallback) {
   MOZ_ASSERT(!aParams.builder->IsForGenerateGlyphMask(),
              "Filter effect is discarded while generating glyph mask.");
-  MOZ_ASSERT(aParams.frame->StyleEffects()->HasFilters(),
+  MOZ_ASSERT(!aFilters.IsEmpty(),
              "Should not use this method when no filter effect on this frame");
 
   nsIFrame* frame = aParams.frame;
@@ -1060,20 +920,19 @@ void SVGIntegrationUtils::PaintFilter(const PaintFramesParams& aParams,
     return;
   }
 
-  /* Properties are added lazily and may have been removed by a restyle,
-     so make sure all applicable ones are set again. */
+  // Properties are added lazily and may have been removed by a restyle, so make
+  // sure all applicable ones are set again.
   nsIFrame* firstFrame =
       nsLayoutUtils::FirstContinuationOrIBSplitSibling(frame);
   // Note: we do not return here for eHasNoRefs since we must still handle any
   // CSS filter functions.
-  // TODO: We currently pass nullptr instead of an nsTArray* here, but we
-  // actually should get the filter frames and then pass them into
-  // PaintFilteredFrame below!  See bug 1494263.
   // XXX: Do we need to check for eHasRefsSomeInvalid here given that
   // nsDisplayFilter::BuildLayer returns nullptr for eHasRefsSomeInvalid?
   // Or can we just assert !eHasRefsSomeInvalid?
-  if (SVGObserverUtils::GetAndObserveFilters(firstFrame, nullptr) ==
+  nsTArray<SVGFilterFrame*> filterFrames;
+  if (SVGObserverUtils::GetAndObserveFilters(firstFrame, &filterFrames) ==
       SVGObserverUtils::eHasRefsSomeInvalid) {
+    aCallback(aParams.ctx, aParams.imgParams, nullptr, nullptr);
     return;
   }
 
@@ -1085,13 +944,22 @@ void SVGIntegrationUtils::PaintFilter(const PaintFramesParams& aParams,
   /* Paint the child and apply filters */
   nsRegion dirtyRegion = aParams.dirtyRect - offsets.offsetToBoundingBox;
 
-  FilterInstance::PaintFilteredFrame(frame, &context, aCallback, &dirtyRegion,
-                                     aParams.imgParams, opacity);
+  FilterInstance::PaintFilteredFrame(frame, aFilters, filterFrames, &context,
+                                     aCallback, &dirtyRegion, aParams.imgParams,
+                                     opacity);
 }
 
-bool SVGIntegrationUtils::CreateWebRenderCSSFilters(
+WrFiltersStatus SVGIntegrationUtils::CreateWebRenderCSSFilters(
     Span<const StyleFilter> aFilters, nsIFrame* aFrame,
     WrFiltersHolder& aWrFilters) {
+  // Check if prefs are set to convert the CSS filters to SVG filters and use
+  // the new WebRender SVG filter rendering, rather than the existing CSS filter
+  // support
+  if (StaticPrefs::gfx_webrender_svg_filter_effects() &&
+      StaticPrefs::
+          gfx_webrender_svg_filter_effects_also_convert_css_filters()) {
+    return WrFiltersStatus::BLOB_FALLBACK;
+  }
   // All CSS filters are supported by WebRender. SVG filters are not fully
   // supported, those use NS_STYLE_FILTER_URL and are handled separately.
 
@@ -1099,8 +967,10 @@ bool SVGIntegrationUtils::CreateWebRenderCSSFilters(
   // succeeded, and don't render any of them.
   if (aFilters.Length() >
       StaticPrefs::gfx_webrender_max_filter_ops_per_chain()) {
-    return true;
+    return WrFiltersStatus::DISABLED_FOR_PERFORMANCE;
   }
+  // Track status so we can do cleanup if unsupported filters are found.
+  WrFiltersStatus status = WrFiltersStatus::CHAIN;
   aWrFilters.filters.SetCapacity(aFilters.Length());
   auto& wrFilters = aWrFilters.filters;
   for (const StyleFilter& filter : aFilters) {
@@ -1164,38 +1034,46 @@ bool SVGIntegrationUtils::CreateWebRenderCSSFilters(
         break;
       }
       default:
-        return false;
+        status = WrFiltersStatus::BLOB_FALLBACK;
+        break;
+    }
+    if (status != WrFiltersStatus::CHAIN) {
+      break;
     }
   }
-
-  return true;
+  if (status != WrFiltersStatus::CHAIN) {
+    // Clean up the filters holder if we can't render filters this way.
+    aWrFilters = {};
+  }
+  return status;
 }
 
-bool SVGIntegrationUtils::BuildWebRenderFilters(
+WrFiltersStatus SVGIntegrationUtils::BuildWebRenderFilters(
     nsIFrame* aFilteredFrame, Span<const StyleFilter> aFilters,
-    WrFiltersHolder& aWrFilters, Maybe<nsRect>& aPostFilterClip) {
+    StyleFilterType aStyleFilterType, WrFiltersHolder& aWrFilters,
+    const nsPoint& aOffsetForSVGFilters) {
   return FilterInstance::BuildWebRenderFilters(aFilteredFrame, aFilters,
-                                               aWrFilters, aPostFilterClip);
+                                               aStyleFilterType, aWrFilters,
+                                               aOffsetForSVGFilters);
 }
 
 bool SVGIntegrationUtils::CanCreateWebRenderFiltersForFrame(nsIFrame* aFrame) {
   WrFiltersHolder wrFilters;
-  Maybe<nsRect> filterClip;
   auto filterChain = aFrame->StyleEffects()->mFilters.AsSpan();
-  return CreateWebRenderCSSFilters(filterChain, aFrame, wrFilters) ||
-         BuildWebRenderFilters(aFrame, filterChain, wrFilters, filterClip);
+  WrFiltersStatus status =
+      CreateWebRenderCSSFilters(filterChain, aFrame, wrFilters);
+  if (status == WrFiltersStatus::BLOB_FALLBACK) {
+    status = BuildWebRenderFilters(aFrame, filterChain, StyleFilterType::Filter,
+                                   wrFilters, nsPoint());
+  }
+  return status == WrFiltersStatus::CHAIN || status == WrFiltersStatus::SVGFE;
 }
 
 bool SVGIntegrationUtils::UsesSVGEffectsNotSupportedInCompositor(
     nsIFrame* aFrame) {
   // WebRender supports masks / clip-paths and some filters in the compositor.
-  // Non-WebRender doesn't support any SVG effects in the compositor.
   if (aFrame->StyleEffects()->HasFilters()) {
-    return !gfx::gfxVars::UseWebRender() ||
-           !SVGIntegrationUtils::CanCreateWebRenderFiltersForFrame(aFrame);
-  }
-  if (SVGIntegrationUtils::UsingMaskOrClipPathForFrame(aFrame)) {
-    return !gfx::gfxVars::UseWebRender();
+    return !SVGIntegrationUtils::CanCreateWebRenderFiltersForFrame(aFrame);
   }
   return false;
 }
@@ -1232,9 +1110,7 @@ bool PaintFrameCallback::operator()(gfxContext* aContext,
   aContext->Save();
 
   // Clip to aFillRect so that we don't paint outside.
-  aContext->NewPath();
-  aContext->Rectangle(aFillRect);
-  aContext->Clip();
+  aContext->Clip(aFillRect);
 
   gfxMatrix invmatrix = aTransform;
   if (!invmatrix.Invert()) {
@@ -1312,7 +1188,11 @@ already_AddRefed<gfxDrawable> SVGIntegrationUtils::DrawableFromPaintServer(
     gfxRect overrideBounds(0, 0, aPaintServerSize.width,
                            aPaintServerSize.height);
     overrideBounds.Scale(1.0 / aFrame->PresContext()->AppUnitsPerDevPixel());
-    imgDrawingParams imgParams(aFlags);
+    uint32_t imgFlags = imgIContainer::FLAG_ASYNC_NOTIFY;
+    if (aFlags & SVGIntegrationUtils::FLAG_SYNC_DECODE_IMAGES) {
+      imgFlags |= imgIContainer::FLAG_SYNC_DECODE;
+    }
+    imgDrawingParams imgParams(imgFlags);
     RefPtr<gfxPattern> pattern = server->GetPaintServerPattern(
         aTarget, aDrawTarget, aContextMatrix, &nsStyleSVG::mFill, 1.0,
         imgParams, &overrideBounds);
@@ -1330,11 +1210,10 @@ already_AddRefed<gfxDrawable> SVGIntegrationUtils::DrawableFromPaintServer(
     gfxFloat scaleY = overrideBounds.Height() / aRenderSize.height;
     gfxMatrix scaleMatrix = gfxMatrix::Scaling(scaleX, scaleY);
     pattern->SetMatrix(scaleMatrix * pattern->GetMatrix());
-    RefPtr<gfxDrawable> drawable = new gfxPatternDrawable(pattern, aRenderSize);
-    return drawable.forget();
+    return do_AddRef(new gfxPatternDrawable(pattern, aRenderSize));
   }
 
-  if (aFrame->IsFrameOfType(nsIFrame::eSVG) &&
+  if (aFrame->IsSVGFrame() &&
       !static_cast<ISVGDisplayableFrame*>(do_QueryFrame(aFrame))) {
     MOZ_ASSERT_UNREACHABLE(
         "We should prevent painting of unpaintable SVG "
@@ -1346,8 +1225,7 @@ already_AddRefed<gfxDrawable> SVGIntegrationUtils::DrawableFromPaintServer(
   // set up a drawing callback.
   RefPtr<gfxDrawingCallback> cb =
       new PaintFrameCallback(aFrame, aPaintServerSize, aRenderSize, aFlags);
-  RefPtr<gfxDrawable> drawable = new gfxCallbackDrawable(cb, aRenderSize);
-  return drawable.forget();
+  return do_AddRef(new gfxCallbackDrawable(cb, aRenderSize));
 }
 
 }  // namespace mozilla
