@@ -13,13 +13,13 @@ import tempfile
 import time
 import traceback
 from contextlib import closing
+from urllib.request import urlopen
 
 import mozcrash
 import reftestcommandline
 from mozdevice import ADBDeviceFactory, RemoteProcessMonitor
 from output import OutputHandler
 from runreftest import RefTest, ReftestResolver, build_obj
-from six.moves.urllib_request import urlopen
 
 # We need to know our current directory so that we can serve our test files from it.
 SCRIPT_DIRECTORY = os.path.abspath(os.path.realpath(os.path.dirname(__file__)))
@@ -435,7 +435,7 @@ class RemoteReftest(RefTest):
         startTime = datetime.datetime.now()
         status = 0
         profileDirectory = self.remoteProfile + "/"
-        cmdargs.extend(("-no-remote", "-profile", profileDirectory))
+        cmdargs.extend(("-profile", profileDirectory))
 
         pid = rpm.launch(
             binary,
