@@ -5,12 +5,7 @@
 """
 Runs the reftest test harness.
 """
-from __future__ import print_function
-
-from __future__ import absolute_import, print_function
-
 import json
-import multiprocessing
 import os
 import platform
 import posixpath
@@ -37,11 +32,10 @@ import mozlog
 import mozprocess
 import mozprofile
 import mozrunner
-from manifestparser import TestManifest, filters as mpf
+from manifestparser import TestManifest
+from manifestparser import filters as mpf
 from mozrunner.utils import get_stack_fixer_function, test_environment
-from mozscreenshot import printstatus, dump_screen
-from six import reraise, string_types
-from six.moves import range
+from mozscreenshot import dump_screen, printstatus
 
 try:
     from marionette_driver.addons import Addons
@@ -57,17 +51,19 @@ except ImportError as e:  # noqa
 
     Marionette = reraise_
 
-from output import OutputHandler, ReftestFormatter
 import reftestcommandline
+from output import OutputHandler, ReftestFormatter
 
 here = os.path.abspath(os.path.dirname(__file__))
 
 try:
     from mozbuild.base import MozbuildObject
+    from mozbuild.util import cpu_count
 
     build_obj = MozbuildObject.from_environment(cwd=here)
 except ImportError:
     build_obj = None
+    from multiprocessing import cpu_count
 
 
 def categoriesToRegex(categoryList):
@@ -100,18 +96,12 @@ summaryLines = [
 ]
 
 
-if sys.version_info[0] == 3:
-
-    def reraise_(tp_, value_, tb_=None):
-        if value_ is None:
-            value_ = tp_()
-        if value_.__traceback__ is not tb_:
-            raise value_.with_traceback(tb_)
-        raise value_
-
-
-else:
-    exec("def reraise_(tp_, value_, tb_=None):\n    raise tp_, value_, tb_\n")
+def reraise_(tp_, value_, tb_=None):
+    if value_ is None:
+        value_ = tp_()
+    if value_.__traceback__ is not tb_:
+        raise value_.with_traceback(tb_)
+    raise value_
 
 
 def update_mozinfo():
@@ -197,7 +187,7 @@ class ReftestThread(threading.Thread):
                 yield line
 
 
-class ReftestResolver(object):
+class ReftestResolver:
     def defaultManifest(self, suite):
         return {
             "reftest": "reftest.list",
@@ -256,7 +246,7 @@ class ReftestResolver(object):
                 rv = [
                     (
                         os.path.join(dirname, default_manifest),
-                        r".*(?:/|\\)%s(?:[#?].*)?$" % pathname.replace("?", "\?"),
+                        r".*%s(?:[#?].*)?$" % pathname.replace("?", r"\?"),
                     )
                 ]
 
@@ -291,7 +281,7 @@ class ReftestResolver(object):
         return manifests_by_url
 
 
-class RefTest(object):
+class RefTest:
     oldcwd = os.getcwd()
     resolver_cls = ReftestResolver
     use_marionette = True
@@ -305,6 +295,8 @@ class RefTest(object):
         self.log = None
         self.outputHandler = None
         self.testDumpFile = os.path.join(tempfile.gettempdir(), "reftests.json")
+        self.currentManifest = "No test started"
+        self.gtkTheme = self.getGtkTheme()
 
         self.run_by_manifest = True
         if suite in ("crashtest", "jstestbrowser"):
@@ -331,6 +323,22 @@ class RefTest(object):
             "reftest harness", options, {"tbpl": sys.stdout}, fmt_options
         )
 
+    def getGtkTheme(self):
+        if not platform.system() == "Linux":
+            return ""
+
+        try:
+            theme_cmd = "gsettings get org.gnome.desktop.interface gtk-theme"
+            theme = subprocess.check_output(
+                theme_cmd, shell=True, universal_newlines=True
+            )
+            if theme:
+                theme = theme.strip("\n")
+                theme = theme.strip("'")
+            return theme.strip()
+        except subprocess.CalledProcessError:
+            return ""
+
     def getFullPath(self, path):
         "Get an absolute path relative to self.oldcwd."
         return os.path.normpath(os.path.join(self.oldcwd, os.path.expanduser(path)))
@@ -360,14 +368,14 @@ class RefTest(object):
         locations.add_host(server, scheme="http", port=port)
         locations.add_host(server, scheme="https", port=port)
 
-        sandbox_whitelist_paths = options.sandboxReadWhitelist
+        sandbox_allowlist_paths = options.sandboxReadWhitelist
         if platform.system() == "Linux" or platform.system() in (
             "Windows",
             "Microsoft",
         ):
             # Trailing slashes are needed to indicate directories on Linux and Windows
-            sandbox_whitelist_paths = map(
-                lambda p: os.path.join(p, ""), sandbox_whitelist_paths
+            sandbox_allowlist_paths = map(
+                lambda p: os.path.join(p, ""), sandbox_allowlist_paths
             )
 
         addons = []
@@ -393,7 +401,7 @@ class RefTest(object):
         kwargs = {
             "addons": addons,
             "locations": locations,
-            "whitelistpaths": sandbox_whitelist_paths,
+            "allowlistpaths": sandbox_allowlist_paths,
         }
         if profile_to_clone:
             profile = mozprofile.Profile.clone(profile_to_clone, **kwargs)
@@ -420,7 +428,7 @@ class RefTest(object):
             if os.path.isdir(path):
                 profile_data_dir = path
 
-        with open(os.path.join(profile_data_dir, "profiles.json"), "r") as fh:
+        with open(os.path.join(profile_data_dir, "profiles.json")) as fh:
             base_profiles = json.load(fh)["reftest"]
 
         for name in base_profiles:
@@ -455,16 +463,13 @@ class RefTest(object):
         # Run the "deferred" font-loader immediately, because if it finishes
         # mid-test, the extra reflow that is triggered can disrupt the test.
         prefs["gfx.font_loader.delay"] = 0
-        prefs["gfx.font_loader.interval"] = 0
         # Ensure bundled fonts are activated, even if not enabled by default
         # on the platform, so that tests can rely on them.
         prefs["gfx.bundled-fonts.activate"] = 1
         # Disable dark scrollbars because it's semi-transparent.
         prefs["widget.disable-dark-scrollbar"] = True
-        prefs["reftest.isCoverageBuild"] = mozinfo.info.get("ccov", False)
 
-        # config specific flags
-        prefs["sandbox.apple_silicon"] = mozinfo.info.get("apple_silicon", False)
+        prefs["sandbox.mozinfo"] = json.dumps(mozinfo.info)
 
         # Set tests to run or manifests to parse.
         if tests:
@@ -475,10 +480,10 @@ class RefTest(object):
         elif manifests:
             prefs["reftest.manifests"] = json.dumps(manifests)
 
-        # Unconditionally update the e10s pref, default True
-        prefs["browser.tabs.remote.autostart"] = True
-        if not options.e10s:
-            prefs["browser.tabs.remote.autostart"] = False
+        # Avoid unncessary recursion when MOZHARNESS_TEST_PATHS is set
+        prefs["reftest.mozharness_test_paths"] = (
+            len(os.environ.get("MOZHARNESS_TEST_PATHS", "")) > 0
+        )
 
         # default fission to True
         prefs["fission.autostart"] = True
@@ -491,22 +496,6 @@ class RefTest(object):
             if options.thisChunk:
                 prefs["reftest.thisChunk"] = options.thisChunk
 
-        # Bug 1262954: For winXP + e10s disable acceleration
-        if (
-            platform.system() in ("Windows", "Microsoft")
-            and "5.1" in platform.version()
-            and options.e10s
-        ):
-            prefs["layers.acceleration.disabled"] = True
-
-        # Bug 1300355: Disable canvas cache for win7 as it uses
-        # too much memory and causes OOMs.
-        if (
-            platform.system() in ("Windows", "Microsoft")
-            and "6.1" in platform.version()
-        ):
-            prefs["reftest.nocache"] = True
-
         if options.marionette:
             # options.marionette can specify host:port
             port = options.marionette.split(":")[1]
@@ -514,7 +503,7 @@ class RefTest(object):
 
         # Enable tracing output for detailed failures in case of
         # failing connection attempts, and hangs (bug 1397201)
-        prefs["marionette.log.level"] = "Trace"
+        prefs["remote.log.level"] = "Trace"
 
         # Third, set preferences passed in via the command line.
         for v in options.extraPrefs:
@@ -533,9 +522,7 @@ class RefTest(object):
 
         self.copyExtraFilesToProfile(options, profile)
 
-        self.log.info(
-            "Running with e10s: {}".format(prefs["browser.tabs.remote.autostart"])
-        )
+        self.log.info(f"Running with e10s: {options.e10s}")
         self.log.info("Running with fission: {}".format(prefs["fission.autostart"]))
 
         return profile
@@ -549,10 +536,6 @@ class RefTest(object):
             xrePath=options.xrePath, debugger=options.debugger
         )
         browserEnv["XPCOM_DEBUG_BREAK"] = "stack"
-        if options.topsrcdir:
-            browserEnv["MOZ_DEVELOPER_REPO_DIR"] = options.topsrcdir
-        if hasattr(options, "topobjdir"):
-            browserEnv["MOZ_DEVELOPER_OBJ_DIR"] = options.topobjdir
 
         if mozinfo.info["asan"]:
             # Disable leak checking for reftests for now
@@ -567,6 +550,9 @@ class RefTest(object):
             browserEnv["TZ"] = "PST8PDT"
             browserEnv["LC_ALL"] = "en_US.UTF-8"
 
+        # This should help with consistency
+        browserEnv["GTK_THEME"] = "Adwaita"
+
         for v in options.environment:
             ix = v.find("=")
             if ix <= 0:
@@ -585,6 +571,9 @@ class RefTest(object):
         if options.headless:
             browserEnv["MOZ_HEADLESS"] = "1"
 
+        if not options.e10s:
+            browserEnv["MOZ_FORCE_DISABLE_E10S"] = "1"
+
         return browserEnv
 
     def cleanup(self, profileDir):
@@ -659,13 +648,13 @@ class RefTest(object):
         ]
 
         stepResults = {}
-        for (descr, step) in steps:
+        for descr, step in steps:
             stepResults[descr] = "not run / incomplete"
 
         startTime = datetime.now()
         maxTime = timedelta(seconds=options.verify_max_time)
         finalResult = "PASSED"
-        for (descr, step) in steps:
+        for descr, step in steps:
             if (datetime.now() - startTime) > maxTime:
                 self.log.info("::: Test verification is taking too long: Giving up!")
                 self.log.info(
@@ -713,7 +702,7 @@ class RefTest(object):
         if not getattr(options, "runTestsInParallel", False):
             return self.runSerialTests(manifests, options, cmdargs)
 
-        cpuCount = multiprocessing.cpu_count()
+        cpuCount = cpu_count()
 
         # We have the directive, technology, and machine to run multiple test instances.
         # Experimentation says that reftests are not overly CPU-intensive, so we can run
@@ -737,7 +726,7 @@ class RefTest(object):
         # First job is only needs-focus tests.  Remaining jobs are
         # non-needs-focus and chunked.
         perProcessArgs[0].insert(-1, "--focus-filter-mode=needs-focus")
-        for (chunkNumber, jobArgs) in enumerate(perProcessArgs[1:], start=1):
+        for chunkNumber, jobArgs in enumerate(perProcessArgs[1:], start=1):
             jobArgs[-1:-1] = [
                 "--focus-filter-mode=non-needs-focus",
                 "--total-chunks=%d" % jobsWithoutFocus,
@@ -777,16 +766,16 @@ class RefTest(object):
         # Output the summaries that the ReftestThread filters suppressed.
         summaryObjects = [defaultdict(int) for s in summaryLines]
         for t in threads:
-            for (summaryObj, (text, categories)) in zip(summaryObjects, summaryLines):
+            for summaryObj, (text, categories) in zip(summaryObjects, summaryLines):
                 threadMatches = t.summaryMatches[text]
-                for (attribute, description) in categories:
+                for attribute, description in categories:
                     amount = int(threadMatches.group(attribute) if threadMatches else 0)
                     summaryObj[attribute] += amount
                 amount = int(threadMatches.group("total") if threadMatches else 0)
                 summaryObj["total"] += amount
 
         print("REFTEST INFO | Result summary:")
-        for (summaryObj, (text, categories)) in zip(summaryObjects, summaryLines):
+        for summaryObj, (text, categories) in zip(summaryObjects, summaryLines):
             details = ", ".join(
                 [
                     "%d %s" % (summaryObj[attribute], description)
@@ -813,7 +802,7 @@ class RefTest(object):
             "%s | application timed out after %d seconds with no output"
             % (self.lastTestSeen, int(timeout))
         )
-        self.log.error("Force-terminating active process(es).")
+        self.log.warning("Force-terminating active process(es).")
         self.killAndGetStack(
             proc, utilityPath, debuggerInfo, dump_screen=not debuggerInfo
         )
@@ -868,9 +857,8 @@ class RefTest(object):
         valgrindPath=None,
         valgrindArgs=None,
         valgrindSuppFiles=None,
-        **profileArgs
+        **profileArgs,
     ):
-
         if cmdargs is None:
             cmdargs = []
         cmdargs = cmdargs[:]
@@ -905,7 +893,7 @@ class RefTest(object):
                 self.lastTestSeen = testid(message["test"])
             elif message["action"] == "test_end":
                 if self.lastTest and message["test"] == self.lastTest:
-                    self.lastTestSeen = "Last test finished"
+                    self.lastTestSeen = self.currentManifest
                 else:
                     self.lastTestSeen = "{} (finished)".format(testid(message["test"]))
 
@@ -944,7 +932,7 @@ class RefTest(object):
             debug_args=debug_args, interactive=interactive, outputTimeout=timeout
         )
         proc = runner.process_handler
-        self.outputHandler.proc_name = "GECKO({})".format(proc.pid)
+        self.outputHandler.proc_name = f"GECKO({proc.pid})"
 
         # Used to defer a possible IOError exception from Marionette
         marionette_exception = None
@@ -971,47 +959,64 @@ class RefTest(object):
                 addons.install(options.reftestExtensionPath, temp=True)
 
                 marionette.delete_session()
-            except IOError:
+            except OSError as e:
                 # Any IOError as thrown by Marionette means that something is
                 # wrong with the process, like a crash or the socket is no
                 # longer open. We defer raising this specific error so that
                 # post-test checks for leaks and crashes are performed and
                 # reported first.
-                marionette_exception = sys.exc_info()
+                marionette_exception = e
 
         status = runner.wait()
         runner.process_handler = None
         self.outputHandler.proc_name = None
 
-        if status:
-            msg = (
-                "TEST-UNEXPECTED-FAIL | %s | application terminated with exit code %s"
-                % (self.lastTestSeen, status)
-            )
-            # use process_output so message is logged verbatim
-            self.log.process_output(None, msg)
-
         crashed = mozcrash.log_crashes(
             self.log,
             os.path.join(profile.profile, "minidumps"),
             options.symbolsPath,
             test=self.lastTestSeen,
         )
+
+        if crashed:
+            # log suite_end to wrap up, this is usually done with in in-browser harness
+            if not self.outputHandler.results:
+                # TODO: while .results is a defaultdict(int), it is proxied via log_actions as data, not type
+                self.outputHandler.results = {
+                    "Pass": 0,
+                    "LoadOnly": 0,
+                    "Exception": 0,
+                    "FailedLoad": 0,
+                    "UnexpectedFail": 1,
+                    "UnexpectedPass": 0,
+                    "AssertionUnexpected": 0,
+                    "AssertionUnexpectedFixed": 0,
+                    "KnownFail": 0,
+                    "AssertionKnown": 0,
+                    "Random": 0,
+                    "Skip": 0,
+                    "Slow": 0,
+                }
+            self.log.suite_end(extra={"results": self.outputHandler.results})
+
         if not status and crashed:
             status = 1
 
+        if status and not crashed:
+            msg = "application terminated with exit code %s" % (status)
+            self.log.shutdown_failure(group=self.lastTestSeen, message=msg)
+
         runner.cleanup()
         self.cleanup(profile.profile)
 
         if marionette_exception is not None:
-            exc, value, tb = marionette_exception
-            raise reraise(exc, value, tb)
+            raise marionette_exception
 
         self.log.info("Process mode: {}".format("e10s" if options.e10s else "non-e10s"))
         return status
 
     def getActiveTests(self, manifests, options, testDumpFile=None):
-        # These prefs will cause reftest.jsm to parse the manifests,
+        # These prefs will cause reftest.sys.mjs to parse the manifests,
         # dump the resulting tests to a file, and exit.
         prefs = {
             "reftest.manifests": json.dumps(manifests),
@@ -1024,7 +1029,7 @@ class RefTest(object):
             print("Error: parsing manifests failed!")
             sys.exit(1)
 
-        with open(self.testDumpFile, "r") as fh:
+        with open(self.testDumpFile) as fh:
             tests = json.load(fh)
 
         if os.path.isfile(self.testDumpFile):
@@ -1056,7 +1061,7 @@ class RefTest(object):
         def run(**kwargs):
             if kwargs.get("tests"):
                 self.lastTest = kwargs["tests"][-1]["identifier"]
-                if not isinstance(self.lastTest, string_types):
+                if not isinstance(self.lastTest, str):
                     self.lastTest = " ".join(self.lastTest)
 
             status = self.runApp(
@@ -1076,16 +1081,18 @@ class RefTest(object):
                 timeout=options.timeout + 70.0,
                 debuggerInfo=debuggerInfo,
                 symbolsPath=options.symbolsPath,
-                **kwargs
+                **kwargs,
             )
 
-            mozleak.process_leak_log(
-                self.leakLogFile,
-                leak_thresholds=options.leakThresholds,
-                stack_fixer=get_stack_fixer_function(
-                    options.utilityPath, options.symbolsPath
-                ),
-            )
+            # do not process leak log when we crash/assert
+            if status == 0:
+                mozleak.process_leak_log(
+                    self.leakLogFile,
+                    leak_thresholds=options.leakThresholds,
+                    stack_fixer=get_stack_fixer_function(
+                        options.utilityPath, options.symbolsPath
+                    ),
+                )
             return status
 
         if not self.run_by_manifest:
@@ -1097,17 +1104,29 @@ class RefTest(object):
         for t in tests:
             tests_by_manifest[t["manifest"]].append(t)
             test_id = t["identifier"]
-            if not isinstance(test_id, string_types):
+            if not isinstance(test_id, str):
                 test_id = " ".join(test_id)
             ids_by_manifest[t["manifestID"]].append(test_id)
 
         self.log.suite_start(ids_by_manifest, name=options.suite)
 
         overall = 0
+        status = -1
         for manifest, tests in tests_by_manifest.items():
-            self.log.info("Running tests in {}".format(manifest))
+            if self.getGtkTheme() != self.gtkTheme:
+                self.log.error(
+                    "Theme (%s) has changed to (%s), terminating job as this is unstable"
+                    % (self.gtkTheme, self.getGtkTheme())
+                )
+                return 1
+
+            self.log.info(f"Running tests in {manifest}")
+            self.currentManifest = manifest
             status = run(tests=tests)
             overall = overall or status
+        if status == -1:
+            # we didn't run anything
+            overall = 1
 
         self.log.suite_end(extra={"results": self.outputHandler.results})
         return overall
