@@ -13,16 +13,14 @@
 
 #include "mozilla/Attributes.h"
 #include "mozilla/gfx/Types.h"
+#include "mozilla/StaticPrefs_ui.h"
+#include "mozilla/TimeStamp.h"
 #include "nsAtom.h"
-#include "nsIDOMXULSelectCntrlEl.h"
-#include "nsGkAtoms.h"
 #include "nsCOMPtr.h"
-#include "nsMenuFrame.h"
+#include "nsIDOMEventListener.h"
+#include "nsXULPopupManager.h"
 
-#include "nsBoxFrame.h"
-#include "nsMenuParent.h"
-
-#include "nsITimer.h"
+#include "nsBlockFrame.h"
 
 #include "Units.h"
 
@@ -30,53 +28,17 @@ class nsIWidget;
 
 namespace mozilla {
 class PresShell;
+enum class WindowShadow : uint8_t;
 namespace dom {
 class KeyboardEvent;
+class XULButtonElement;
+class XULPopupElement;
 }  // namespace dom
+namespace widget {
+enum class PopupLevel : uint8_t;
+}
 }  // namespace mozilla
 
-// XUL popups can be in several different states. When opening a popup, the
-// state changes as follows:
-//   ePopupClosed - initial state
-//   ePopupShowing - during the period when the popupshowing event fires
-//   ePopupOpening - between the popupshowing event and being visible. Creation
-//                   of the child frames, layout and reflow occurs in this
-//                   state. The popup is stored in the popup manager's list of
-//                   open popups during this state.
-//   ePopupVisible - layout is done and the popup's view and widget are made
-//                   visible. The popup is visible on screen but may be
-//                   transitioning. The popupshown event has not yet fired.
-//   ePopupShown - the popup has been shown and is fully ready. This state is
-//                 assigned just before the popupshown event fires.
-// When closing a popup:
-//   ePopupHidden - during the period when the popuphiding event fires and
-//                  the popup is removed.
-//   ePopupClosed - the popup's widget is made invisible.
-enum nsPopupState {
-  // state when a popup is not open
-  ePopupClosed,
-  // state from when a popup is requested to be shown to after the
-  // popupshowing event has been fired.
-  ePopupShowing,
-  // state while a popup is waiting to be laid out and positioned
-  ePopupPositioning,
-  // state while a popup is open but the widget is not yet visible
-  ePopupOpening,
-  // state while a popup is visible and waiting for the popupshown event
-  ePopupVisible,
-  // state while a popup is open and visible on screen
-  ePopupShown,
-  // state from when a popup is requested to be hidden to when it is closed.
-  ePopupHiding,
-  // state which indicates that the popup was hidden without firing the
-  // popuphiding or popuphidden events. It is used when executing a menu
-  // command because the menu needs to be hidden before the command event
-  // fires, yet the popuphiding and popuphidden events are fired after. This
-  // state can also occur when the popup is removed because the document is
-  // unloaded.
-  ePopupInvisible
-};
-
 enum ConsumeOutsideClicksResult {
   ConsumeOutsideClicks_ParentOnly =
       0,                          // Only consume clicks on the parent anchor
@@ -84,27 +46,27 @@ enum ConsumeOutsideClicksResult {
   ConsumeOutsideClicks_Never = 2  // Never consume clicks
 };
 
-// How a popup may be flipped. Flipping to the outside edge is like how
-// a submenu would work. The entire popup is flipped to the opposite side
-// of the anchor.
-enum FlipStyle {
-  FlipStyle_None = 0,
-  FlipStyle_Outside = 1,
-  FlipStyle_Inside = 2
+// How a popup may be flipped. Flipping to the outside edge is like how a
+// submenu would work. The entire popup is flipped to the opposite side of the
+// anchor.
+enum class FlipStyle {
+  None = 0,
+  Outside = 1,
+  Inside = 2,
 };
 
 // Values for the flip attribute
-enum FlipType {
-  FlipType_Default = 0,
-  FlipType_None = 1,  // don't try to flip or translate to stay onscreen
-  FlipType_Both = 2,  // flip in both directions
-  FlipType_Slide = 3  // allow the arrow to "slide" instead of resizing
+enum class FlipType {
+  Default = 0,
+  None = 1,   // don't try to flip or translate to stay onscreen
+  Both = 2,   // flip in both directions
+  Slide = 3,  // allow the arrow to "slide" instead of resizing
 };
 
-enum MenuPopupAnchorType {
-  MenuPopupAnchorType_Node = 0,   // anchored to a node
-  MenuPopupAnchorType_Point = 1,  // unanchored and positioned at a screen point
-  MenuPopupAnchorType_Rect = 2,   // anchored at a screen rectangle
+enum class MenuPopupAnchorType : uint8_t {
+  Node = 0,   // anchored to a node
+  Point = 1,  // unanchored, and positioned at a screen point
+  Rect = 2,   // anchored at a screen rectangle
 };
 
 // values are selected so that the direction can be flipped just by
@@ -160,42 +122,28 @@ class nsXULPopupShownEvent final : public mozilla::Runnable,
   void CancelListener();
 
  protected:
-  virtual ~nsXULPopupShownEvent() {}
+  virtual ~nsXULPopupShownEvent() = default;
 
  private:
-  nsCOMPtr<nsIContent> mPopup;
-  RefPtr<nsPresContext> mPresContext;
+  const nsCOMPtr<nsIContent> mPopup;
+  const RefPtr<nsPresContext> mPresContext;
 };
 
-class nsMenuPopupFrame final : public nsBoxFrame,
-                               public nsMenuParent,
-                               public nsIReflowCallback {
+class nsMenuPopupFrame final : public nsBlockFrame {
+  using PopupLevel = mozilla::widget::PopupLevel;
+  using PopupType = mozilla::widget::PopupType;
+
  public:
   NS_DECL_QUERYFRAME
   NS_DECL_FRAMEARENA_HELPERS(nsMenuPopupFrame)
 
   explicit nsMenuPopupFrame(ComputedStyle* aStyle, nsPresContext* aPresContext);
-
-  // nsMenuParent interface
-  virtual nsMenuFrame* GetCurrentMenuItem() override;
-  MOZ_CAN_RUN_SCRIPT_BOUNDARY
-  NS_IMETHOD SetCurrentMenuItem(nsMenuFrame* aMenuItem) override;
-  virtual void CurrentMenuIsBeingDestroyed() override;
-  MOZ_CAN_RUN_SCRIPT_BOUNDARY
-  NS_IMETHOD ChangeMenuItem(nsMenuFrame* aMenuItem, bool aSelectFirstItem,
-                            bool aFromKey) override;
+  ~nsMenuPopupFrame();
 
   // as popups are opened asynchronously, the popup pending state is used to
   // prevent multiple requests from attempting to open the same popup twice
-  nsPopupState PopupState() { return mPopupState; }
-  void SetPopupState(nsPopupState aPopupState) { mPopupState = aPopupState; }
-
-  NS_IMETHOD SetActive(bool aActiveFlag) override {
-    // We don't care.
-    return NS_OK;
-  }
-  virtual bool IsActive() override { return false; }
-  virtual bool IsMenuBar() override { return false; }
+  nsPopupState PopupState() const { return mPopupState; }
+  void SetPopupState(nsPopupState);
 
   /*
    * When this popup is open, should clicks outside of it be consumed?
@@ -215,93 +163,130 @@ class nsMenuPopupFrame final : public nsBoxFrame,
    */
   ConsumeOutsideClicksResult ConsumeOutsideClicks();
 
-  virtual bool IsContextMenu() override { return mIsContextMenu; }
+  mozilla::dom::XULPopupElement& PopupElement() const;
 
-  virtual bool MenuClosed() override { return true; }
+  nscoord IntrinsicISize(const mozilla::IntrinsicSizeInput& aInput,
+                         mozilla::IntrinsicISizeType aType) override;
 
-  virtual void LockMenuUntilClosed(bool aLock) override;
-  virtual bool IsMenuLocked() override { return mIsMenuLocked; }
+  void Reflow(nsPresContext* aPresContext, ReflowOutput& aDesiredSize,
+              const ReflowInput& aReflowInput,
+              nsReflowStatus& aStatus) override;
 
-  nsIWidget* GetWidget();
+  nsIWidget* GetWidget() const;
+  already_AddRefed<nsIWidget> ComputeParentWidget() const;
+
+  enum class WidgetStyle : uint8_t {
+    ColorScheme,
+    InputRegion,
+    Opacity,
+    Shadow,
+    Transform,
+    MicaBackdrop,
+  };
+  using WidgetStyleFlags = mozilla::EnumSet<WidgetStyle>;
+  static constexpr WidgetStyleFlags AllWidgetStyleFlags() {
+    return {WidgetStyle::ColorScheme, WidgetStyle::InputRegion,
+            WidgetStyle::Opacity,     WidgetStyle::Shadow,
+            WidgetStyle::Transform,   WidgetStyle::MicaBackdrop};
+  }
+  void PropagateStyleToWidget(WidgetStyleFlags = AllWidgetStyleFlags()) const;
 
   // Overridden methods
-  virtual void Init(nsIContent* aContent, nsContainerFrame* aParent,
-                    nsIFrame* aPrevInFlow) override;
+  void Init(nsIContent* aContent, nsContainerFrame* aParent,
+            nsIFrame* aPrevInFlow) override;
 
-  virtual nsresult AttributeChanged(int32_t aNameSpaceID, nsAtom* aAttribute,
-                                    int32_t aModType) override;
+  nsresult AttributeChanged(int32_t aNameSpaceID, nsAtom* aAttribute,
+                            int32_t aModType) override;
 
-  virtual void DestroyFrom(nsIFrame* aDestructRoot,
-                           PostDestroyData& aPostDestroyData) override;
+  // FIXME: This shouldn't run script (this can end up calling HidePopup).
+  MOZ_CAN_RUN_SCRIPT_BOUNDARY void Destroy(DestroyContext&) override;
 
   bool HasRemoteContent() const;
 
-  // returns true if the popup is a panel with the noautohide attribute set to
+  // Whether we should create a widget on Init().
+  bool ShouldCreateWidgetUpfront() const;
+
+  // Whether we should expand the menu to take the size of the parent menulist.
+  bool ShouldExpandToInflowParentOrAnchor() const;
+
+  // Returns true if the popup is a panel with the noautohide attribute set to
   // true. These panels do not roll up automatically.
   bool IsNoAutoHide() const;
 
-  nsPopupLevel PopupLevel() const { return PopupLevel(IsNoAutoHide()); }
+  PopupLevel GetPopupLevel() const { return GetPopupLevel(IsNoAutoHide()); }
 
   // Ensure that a widget has already been created for this view, and create
-  // one if it hasn't. If aRecreate is true, destroys any existing widget and
-  // creates a new one, regardless of whether one has already been created.
-  void EnsureWidget(bool aRecreate = false);
+  // one if it hasn't. If aForceRecreate is true, destroys any existing widget
+  // and creates a new one, regardless of whether one has already been created.
+  // Otherwise does so only if needed.
+  void PrepareWidget(bool aForceRecreate = false);
+
+  MOZ_CAN_RUN_SCRIPT void EnsureActiveMenuListItemIsVisible();
 
   nsresult CreateWidgetForView(nsView* aView);
-  uint8_t GetShadowStyle();
+  mozilla::WindowShadow GetShadowStyle() const;
 
-  virtual bool IsLeafDynamic() const override;
-
-  virtual void UpdateWidgetProperties() override;
+  void DidSetComputedStyle(ComputedStyle* aOldStyle) override;
 
   // layout, position and display the popup as needed
   MOZ_CAN_RUN_SCRIPT_BOUNDARY
-  void LayoutPopup(nsBoxLayoutState& aState, nsIFrame* aParentMenu,
-                   nsIFrame* aAnchor, bool aSizedToPopup);
+  void LayoutPopup(nsPresContext*, ReflowOutput&, const ReflowInput&,
+                   nsReflowStatus&);
 
-  nsView* GetRootViewForPopup(nsIFrame* aStartFrame);
-
-  // Set the position of the popup either relative to the anchor aAnchorFrame
-  // (or the frame for mAnchorContent if aAnchorFrame is null), anchored at a
+  // Set the position of the popup relative to the anchor content, anchored at a
   // rectangle, or at a specific point if a screen position is set. The popup
   // will be adjusted so that it is on screen. If aIsMove is true, then the
-  // popup is being moved, and should not be flipped. If aNotify is true, then
-  // a popuppositioned event is sent.
-  nsresult SetPopupPosition(nsIFrame* aAnchorFrame, bool aIsMove,
-                            bool aSizedToPopup, bool aNotify);
+  // popup is being moved, and should not be flipped.
+  void SetPopupPosition(bool aIsMove);
 
-  // Force the children to be generated if they have not already been generated.
-  void GenerateFrames();
+  // Called when the Enter key is pressed while the popup is open. This will
+  // just pass the call down to the current menu, if any.
+  // Also, calling Enter will reset the current incremental search string,
+  // calculated in FindMenuWithShortcut.
+  MOZ_CAN_RUN_SCRIPT void HandleEnterKeyPress(mozilla::WidgetEvent&);
 
-  // called when the Enter key is pressed while the popup is open. This will
-  // just pass the call down to the current menu, if any. If a current menu
-  // should be opened as a result, this method should return the frame for
-  // that menu, or null if no menu should be opened. Also, calling Enter will
-  // reset the current incremental search string, calculated in
-  // FindMenuWithShortcut.
-  nsMenuFrame* Enter(mozilla::WidgetGUIEvent* aEvent);
+  // Locate and return the menu frame that should be activated for the supplied
+  // key event. If aDoAction is set to true by this method, then the menu's
+  // action should be carried out, as if the user had pressed the Enter key. If
+  // aDoAction is false, the menu should just be highlighted.
+  // This method also handles incremental searching in menus so the user can
+  // type the first few letters of an item/s name to select it.
+  mozilla::dom::XULButtonElement* FindMenuWithShortcut(
+      mozilla::dom::KeyboardEvent& aKeyEvent, bool& aDoAction);
 
-  nsPopupType PopupType() const { return mPopupType; }
-  bool IsMenu() override { return mPopupType == ePopupTypeMenu; }
-  bool IsOpen() override {
+  mozilla::dom::XULButtonElement* GetCurrentMenuItem() const;
+  nsIFrame* GetCurrentMenuItemFrame() const;
+
+  PopupType GetPopupType() const { return mPopupType; }
+  bool IsContextMenu() const { return mIsContextMenu; }
+
+  bool IsOpen() const {
     return mPopupState == ePopupOpening || mPopupState == ePopupVisible ||
            mPopupState == ePopupShown;
   }
   bool IsVisible() {
     return mPopupState == ePopupVisible || mPopupState == ePopupShown;
   }
+  bool IsVisibleOrShowing() {
+    return IsOpen() || mPopupState == ePopupPositioning ||
+           mPopupState == ePopupShowing;
+  }
+  bool IsNativeMenu() const { return mIsNativeMenu; }
+  bool IsMouseTransparent() const;
 
   // Return true if the popup is for a menulist.
-  bool IsMenuList();
+  bool IsMenuList() const;
 
-  bool IsMouseTransparent() { return mMouseTransparent; }
+  bool IsDragSource() const { return mIsDragSource; }
+  void SetIsDragSource(bool aIsDragSource) { mIsDragSource = aIsDragSource; }
 
   static nsIContent* GetTriggerContent(nsMenuPopupFrame* aMenuPopupFrame);
   void ClearTriggerContent() { mTriggerContent = nullptr; }
+  void ClearTriggerContentIncludingDocument();
 
   // returns true if the popup is in a content shell, or false for a popup in
   // a chrome shell
-  bool IsInContentShell() { return mInContentShell; }
+  bool IsInContentShell() const { return mInContentShell; }
 
   // the Initialize methods are used to set the anchor position for
   // each way of opening a popup.
@@ -322,91 +307,96 @@ class nsMenuPopupFrame final : public nsBoxFrame,
   void InitializePopupAtScreen(nsIContent* aTriggerContent, int32_t aXPos,
                                int32_t aYPos, bool aIsContextMenu);
 
+  // Called if this popup should be displayed as an OS-native context menu.
+  void InitializePopupAsNativeContextMenu(nsIContent* aTriggerContent,
+                                          int32_t aXPos, int32_t aYPos);
+
   // indicate that the popup should be opened
   void ShowPopup(bool aIsContextMenu);
   // indicate that the popup should be hidden. The new state should either be
   // ePopupClosed or ePopupInvisible.
-  void HidePopup(bool aDeselectMenu, nsPopupState aNewState);
-
-  // locate and return the menu frame that should be activated for the
-  // supplied key event. If doAction is set to true by this method,
-  // then the menu's action should be carried out, as if the user had pressed
-  // the Enter key. If doAction is false, the menu should just be highlighted.
-  // This method also handles incremental searching in menus so the user can
-  // type the first few letters of an item/s name to select it.
-  nsMenuFrame* FindMenuWithShortcut(mozilla::dom::KeyboardEvent* aKeyEvent,
-                                    bool& doAction);
+  MOZ_CAN_RUN_SCRIPT void HidePopup(bool aDeselectMenu, nsPopupState aNewState,
+                                    bool aFromFrameDestruction = false);
 
   void ClearIncrementalString() { mIncrementalString.Truncate(); }
-  static bool IsWithinIncrementalTime(DOMTimeStamp time) {
-    return !sTimeoutOfIncrementalSearch ||
-           time - sLastKeyTime <= sTimeoutOfIncrementalSearch;
+  static bool IsWithinIncrementalTime(mozilla::TimeStamp time) {
+    return !sLastKeyTime.IsNull() &&
+           ((time - sLastKeyTime).ToMilliseconds() <=
+            mozilla::StaticPrefs::ui_menu_incremental_search_timeout());
   }
 
 #ifdef DEBUG_FRAME_DUMP
   virtual nsresult GetFrameName(nsAString& aResult) const override {
-    return MakeFrameName(NS_LITERAL_STRING("MenuPopup"), aResult);
+    return MakeFrameName(u"MenuPopup"_ns, aResult);
   }
 #endif
 
-  MOZ_CAN_RUN_SCRIPT void EnsureMenuItemIsVisible(nsMenuFrame* aMenuFrame);
-
-  void ChangeByPage(bool aIsUp);
+  MOZ_CAN_RUN_SCRIPT void ChangeByPage(bool aIsUp);
 
   // Move the popup to the screen coordinate |aPos| in CSS pixels.
   // If aUpdateAttrs is true, and the popup already has left or top attributes,
   // then those attributes are updated to the new location.
   // The frame may be destroyed by this method.
-  void MoveTo(const mozilla::CSSIntPoint& aPos, bool aUpdateAttrs);
+  void MoveTo(const mozilla::CSSPoint& aPos, bool aUpdateAttrs,
+              bool aByMoveToRect = false);
 
   void MoveToAnchor(nsIContent* aAnchorContent, const nsAString& aPosition,
                     int32_t aXPos, int32_t aYPos, bool aAttributesOverride);
 
-  bool GetAutoPosition();
-  void SetAutoPosition(bool aShouldAutoPosition);
+  mozilla::ScrollContainerFrame* GetScrollContainerFrame() const;
 
-  nsIScrollableFrame* GetScrollFrame(nsIFrame* aStart);
-
-  void SetOverrideConstraintRect(mozilla::LayoutDeviceIntRect aRect) {
-    mOverrideConstraintRect = ToAppUnits(aRect, mozilla::AppUnitsPerCSSPixel());
+  void SetOverrideConstraintRect(const mozilla::CSSIntRect& aRect) {
+    mOverrideConstraintRect = mozilla::CSSIntRect::ToAppUnits(aRect);
   }
 
-  // For a popup that should appear anchored at the given rect, determine
-  // the screen area that it is constrained by. This will be the available
-  // area of the screen the popup should be displayed on. Content popups,
-  // however, will also be constrained by the content area, given by
-  // aRootScreenRect. All coordinates are in app units.
+  bool IsConstrainedByLayout() const { return mConstrainedByLayout; }
+
+  struct Rects {
+    // For anchored popups, the anchor rectangle. For non-anchored popups, the
+    // size will be 0.
+    nsRect mAnchorRect;
+    // mAnchorRect before accounting for flipping / resizing / intersecting with
+    // the screen. This is needed for Wayland, which flips / resizes at the
+    // widget level.
+    nsRect mUntransformedAnchorRect;
+    // The final used rect we want to occupy.
+    nsRect mUsedRect;
+    // The alignment offset for sliding the panel, see
+    // nsMenuPopupFrame::mAlignmentOffset.
+    nscoord mAlignmentOffset = 0;
+    bool mHFlip = false;
+    bool mVFlip = false;
+    bool mConstrainedByLayout = false;
+    // The client offset of our widget.
+    mozilla::LayoutDeviceIntPoint mClientOffset;
+    nsPoint mViewPoint;
+  };
+
+  // For a popup that should appear anchored at the given rect, gets the anchor
+  // and constraint rects for that popup.
+  // This will be the available area of the screen the popup should be displayed
+  // on. Content popups, however, will also be constrained by the content area.
+  //
   // For non-toplevel popups (which will always be panels), we will also
   // constrain them to the available screen rect, ie they will not fall
   // underneath the taskbar, dock or other fixed OS elements.
-  // This operates in device pixels.
-  mozilla::LayoutDeviceIntRect GetConstraintRect(
-      const mozilla::LayoutDeviceIntRect& aAnchorRect,
-      const mozilla::LayoutDeviceIntRect& aRootScreenRect,
-      nsPopupLevel aPopupLevel);
-
-  // Determines whether the given edges of the popup may be moved, where
-  // aHorizontalSide and aVerticalSide are one of the enum Side constants.
-  // aChange is the distance to move on those sides. If will be reset to 0
-  // if the side cannot be adjusted at all in that direction. For example, a
-  // popup cannot be moved if it is anchored on a particular side.
-  //
-  // Later, when bug 357725 is implemented, we can make this adjust aChange by
-  // the amount that the side can be resized, so that minimums and maximums
-  // can be taken into account.
-  void CanAdjustEdges(mozilla::Side aHorizontalSide,
-                      mozilla::Side aVerticalSide,
-                      mozilla::LayoutDeviceIntPoint& aChange);
+  Rects GetRects(const nsSize& aPrefSize) const;
+  Maybe<nsRect> GetConstraintRect(const nsRect& aAnchorRect,
+                                  const nsRect& aRootScreenRect,
+                                  PopupLevel) const;
+  void PerformMove(const Rects&);
 
   // Return true if the popup is positioned relative to an anchor.
-  bool IsAnchored() const { return mAnchorType != MenuPopupAnchorType_Point; }
+  bool IsAnchored() const { return mAnchorType != MenuPopupAnchorType::Point; }
 
   // Return the anchor if there is one.
   nsIContent* GetAnchor() const { return mAnchorContent; }
 
   // Return the screen coordinates in CSS pixels of the popup,
   // or (-1, -1, 0, 0) if anchored.
-  nsIntRect GetScreenAnchorRect() const { return mScreenRect; }
+  mozilla::CSSIntRect GetScreenAnchorRect() const {
+    return mozilla::CSSRect::FromAppUnitsRounded(mScreenRect);
+  }
 
   mozilla::LayoutDeviceIntPoint GetLastClientOffset() const {
     return mLastClientOffset;
@@ -430,10 +420,7 @@ class nsMenuPopupFrame final : public nsBoxFrame,
     return false;
   }
 
-  void ShowWithPositionedEvent() {
-    mPopupState = ePopupPositioning;
-    mShouldAutoPosition = true;
-  }
+  void ShowWithPositionedEvent() { mPopupState = ePopupPositioning; }
 
   // Checks for the anchor to change and either moves or hides the popup
   // accordingly. The original position of the anchor should be supplied as
@@ -442,16 +429,11 @@ class nsMenuPopupFrame final : public nsBoxFrame,
   // with the new rectangle.
   void CheckForAnchorChange(nsRect& aRect);
 
-  // nsIReflowCallback
-  virtual bool ReflowFinished() override;
-  virtual void ReflowCallbackCanceled() override;
+  void WillDispatchPopupPositioned() { mPendingPositionedEvent = false; }
 
  protected:
   // returns the popup's level.
-  nsPopupLevel PopupLevel(bool aIsNoAutoHide) const;
-
-  // redefine to tell the box system not to move the views.
-  virtual uint32_t GetXULLayoutFlags() override;
+  PopupLevel GetPopupLevel(bool aIsNoAutoHide) const;
 
   void InitPositionFromAnchorAlign(const nsAString& aAnchor,
                                    const nsAString& aAlign);
@@ -459,12 +441,14 @@ class nsMenuPopupFrame final : public nsBoxFrame,
   // return the position where the popup should be, when it should be
   // anchored at anchorRect. aHFlip and aVFlip will be set if the popup may be
   // flipped in that direction if there is not enough space available.
-  nsPoint AdjustPositionForAnchorAlign(nsRect& anchorRect, FlipStyle& aHFlip,
-                                       FlipStyle& aVFlip);
+  nsPoint AdjustPositionForAnchorAlign(nsRect& aAnchorRect,
+                                       const nsSize& aPrefSize,
+                                       FlipStyle& aHFlip,
+                                       FlipStyle& aVFlip) const;
 
   // For popups that are going to align to their selected item, get the frame of
   // the selected item.
-  nsIFrame* GetSelectedItemForAlignment();
+  nsIFrame* GetSelectedItemForAlignment() const;
 
   // check if the popup will fit into the available space and resize it. This
   // method handles only one axis at a time so is called twice, once for
@@ -478,15 +462,13 @@ class nsMenuPopupFrame final : public nsBoxFrame,
   //   aAnchorEnd - the right or bottom edge of the anchor rectangle
   //   aMarginBegin - the left or top margin of the popup
   //   aMarginEnd - the right or bottom margin of the popup
-  //   aOffsetForContextMenu - the additional offset to add for context menus
   //   aFlip - how to flip or resize the popup when there isn't space
   //   aFlipSide - pointer to where current flip mode is stored
   nscoord FlipOrResize(nscoord& aScreenPoint, nscoord aSize,
                        nscoord aScreenBegin, nscoord aScreenEnd,
                        nscoord aAnchorBegin, nscoord aAnchorEnd,
                        nscoord aMarginBegin, nscoord aMarginEnd,
-                       nscoord aOffsetForContextMenu, FlipStyle aFlip,
-                       bool aIsOnEnd, bool* aFlipSide);
+                       FlipStyle aFlip, bool aIsOnEnd, bool* aFlipSide) const;
 
   // check if the popup can fit into the available space by "sliding" (i.e.,
   // by having the anchor arrow slide along one axis and only resizing if that
@@ -504,32 +486,17 @@ class nsMenuPopupFrame final : public nsBoxFrame,
   // as aSize, unless aSize is greater than the screen width/height.
   nscoord SlideOrResize(nscoord& aScreenPoint, nscoord aSize,
                         nscoord aScreenBegin, nscoord aScreenEnd,
-                        nscoord* aOffset);
+                        nscoord* aOffset) const;
 
   // Given an anchor frame, compute the anchor rectangle relative to the screen,
   // using the popup frame's app units, and taking into account transforms.
   nsRect ComputeAnchorRect(nsPresContext* aRootPresContext,
-                           nsIFrame* aAnchorFrame);
+                           nsIFrame* aAnchorFrame) const;
 
   // Move the popup to the position specified in its |left| and |top|
   // attributes.
   void MoveToAttributePosition();
 
-  /**
-   * Return whether the popup direction should be RTL.
-   * If the popup has an anchor, its direction is the anchor direction.
-   * Otherwise, its the general direction of the UI.
-   *
-   * Return whether the popup direction should be RTL.
-   */
-  bool IsDirectionRTL() const {
-    return mAnchorContent && mAnchorContent->GetPrimaryFrame()
-               ? mAnchorContent->GetPrimaryFrame()
-                         ->StyleVisibility()
-                         ->mDirection == NS_STYLE_DIRECTION_RTL
-               : StyleVisibility()->mDirection == NS_STYLE_DIRECTION_RTL;
-  }
-
   // Create a popup view for this frame. The view is added a child of the root
   // view, and is initially hidden.
   void CreatePopupView();
@@ -543,9 +510,20 @@ class nsMenuPopupFrame final : public nsBoxFrame,
   // is no longer visible, or a parent deck page is changed, the popup hides
   // as well. The second variation also sets the anchor rectangle, relative to
   // the popup frame.
-  bool ShouldFollowAnchor();
+  bool ShouldFollowAnchor() const;
+
+  nsIFrame* GetAnchorFrame() const;
 
  public:
+  /**
+   * Return whether the popup direction should be RTL.
+   * If the popup has an anchor, its direction is the anchor direction.
+   * Otherwise, its the general direction of the UI.
+   *
+   * Return whether the popup direction should be RTL.
+   */
+  bool IsDirectionRTL() const;
+
   bool ShouldFollowAnchor(nsRect& aRect);
 
   // Returns parent menu widget for submenus that are in the same
@@ -553,6 +531,31 @@ class nsMenuPopupFrame final : public nsBoxFrame,
   // strict popup windows hierarchy.
   nsIWidget* GetParentMenuWidget();
 
+  // Returns the effective margin for this popup. This is the CSS margin plus
+  // the context-menu shift, if needed.
+  nsMargin GetMargin() const;
+
+  // These are used by Wayland backend.
+  const nsRect& GetUntransformedAnchorRect() const {
+    return mUntransformedAnchorRect;
+  }
+  int8_t GetUntransformedPopupAlignment() const {
+    return mUntransformedPopupAlignment;
+  }
+  int8_t GetUntransformedPopupAnchor() const {
+    return mUntransformedPopupAnchor;
+  }
+
+  int8_t GetPopupAlignment() const { return mPopupAlignment; }
+  int8_t GetPopupAnchor() const { return mPopupAnchor; }
+  FlipType GetFlipType() const { return mFlip; }
+
+  uint64_t GetAPZFocusSequenceNumber() const { return mAPZFocusSequenceNumber; }
+
+  void UpdateAPZFocusSequenceNumber(uint64_t aNewNumber) {
+    mAPZFocusSequenceNumber = aNewNumber;
+  }
+
  protected:
   nsString mIncrementalString;  // for incremental typing navigation
 
@@ -564,113 +567,100 @@ class nsMenuPopupFrame final : public nsBoxFrame,
   // was clicked. It will be cleared when the popup is hidden.
   nsCOMPtr<nsIContent> mTriggerContent;
 
-  nsMenuFrame* mCurrentMenu;  // The current menu that is active.
-  nsView* mView;
+  nsView* mView = nullptr;
 
   RefPtr<nsXULPopupShownEvent> mPopupShownDispatcher;
 
   // The popup's screen rectangle in app units.
-  nsIntRect mUsedScreenRect;
+  nsRect mUsedScreenRect;
 
   // A popup's preferred size may be different than its actual size stored in
   // mRect in the case where the popup was resized because it was too large
   // for the screen. The preferred size mPrefSize holds the full size the popup
   // would be before resizing. Computations are performed using this size.
-  nsSize mPrefSize;
+  nsSize mPrefSize{-1, -1};
 
-  // The position of the popup, in CSS pixels.
-  // The screen coordinates, if set to values other than -1,
-  // override mXPos and mYPos.
-  int32_t mXPos;
-  int32_t mYPos;
-  nsIntRect mScreenRect;
+  // A point with extra offsets to apply in the horizontal and vertical axes. We
+  // don't use an nsMargin because the values would be the same for the same
+  // axis.
+  nsPoint mExtraMargin;
+
+  nsRect mScreenRect;
+  // Used for store rectangle which the popup is going to be anchored to, we
+  // need that for Wayland. It's important that this rect is unflipped, and
+  // without margins applied, as GTK is what takes care of determining how to
+  // flip etc. on Wayland.
+  nsRect mUntransformedAnchorRect;
 
   // If the panel prefers to "slide" rather than resize, then the arrow gets
   // positioned at this offset (along either the x or y axis, depending on
   // mPosition)
-  nscoord mAlignmentOffset;
+  nscoord mAlignmentOffset = 0;
 
   // The value of the client offset of our widget the last time we positioned
   // ourselves. We store this so that we can detect when it changes but the
   // position of our widget didn't change.
   mozilla::LayoutDeviceIntPoint mLastClientOffset;
 
-  nsPopupType mPopupType;    // type of popup
-  nsPopupState mPopupState;  // open state of the popup
+  // The focus sequence number of the last processed input event
+  uint64_t mAPZFocusSequenceNumber = 0;
+
+  PopupType mPopupType = PopupType::Panel;  // type of popup
+  nsPopupState mPopupState = ePopupClosed;  // open state of the popup
 
   // popup alignment relative to the anchor node
-  int8_t mPopupAlignment;
-  int8_t mPopupAnchor;
-  int8_t mPosition;
+  // The untransformed variants are needed for Wayland
+  int8_t mUntransformedPopupAlignment = POPUPALIGNMENT_NONE;
+  int8_t mUntransformedPopupAnchor = POPUPALIGNMENT_NONE;
+  int8_t mPopupAlignment = POPUPALIGNMENT_NONE;
+  int8_t mPopupAnchor = POPUPALIGNMENT_NONE;
+  int8_t mPosition = POPUPPOSITION_UNKNOWN;
 
-  FlipType mFlip;  // Whether to flip
+  FlipType mFlip = FlipType::Default;  // Whether to flip
 
-  struct ReflowCallbackData {
-    ReflowCallbackData()
-        : mPosted(false),
-          mAnchor(nullptr),
-          mSizedToPopup(false),
-          mIsOpenChanged(false) {}
-    void MarkPosted(nsIFrame* aAnchor, bool aSizedToPopup,
-                    bool aIsOpenChanged) {
-      mPosted = true;
-      mAnchor = aAnchor;
-      mSizedToPopup = aSizedToPopup;
-      mIsOpenChanged = aIsOpenChanged;
-    }
-    void Clear() {
-      mPosted = false;
-      mAnchor = nullptr;
-      mSizedToPopup = false;
-      mIsOpenChanged = false;
-    }
-    bool mPosted;
-    nsIFrame* mAnchor;
-    bool mSizedToPopup;
-    bool mIsOpenChanged;
-  };
-  ReflowCallbackData mReflowCallbackData;
+  // Whether we were moved by the move-to-rect Wayland callback. In that case,
+  // we stop updating the anchor so that we can end up with a stable position.
+  bool mPositionedByMoveToRect = false;
+  // true if the open state changed since the last layout.
+  bool mIsOpenChanged = false;
+  // true for context menus and their submenus.
+  bool mIsContextMenu = false;
+  // true for the topmost context menu.
+  bool mIsTopLevelContextMenu = false;
+  // true if the popup is in a content shell.
+  bool mInContentShell = true;
 
-  bool mIsOpenChanged;  // true if the open state changed since the last layout
-  bool mIsContextMenu;  // true for context menus
-  // true if we need to offset the popup to ensure it's not under the mouse
-  bool mAdjustOffsetForContextMenu;
-  bool mGeneratedChildren;  // true if the contents have been created
+  // The flip modes that were used when the popup was opened
+  bool mHFlip = false;
+  bool mVFlip = false;
+  // Whether layout has constrained this popup in some way.
+  bool mConstrainedByLayout = false;
 
-  bool mMenuCanOverlapOSBar;  // can we appear over the taskbar/menubar?
-  bool mShouldAutoPosition;   // Should SetPopupPosition be allowed to auto
-                              // position popup?
-  bool mInContentShell;       // True if the popup is in a content shell
-  bool mIsMenuLocked;         // Should events inside this menu be ignored?
-  bool mMouseTransparent;     // True if this is a popup is transparent to mouse
-                              // events
+  // Whether the most recent initialization of this menupopup happened via
+  // InitializePopupAsNativeContextMenu.
+  bool mIsNativeMenu = false;
 
-  // True if this popup has been offset due to moving off / near the edge of the
-  // screen. (This is useful for ensuring that a move, which can't offset the
-  // popup, doesn't undo a previously set offset.)
-  bool mIsOffset;
+  // Whether we have a pending `popuppositioned` event.
+  bool mPendingPositionedEvent = false;
 
-  // the flip modes that were used when the popup was opened
-  bool mHFlip;
-  bool mVFlip;
+  // Whether this popup is source of D&D operation. We can't close such
+  // popup on Wayland as it cancel whole D&D operation.
+  bool mIsDragSource = false;
 
   // When POPUPPOSITION_SELECTION is used, this indicates the vertical offset
   // that the original selected item was. This needs to be used in case the
   // popup gets changed so that we can keep the popup at the same vertical
   // offset.
-  nscoord mPositionedOffset;
+  // TODO(emilio): try to make this not mutable.
+  mutable nscoord mPositionedOffset = 0;
 
   // How the popup is anchored.
-  MenuPopupAnchorType mAnchorType;
+  MenuPopupAnchorType mAnchorType = MenuPopupAnchorType::Node;
 
   nsRect mOverrideConstraintRect;
 
-  static int8_t sDefaultLevelIsTop;
+  static mozilla::TimeStamp sLastKeyTime;
 
-  static DOMTimeStamp sLastKeyTime;
-
-  // If 0, never timed out.  Otherwise, the value is in milliseconds.
-  static uint32_t sTimeoutOfIncrementalSearch;
 };  // class nsMenuPopupFrame
 
 #endif
