@@ -10,7 +10,6 @@
 #include "mozilla/AtomArray.h"
 #include "mozilla/Attributes.h"
 
-#include "nsLeafBoxFrame.h"
 #include "nsITreeView.h"
 #include "nsIScrollbarMediator.h"
 #include "nsITimer.h"
@@ -18,11 +17,13 @@
 #include "nsTArray.h"
 #include "nsTreeStyleCache.h"
 #include "nsTreeColumns.h"
-#include "nsDataHashtable.h"
+#include "nsTHashMap.h"
+#include "nsTHashSet.h"
 #include "imgIRequest.h"
 #include "imgINotificationObserver.h"
 #include "nsScrollbarFrame.h"
 #include "nsThreadUtils.h"
+#include "SimpleXULLeafFrame.h"
 #include "mozilla/LookAndFeel.h"
 
 class nsFontMetrics;
@@ -31,6 +32,7 @@ class nsTreeImageListener;
 
 namespace mozilla {
 class PresShell;
+class ScrollContainerFrame;
 namespace layout {
 class ScrollbarActivity;
 }  // namespace layout
@@ -38,17 +40,16 @@ class ScrollbarActivity;
 
 // An entry in the tree's image cache
 struct nsTreeImageCacheEntry {
-  nsTreeImageCacheEntry() = default;
-  nsTreeImageCacheEntry(imgIRequest* aRequest,
-                        imgINotificationObserver* aListener)
-      : request(aRequest), listener(aListener) {}
+  nsTreeImageCacheEntry();
+  nsTreeImageCacheEntry(imgIRequest* aRequest, nsTreeImageListener* aListener);
+  ~nsTreeImageCacheEntry();
 
   nsCOMPtr<imgIRequest> request;
-  nsCOMPtr<imgINotificationObserver> listener;
+  RefPtr<nsTreeImageListener> listener;
 };
 
 // The actual frame that paints the cells and rows.
-class nsTreeBodyFrame final : public nsLeafBoxFrame,
+class nsTreeBodyFrame final : public mozilla::SimpleXULLeafFrame,
                               public nsIScrollbarMediator,
                               public nsIReflowCallback {
   typedef mozilla::layout::ScrollbarActivity ScrollbarActivity;
@@ -58,13 +59,15 @@ class nsTreeBodyFrame final : public nsLeafBoxFrame,
   explicit nsTreeBodyFrame(ComputedStyle* aStyle, nsPresContext* aPresContext);
   ~nsTreeBodyFrame();
 
+  mozilla::IntrinsicSize GetIntrinsicSize() override;
+
   NS_DECL_QUERYFRAME
   NS_DECL_FRAMEARENA_HELPERS(nsTreeBodyFrame)
 
   // Callback handler methods for refresh driver based animations.
   // Calls to these functions are forwarded from nsTreeImageListener. These
   // mirror how nsImageFrame works.
-  nsresult OnImageIsAnimated(imgIRequest* aRequest);
+  void OnImageIsAnimated(imgIRequest* aRequest);
 
   // non-virtual signatures like nsITreeBodyFrame
   already_AddRefed<nsTreeColumns> Columns() const {
@@ -75,6 +78,7 @@ class nsTreeBodyFrame final : public nsLeafBoxFrame,
     nsCOMPtr<nsITreeView> view = mView;
     return view.forget();
   }
+  already_AddRefed<nsITreeSelection> GetSelection() const;
   nsresult GetView(nsITreeView** aView);
   nsresult SetView(nsITreeView* aView);
   bool GetFocused() const { return mFocused; }
@@ -82,7 +86,6 @@ class nsTreeBodyFrame final : public nsLeafBoxFrame,
   nsresult GetTreeBody(mozilla::dom::Element** aElement);
   int32_t RowHeight() const;
   int32_t RowWidth();
-  int32_t GetHorizontalPosition() const;
   mozilla::Maybe<mozilla::CSSIntRegion> GetSelectionRegion();
   int32_t FirstVisibleRow() const { return mTopRowIndex; }
   int32_t LastVisibleRow() const { return mTopRowIndex + mPageLength; }
@@ -112,72 +115,66 @@ class nsTreeBodyFrame final : public nsLeafBoxFrame,
 
   void CancelImageRequests();
 
-  void ManageReflowCallback(const nsRect& aRect, nscoord aHorzWidth);
+  void ManageReflowCallback();
 
-  virtual nsSize GetXULMinSize(nsBoxLayoutState& aBoxLayoutState) override;
-  virtual void SetXULBounds(nsBoxLayoutState& aBoxLayoutState,
-                            const nsRect& aRect,
-                            bool aRemoveOverflowArea = false) override;
+  void DidReflow(nsPresContext*, const ReflowInput*) override;
 
   // nsIReflowCallback
-  virtual bool ReflowFinished() override;
-  virtual void ReflowCallbackCanceled() override;
+  bool ReflowFinished() override;
+  void ReflowCallbackCanceled() override;
 
   // nsIScrollbarMediator
-  virtual void ScrollByPage(nsScrollbarFrame* aScrollbar, int32_t aDirection,
-                            nsIScrollbarMediator::ScrollSnapMode aSnap =
-                                nsIScrollbarMediator::DISABLE_SNAP) override;
-  virtual void ScrollByWhole(nsScrollbarFrame* aScrollbar, int32_t aDirection,
-                             nsIScrollbarMediator::ScrollSnapMode aSnap =
-                                 nsIScrollbarMediator::DISABLE_SNAP) override;
-  virtual void ScrollByLine(nsScrollbarFrame* aScrollbar, int32_t aDirection,
-                            nsIScrollbarMediator::ScrollSnapMode aSnap =
-                                nsIScrollbarMediator::DISABLE_SNAP) override;
-  virtual void RepeatButtonScroll(nsScrollbarFrame* aScrollbar) override;
-  virtual void ThumbMoved(nsScrollbarFrame* aScrollbar, nscoord aOldPos,
-                          nscoord aNewPos) override;
-  virtual void ScrollbarReleased(nsScrollbarFrame* aScrollbar) override {}
-  virtual void VisibilityChanged(bool aVisible) override { Invalidate(); }
-  virtual nsIFrame* GetScrollbarBox(bool aVertical) override {
+  void ScrollByPage(nsScrollbarFrame* aScrollbar, int32_t aDirection,
+                    mozilla::ScrollSnapFlags aSnapFlags =
+                        mozilla::ScrollSnapFlags::Disabled) override;
+  void ScrollByWhole(nsScrollbarFrame* aScrollbar, int32_t aDirection,
+                     mozilla::ScrollSnapFlags aSnapFlags =
+                         mozilla::ScrollSnapFlags::Disabled) override;
+  void ScrollByLine(nsScrollbarFrame* aScrollbar, int32_t aDirection,
+                    mozilla::ScrollSnapFlags aSnapFlags =
+                        mozilla::ScrollSnapFlags::Disabled) override;
+  void ScrollByUnit(nsScrollbarFrame* aScrollbar, mozilla::ScrollMode aMode,
+                    int32_t aDirection, mozilla::ScrollUnit aUnit,
+                    mozilla::ScrollSnapFlags aSnapFlags =
+                        mozilla::ScrollSnapFlags::Disabled) override;
+  void RepeatButtonScroll(nsScrollbarFrame* aScrollbar) override;
+  void ThumbMoved(nsScrollbarFrame* aScrollbar, nscoord aOldPos,
+                  nscoord aNewPos) override;
+  void ScrollbarReleased(nsScrollbarFrame* aScrollbar) override {}
+  void VisibilityChanged(bool aVisible) override { Invalidate(); }
+  nsScrollbarFrame* GetScrollbarBox(bool aVertical) override {
     ScrollParts parts = GetScrollParts();
-    return aVertical ? parts.mVScrollbar : parts.mHScrollbar;
+    return aVertical ? parts.mVScrollbar : nullptr;
   }
-  virtual void ScrollbarActivityStarted() const override;
-  virtual void ScrollbarActivityStopped() const override;
-  virtual bool IsScrollbarOnRight() const override {
-    return (StyleVisibility()->mDirection == NS_STYLE_DIRECTION_LTR);
-  }
-  virtual bool ShouldSuppressScrollbarRepaints() const override {
-    return false;
+  void ScrollbarActivityStarted() const override;
+  void ScrollbarActivityStopped() const override;
+  bool IsScrollbarOnRight() const override {
+    return StyleVisibility()->mDirection == mozilla::StyleDirection::Ltr;
   }
+  bool ShouldSuppressScrollbarRepaints() const override { return false; }
 
   // Overridden from nsIFrame to cache our pres context.
-  virtual void Init(nsIContent* aContent, nsContainerFrame* aParent,
-                    nsIFrame* aPrevInFlow) override;
-  virtual void DestroyFrom(nsIFrame* aDestructRoot,
-                           PostDestroyData& aPostDestroyData) override;
+  void Init(nsIContent* aContent, nsContainerFrame* aParent,
+            nsIFrame* aPrevInFlow) override;
+  void Destroy(DestroyContext&) override;
 
-  mozilla::Maybe<Cursor> GetCursor(const nsPoint&) override;
+  Cursor GetCursor(const nsPoint&) override;
 
-  virtual nsresult HandleEvent(nsPresContext* aPresContext,
-                               mozilla::WidgetGUIEvent* aEvent,
-                               nsEventStatus* aEventStatus) override;
+  nsresult HandleEvent(nsPresContext* aPresContext,
+                       mozilla::WidgetGUIEvent* aEvent,
+                       nsEventStatus* aEventStatus) override;
 
-  virtual void BuildDisplayList(nsDisplayListBuilder* aBuilder,
-                                const nsDisplayListSet& aLists) override;
+  void BuildDisplayList(nsDisplayListBuilder* aBuilder,
+                        const nsDisplayListSet& aLists) override;
 
-  virtual void DidSetComputedStyle(ComputedStyle* aOldComputedStyle) override;
+  void DidSetComputedStyle(ComputedStyle* aOldComputedStyle) override;
 
   friend nsIFrame* NS_NewTreeBodyFrame(mozilla::PresShell* aPresShell);
   friend class nsTreeColumn;
 
   struct ScrollParts {
-    nsScrollbarFrame* mVScrollbar;
-    RefPtr<Element> mVScrollbarContent;
-    nsScrollbarFrame* mHScrollbar;
-    RefPtr<Element> mHScrollbarContent;
-    nsIFrame* mColumnsFrame;
-    nsIScrollableFrame* mColumnsScrollFrame;
+    nsScrollbarFrame* mVScrollbar = nullptr;
+    RefPtr<mozilla::dom::Element> mVScrollbarContent;
   };
 
   ImgDrawResult PaintTreeBody(gfxContext& aRenderingContext,
@@ -188,7 +185,6 @@ class nsTreeBodyFrame final : public nsLeafBoxFrame,
   mozilla::dom::XULTreeElement* GetBaseElement();
 
   bool GetVerticalOverflow() const { return mVerticalOverflow; }
-  bool GetHorizontalOverflow() const { return mHorizontalOverflow; }
 
   // This returns the property array where atoms are stored for style during
   // draw, whether the row currently being drawn is selected, hovered, etc.
@@ -293,15 +289,15 @@ class nsTreeBodyFrame final : public nsLeafBoxFrame,
                  nsCSSAnonBoxPseudoStaticAtom** aChildElt);
 
   // Retrieve the area for the twisty for a cell.
-  nsITheme* GetTwistyRect(int32_t aRowIndex, nsTreeColumn* aColumn,
-                          nsRect& aImageRect, nsRect& aTwistyRect,
-                          nsPresContext* aPresContext,
-                          ComputedStyle* aTwistyContext);
+  void GetTwistyRect(int32_t aRowIndex, nsTreeColumn* aColumn,
+                     nsRect& aImageRect, nsRect& aTwistyRect,
+                     nsPresContext* aPresContext,
+                     ComputedStyle* aTwistyContext);
 
-  // Fetch an image from the image cache.
-  nsresult GetImage(int32_t aRowIndex, nsTreeColumn* aCol, bool aUseContext,
-                    ComputedStyle* aComputedStyle, bool& aAllowImageRegions,
-                    imgIContainer** aResult);
+  // Fetch an image from the image cache, or request it.
+  already_AddRefed<imgIContainer> GetImage(int32_t aRowIndex,
+                                           nsTreeColumn* aCol, bool aUseContext,
+                                           ComputedStyle* aComputedStyle);
 
   // Returns the size of a given image.   This size *includes* border and
   // padding.  It does not include margins.
@@ -310,12 +306,10 @@ class nsTreeBodyFrame final : public nsLeafBoxFrame,
 
   // Returns the destination size of the image, not including borders and
   // padding.
-  nsSize GetImageDestSize(ComputedStyle* aComputedStyle, bool useImageRegion,
-                          imgIContainer* image);
+  nsSize GetImageDestSize(ComputedStyle*, imgIContainer*);
 
   // Returns the source rectangle of the image to be displayed.
-  nsRect GetImageSourceRect(ComputedStyle* aComputedStyle, bool useImageRegion,
-                            imgIContainer* image);
+  nsRect GetImageSourceRect(ComputedStyle*, imgIContainer*);
 
   // Returns the height of rows in the tree.
   int32_t GetRowHeight();
@@ -345,8 +339,7 @@ class nsTreeBodyFrame final : public nsLeafBoxFrame,
   void UpdateScrollbars(const ScrollParts& aParts);
 
   // Update the maxpos of the scrollbar.
-  void InvalidateScrollbars(const ScrollParts& aParts,
-                            AutoWeakFrame& aWeakColumnsFrame);
+  void InvalidateScrollbars(const ScrollParts& aParts);
 
   // Check overflow and generate events.
   MOZ_CAN_RUN_SCRIPT_BOUNDARY void CheckOverflow(const ScrollParts& aParts);
@@ -363,7 +356,6 @@ class nsTreeBodyFrame final : public nsLeafBoxFrame,
   // Our internal scroll method, used by all the public scroll methods.
   nsresult ScrollInternal(const ScrollParts& aParts, int32_t aRow);
   nsresult ScrollToRowInternal(const ScrollParts& aParts, int32_t aRow);
-  nsresult ScrollHorzInternal(const ScrollParts& aParts, int32_t aPosition);
   nsresult EnsureRowIsVisibleInternal(const ScrollParts& aParts, int32_t aRow);
 
   // Convert client pixels into appunits in our coordinate space.
@@ -374,7 +366,6 @@ class nsTreeBodyFrame final : public nsLeafBoxFrame,
   nsresult GetCellWidth(int32_t aRow, nsTreeColumn* aCol,
                         gfxContext* aRenderingContext, nscoord& aDesiredSize,
                         nscoord& aCurrentSize);
-  nscoord CalcMaxRowWidth();
 
   // Translate the given rect horizontally from tree coordinates into the
   // coordinate system of our nsTreeBodyFrame.  If clip is true, then clip the
@@ -395,20 +386,11 @@ class nsTreeBodyFrame final : public nsLeafBoxFrame,
 
   void InvalidateDropFeedback(int32_t aRow, int16_t aOrientation) {
     InvalidateRow(aRow);
-    if (aOrientation != nsITreeView::DROP_ON)
+    if (aOrientation != nsITreeView::DROP_ON) {
       InvalidateRow(aRow + aOrientation);
+    }
   }
 
- public:
-  /**
-   * Remove an nsITreeImageListener from being tracked by this frame. Only tree
-   * image listeners that are created by this frame are tracked.
-   *
-   * @param aListener A pointer to an nsTreeImageListener to no longer
-   *        track.
-   */
-  void RemoveTreeImageListener(nsTreeImageListener* aListener);
-
  protected:
   // Create a new timer. This method is used to delay various actions like
   // opening/closing folders or tree scrolling.
@@ -438,7 +420,7 @@ class nsTreeBodyFrame final : public nsLeafBoxFrame,
   };
 
   void PostScrollEvent();
-  void FireScrollEvent();
+  MOZ_CAN_RUN_SCRIPT void FireScrollEvent();
 
   /**
    * Clear the pointer to this frame for all nsTreeImageListeners that were
@@ -479,38 +461,34 @@ class nsTreeBodyFrame final : public nsLeafBoxFrame,
  protected:  // Data Members
   class Slots {
    public:
-    Slots()
-        : mDropAllowed(false),
-          mIsDragging(false),
-          mDropRow(-1),
-          mDropOrient(-1),
-          mScrollLines(0),
-          mDragAction(0) {}
+    Slots() = default;
 
     ~Slots() {
-      if (mTimer) mTimer->Cancel();
+      if (mTimer) {
+        mTimer->Cancel();
+      }
     }
 
     friend class nsTreeBodyFrame;
 
    protected:
     // If the drop is actually allowed here or not.
-    bool mDropAllowed;
+    bool mDropAllowed = false;
 
     // True while dragging over the tree.
-    bool mIsDragging;
+    bool mIsDragging = false;
 
     // The row the mouse is hovering over during a drop.
-    int32_t mDropRow;
+    int32_t mDropRow = -1;
 
     // Where we want to draw feedback (above/on this row/below) if allowed.
-    int16_t mDropOrient;
+    int16_t mDropOrient = -1;
 
     // Number of lines to be scrolled.
-    int16_t mScrollLines;
+    int16_t mScrollLines = 0;
 
     // The drag action that was received for this slot
-    uint32_t mDragAction;
+    uint32_t mDragAction = 0;
 
     // Timer for opening/closing spring loaded folders or scrolling the tree.
     nsCOMPtr<nsITimer> mTimer;
@@ -519,7 +497,7 @@ class nsTreeBodyFrame final : public nsLeafBoxFrame,
     nsTArray<int32_t> mArray;
   };
 
-  Slots* mSlots;
+  mozilla::UniquePtr<Slots> mSlots;
 
   nsRevocableEventPtr<ScrollEvent> mScrollEvent;
 
@@ -545,8 +523,7 @@ class nsTreeBodyFrame final : public nsLeafBoxFrame,
   // A hashtable that maps from URLs to image request/listener pairs.  The URL
   // is provided by the view or by the ComputedStyle. The ComputedStyle
   // represents a resolved :-moz-tree-cell-image (or twisty) pseudo-element.
-  // It maps directly to an imgIRequest.
-  nsDataHashtable<nsStringHashKey, nsTreeImageCacheEntry> mImageCache;
+  nsTHashMap<nsURIHashKey, nsTreeImageCacheEntry> mImageCache;
 
   // A scratch array used when looking up cached ComputedStyles.
   mozilla::AtomArray mScratchArray;
@@ -569,15 +546,14 @@ class nsTreeBodyFrame final : public nsLeafBoxFrame,
   // Our desired horizontal width (the width for which we actually have tree
   // columns).
   nscoord mHorzWidth;
-  // The amount by which to adjust the width of the last cell.
-  // This depends on whether or not the columnpicker and scrollbars are present.
-  nscoord mAdjustWidth;
+
+  // Our last reflowed rect, used for invalidation, see ManageReflowCallback().
+  Maybe<nsRect> mLastReflowRect;
 
   // Cached heights and indent info.
   nsRect mInnerBox;  // 4-byte aligned
   int32_t mRowHeight;
   int32_t mIndentation;
-  nscoord mStringWidth;
 
   int32_t mUpdateBatchNest;
 
@@ -594,18 +570,12 @@ class nsTreeBodyFrame final : public nsLeafBoxFrame,
   bool mHasFixedRowCount;
 
   bool mVerticalOverflow;
-  bool mHorizontalOverflow;
 
   bool mReflowCallbackPosted;
 
-  // Set while we flush layout to take account of effects of
-  // overflow/underflow event handlers
+  // Set while we flush layout to take account of effects of overflow/underflow
+  // event handlers
   bool mCheckingOverflow;
-
-  // Hash table to keep track of which listeners we created and thus
-  // have pointers to us.
-  nsTHashtable<nsPtrHashKey<nsTreeImageListener> > mCreatedListeners;
-
 };  // class nsTreeBodyFrame
 
 #endif
