@@ -322,17 +322,6 @@ pub trait ToCssWithGuard {
     }
 }
 
-/// Parameters needed for deep clones.
-#[cfg(feature = "gecko")]
-pub struct DeepCloneParams {
-    /// The new sheet we're cloning rules into.
-    pub reference_sheet: *const crate::gecko_bindings::structs::StyleSheet,
-}
-
-/// Parameters needed for deep clones.
-#[cfg(feature = "servo")]
-pub struct DeepCloneParams;
-
 /// A trait to do a deep clone of a given CSS type. Gets a lock and a read
 /// guard, in order to be able to read and clone nested structures.
 pub trait DeepCloneWithLock: Sized {
@@ -341,7 +330,6 @@ pub trait DeepCloneWithLock: Sized {
         &self,
         lock: &SharedRwLock,
         guard: &SharedRwLockReadGuard,
-        params: &DeepCloneParams,
     ) -> Self;
 }
 
